@@ -1,4 +1,4 @@
-/* $Id: compile.c,v 1.11 2008/01/09 22:36:19 erg Exp $ $Revision: 1.11 $ */
+/* $Id: compile.c,v 1.22 2008/12/15 18:26:25 erg Exp $ $Revision: 1.22 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -32,16 +32,12 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <compile.h>
+#include "compile.h"
 #include <assert.h>
-#ifdef USE_CGRAPH
-#include <cgraph.h>
-#else
-#include <agraph.h>
-#endif
+#include "cgraph.h"
 #include <error.h>
 #include <actions.h>
-#include <sfstr.h>
+#include "sfstr.h"
 #include <string.h>
 #include <stdio.h>
 #include <math.h>
@@ -85,7 +81,7 @@ static char *nameOf(Expr_t * ex, Agobj_t * obj)
 	else
 	    sfputr(tmps, "--", -1);
 	sfputr(tmps, agnameof(AGHEAD(e)), -1);
-	if (*key) {
+	if (key && *key) {
 	    sfputc(tmps, '[');
 	    sfputr(tmps, key, -1);
 	    sfputc(tmps, ']');
@@ -306,6 +302,36 @@ setattr (Agobj_t *objp, char* name, char* val)
     return agxset(objp, gsym, val);
 }
 
+/* kindToStr:
+ */
+static char*
+kindToStr (int kind)
+{
+    char* s;
+
+    switch (kind) {
+    case AGRAPH :
+	s = "graph";
+	break;
+    case AGNODE :
+	s = "node";
+	break;
+    default :
+	s = "edge";
+	break;
+    }
+    return s;
+}
+
+/* kindOf:
+ * Return string rep of object's kind
+ */
+static char*
+kindOf (Agobj_t* objp)
+{
+    return (kindToStr (agobjkind (objp)));
+}
+
 /* lookup:
  * Apply symbol to get field value of objp
  * Assume objp != NULL
@@ -336,11 +362,7 @@ static int lookup(Expr_t * pgm, Agobj_t * objp, Exid_t * sym, Extype_t * v,
 	    break;
 	case M_indegree:
 	    if (AGTYPE(objp) == AGNODE)
-#ifdef USE_CGRAPH
-		v->integer = agdegree(state->curgraph, (Agnode_t *) objp, 1, 0);
-#else
-		v->integer = agdegree((Agnode_t *) objp, 1, 0);
-#endif
+		v->integer = agdegree(agroot(objp), (Agnode_t *) objp, 1, 0);
 	    else {
 		error(ERROR_FATAL, "indegree of non-node");
 		return -1;
@@ -348,11 +370,7 @@ static int lookup(Expr_t * pgm, Agobj_t * objp, Exid_t * sym, Extype_t * v,
 	    break;
 	case M_outdegree:
 	    if (AGTYPE(objp) == AGNODE)
-#ifdef USE_CGRAPH
-		v->integer = agdegree(state->curgraph, (Agnode_t *) objp, 0, 1);
-#else
-		v->integer = agdegree((Agnode_t *) objp, 0, 1);
-#endif
+		v->integer = agdegree(agroot(objp), (Agnode_t *) objp, 0, 1);
 	    else {
 		error(ERROR_FATAL, "outdegree of non-node");
 		return -1;
@@ -360,11 +378,7 @@ static int lookup(Expr_t * pgm, Agobj_t * objp, Exid_t * sym, Extype_t * v,
 	    break;
 	case M_degree:
 	    if (AGTYPE(objp) == AGNODE)
-#ifdef USE_CGRAPH
-		v->integer = agdegree(state->curgraph, (Agnode_t *) objp, 1, 1);
-#else
-		v->integer = agdegree((Agnode_t *) objp, 1, 1);
-#endif
+		v->integer = agdegree(agroot(objp), (Agnode_t *) objp, 1, 1);
 	    else {
 		error(ERROR_FATAL, "degree of non-node");
 		return -1;
@@ -374,7 +388,7 @@ static int lookup(Expr_t * pgm, Agobj_t * objp, Exid_t * sym, Extype_t * v,
 	    if (AGTYPE(objp) == AGRAPH)
 		v->integer = PTR2INT(agparent((Agraph_t *) objp));
 	    else {
-		error(ERROR_FATAL, "root of non-graph");
+		error(ERROR_FATAL, "parent of non-graph");
 		return -1;
 	    }
 	    break;
@@ -418,8 +432,14 @@ static int lookup(Expr_t * pgm, Agobj_t * objp, Exid_t * sym, Extype_t * v,
 	    return -1;
 	    break;
 	}
-    } else
-	v->string = agget(objp, sym->name);
+    } else {
+	Agsym_t *gsym = agattrsym(objp, sym->name);
+	if (!gsym) {
+	    gsym = agattr(agroot(agraphof(objp)), AGTYPE(objp), sym->name, "");
+	    error(ERROR_WARNING, "Using value of uninitialized attribute \"%s\" of %s \"%s\"", sym->name, kindOf (objp), nameOf(pgm, objp));
+	}
+	v->string = agxget(objp, gsym);
+    }
 
     return 0;
 }
@@ -440,7 +460,6 @@ static char *getArg(int n, Gpr_t * state)
 static int
 setDfltAttr (Agraph_t *gp, char* k, char* name, char* value)
 {
-    Agsym_t* sym;
     int kind;
 
     switch (*k) {
@@ -458,21 +477,17 @@ setDfltAttr (Agraph_t *gp, char* k, char* name, char* value)
 	return 1;
 	break;
     }
-    sym = agattr(gp, kind, name, 0);
-    if (!sym)
-	sym = agattr(gp, kind, name, "");
-
     agattr(gp, kind, name, value);
     return 0;
 }
 
-/* getDfltAttr:
+/* toKind:
+ * Map string to object kind
  */
-static char*
-getDfltAttr (Agraph_t *gp, char* k, char* name)
+static int
+toKind (char* k, char* fn)
 {
     int      kind;
-    Agsym_t* sym;
 
     switch (*k) {
     case 'G' :
@@ -485,13 +500,48 @@ getDfltAttr (Agraph_t *gp, char* k, char* name)
 	kind = AGNODE;
 	break;
     default :
-	error(ERROR_WARNING, "Unknown kind \"%s\" passed to getDflt()", k);
-	return 0;
+	error(ERROR_FATAL, "Unknown kind \"%s\" passed to %s()", k, fn);
 	break;
     }
-    sym = agattr (gp, kind, name, 0);
-    if (sym) return sym->defval; 
-    else return 0;
+    return kind;
+}
+
+/* nxtAttr:
+ */
+static char*
+nxtAttr (Agraph_t *gp, char* k, char* name)
+{
+    char* fn = (name ? "nxtAttr" : "fstAttr");
+    int kind = toKind (k, fn);
+    Agsym_t* sym;
+
+    if (name) {
+	sym = agattr (gp, kind, name, 0);
+	if (!sym) {
+	    error(ERROR_FATAL, "Third argument \"%s\" in nxtAttr() must be the name of an existing attribute", name); 
+	}
+	
+    }
+    else sym = NULL;
+    
+    sym = agnxtattr (gp, kind, sym);
+    if (sym) return sym->name; 
+    else return "";
+}
+
+/* getDfltAttr:
+ */
+static char*
+getDfltAttr (Agraph_t *gp, char* k, char* name)
+{
+    int kind = toKind (k, "getDflt");
+    Agsym_t* sym = agattr (gp, kind, name, 0);
+    if (!sym) {
+	sym = agattr(gp, kind, name, "");
+	error(ERROR_WARNING, "Uninitialized %s attribute \"%s\" in %s", 
+	    kindToStr (kind), name, "getDflt");
+    }
+    return sym->defval; 
 }
 
 /* getval:
@@ -511,6 +561,7 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
     Agnode_t *np;
     Agnode_t *hp;
     Agedge_t *ep;
+    char* name;
 
     assert(sym->lex != CONSTANT);
     if (elt == EX_CALL) {
@@ -592,11 +643,7 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	case F_nxtnode:
 	    np = INT2PTR(Agnode_t *, args[0].integer);
 	    if (np) {
-#ifdef USE_CGRAPH
-		np = agnxtnode(state->curgraph, np);
-#else
-		np = agnxtnode(np);
-#endif
+		np = agnxtnode(agroot(np), np);
 		v.integer = PTR2INT(np);
 	    } else {
 		error(ERROR_WARNING, "NULL node passed to nxtnode()");
@@ -604,11 +651,10 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	    }
 	    break;
 	case F_nxtnodesg:
-#ifdef USE_CGRAPH
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
 	    np = INT2PTR(Agnode_t *, args[1].integer);
 	    if (!gp)
-		gp = state->curgraph;
+		gp = agroot(np);
 	    if (np) {
 		np = agnxtnode(gp, np);
 		v.integer = PTR2INT(np);
@@ -616,9 +662,6 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		error(ERROR_WARNING, "NULL node passed to nxtnode_sg()");
 		v.integer = 0;
 	    }
-#else
-		error(ERROR_FATAL, "nxtnode_sg currently unsupported");
-#endif
 	    break;
 	case F_isnode:
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
@@ -630,68 +673,52 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	    }
 	    break;
 	case F_issubnode:
-#ifdef USE_CGRAPH
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
 	    np = INT2PTR(Agnode_t *, args[1].integer);
 	    if (!gp)
-		gp = state->curgraph;
+		gp = agroot(np);
 	    if (np) {
 		v.integer = PTR2INT(addNode(gp, np, 0));
 	    } else {
 		error(ERROR_WARNING, "NULL node passed to isSubnode()");
 		v.integer = 0;
 	    }
-#else
-		error(ERROR_FATAL, "isSubnode currently unsupported");
-#endif
 	    break;
 	case F_indegree:
-#ifdef USE_CGRAPH
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
 	    np = INT2PTR(Agnode_t *, args[1].integer);
 	    if (!gp)
-		gp = state->curgraph;
+		gp = agroot(np);
 	    if (np) {
 		v.integer = agdegree(gp, np, 1, 0);
 	    } else {
 		error(ERROR_WARNING, "NULL node passed to indegreeOf()");
 		v.integer = 0;
 	    }
-#else
-		error(ERROR_FATAL, "indegreeOf currently unsupported");
-#endif
 	    break;
 	case F_outdegree:
-#ifdef USE_CGRAPH
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
 	    np = INT2PTR(Agnode_t *, args[1].integer);
 	    if (!gp)
-		gp = state->curgraph;
+		gp = agroot(np);
 	    if (np) {
 		v.integer = agdegree(gp, np, 0, 1);
 	    } else {
 		error(ERROR_WARNING, "NULL node passed to outdegreeOf()");
 		v.integer = 0;
 	    }
-#else
-		error(ERROR_FATAL, "outdegreeOf currently unsupported");
-#endif
 	    break;
 	case F_degree:
-#ifdef USE_CGRAPH
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
 	    np = INT2PTR(Agnode_t *, args[1].integer);
 	    if (!gp)
-		gp = state->curgraph;
+		gp = agroot(np);
 	    if (np) {
 		v.integer = agdegree(gp, np, 1, 1);
 	    } else {
 		error(ERROR_WARNING, "NULL node passed to degreeOf()");
 		v.integer = 0;
 	    }
-#else
-		error(ERROR_FATAL, "degreeOf currently unsupported");
-#endif
 	    break;
 	case F_isin:
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
@@ -703,7 +730,7 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		error(ERROR_WARNING, "NULL object passed to isIn()");
 		v.integer = 0;
 	    } else
-		v.integer = isIn(gp, objp);
+		v.integer = agcontains (gp, objp);
 	    break;
 	case F_compof:
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
@@ -720,8 +747,7 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	case F_kindof:
 	    objp = INT2PTR(Agobj_t *, args[0].integer);
 	    if (!objp) {
-		error(ERROR_WARNING, "NULL object passed to kindOf()");
-		v.string = 0;
+		error(ERROR_FATAL, "NULL object passed to kindOf()");
 	    } else switch (AGTYPE(objp)) {
 		case AGRAPH :
 		    v.string = "G";
@@ -753,7 +779,6 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	    }
 	    break;
 	case F_edgesg:
-#ifdef USE_CGRAPH
 	    key = args[3].string;
 	    if (*key == '\0')
 		key = 0;
@@ -770,9 +795,6 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		ep = openEdge(gp, np, hp, key);
 		v.integer = PTR2INT(ep);
 	    }
-#else
-		error(ERROR_FATAL, "edge_sg currently unsupported");
-#endif
 	    break;
 	case F_addedge:
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
@@ -799,10 +821,9 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		error(ERROR_WARNING, "NULL head node passed to isEdge()");
 		v.integer = 0;
 	    } else
-		v.integer = PTR2INT(isEdge(state->curgraph, np, hp, key));
+		v.integer = PTR2INT(isEdge(agroot(np), np, hp, key));
 	    break;
 	case F_isedgesg:
-#ifdef USE_CGRAPH
 	    key = args[3].string;
 	    if (*key == '\0')
 		key = 0;
@@ -810,7 +831,7 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	    np = INT2PTR(Agnode_t *, args[1].integer);
 	    hp = INT2PTR(Agnode_t *, args[2].integer);
 	    if (!gp)
-		gp = state->curgraph;
+		gp = agroot(np);
 	    if (!np) {
 		error(ERROR_WARNING, "NULL tail node passed to isEdge_sg()");
 		v.integer = 0;
@@ -819,34 +840,23 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		v.integer = 0;
 	    } else
 		v.integer = PTR2INT(isEdge(gp, np, hp, key));
-#else
-		error(ERROR_FATAL, "isEdge_sg currently unsupported");
-#endif
 	    break;
 	case F_issubedge:
-#ifdef USE_CGRAPH
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
 	    ep = INT2PTR(Agedge_t *, args[1].integer);
 	    if (!gp)
-		gp = state->curgraph;
+		gp = agroot(ep);
 	    if (ep) {
 		v.integer = PTR2INT(addEdge(gp, ep, 0));
 	    } else {
 		error(ERROR_WARNING, "NULL edge passed to isSubedge()");
 		v.integer = 0;
 	    }
-#else
-		error(ERROR_FATAL, "isSubedge currently unsupported");
-#endif
 	    break;
 	case F_fstout:
 	    np = INT2PTR(Agnode_t *, args[0].integer);
 	    if (np) {
-#ifdef USE_CGRAPH
-		ep = agfstout(state->curgraph, np);
-#else
-		ep = agfstout(np);
-#endif
+		ep = agfstout(agroot(np), np);
 		v.integer = PTR2INT(ep);
 	    } else {
 		error(ERROR_WARNING, "NULL node passed to fstout()");
@@ -854,11 +864,10 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	    }
 	    break;
 	case F_fstoutsg:
-#ifdef USE_CGRAPH
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
 	    np = INT2PTR(Agnode_t *, args[1].integer);
 	    if (!gp)
-		gp = state->curgraph;
+		gp = agroot(np);
 	    if (np) {
 		ep = agfstout(gp, np);
 		v.integer = PTR2INT(ep);
@@ -866,18 +875,11 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		error(ERROR_WARNING, "NULL node passed to fstout_sg()");
 		v.integer = 0;
 	    }
-#else
-		error(ERROR_FATAL, "fstout_sg currently unsupported");
-#endif
 	    break;
 	case F_nxtout:
 	    ep = INT2PTR(Agedge_t *, args[0].integer);
 	    if (ep) {
-#ifdef USE_CGRAPH
-		ep = agnxtout(state->curgraph, ep);
-#else
-		ep = agnxtout(ep);
-#endif
+		ep = agnxtout(agroot(ep), ep);
 		v.integer = PTR2INT(ep);
 	    } else {
 		error(ERROR_WARNING, "NULL edge passed to nxtout()");
@@ -885,11 +887,10 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	    }
 	    break;
 	case F_nxtoutsg:
-#ifdef USE_CGRAPH
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
 	    ep = INT2PTR(Agedge_t *, args[1].integer);
 	    if (!gp)
-		gp = state->curgraph;
+		gp = agroot(ep);
 	    if (ep) {
 		ep = agnxtout(gp, ep);
 		v.integer = PTR2INT(ep);
@@ -897,18 +898,11 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		error(ERROR_WARNING, "NULL edge passed to nxtout_sg()");
 		v.integer = 0;
 	    }
-#else
-		error(ERROR_FATAL, "nxtout_sg currently unsupported");
-#endif
 	    break;
 	case F_fstin:
 	    np = INT2PTR(Agnode_t *, args[0].integer);
 	    if (np) {
-#ifdef USE_CGRAPH
-		ep = agfstin(state->curgraph, np);
-#else
-		ep = agfstin(np);
-#endif
+		ep = agfstin(agroot(np), np);
 		v.integer = PTR2INT(ep);
 	    } else {
 		error(ERROR_WARNING, "NULL node passed to fstin()");
@@ -916,11 +910,10 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	    }
 	    break;
 	case F_fstinsg:
-#ifdef USE_CGRAPH
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
 	    np = INT2PTR(Agnode_t *, args[1].integer);
 	    if (!gp)
-		gp = state->curgraph;
+		gp = agroot(np);
 	    if (np) {
 		ep = agfstin(gp, np);
 		v.integer = PTR2INT(ep);
@@ -928,18 +921,11 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		error(ERROR_WARNING, "NULL node passed to fstin_sg()");
 		v.integer = 0;
 	    }
-#else
-		error(ERROR_FATAL, "fstin_sg currently unsupported");
-#endif
 	    break;
 	case F_nxtin:
 	    ep = INT2PTR(Agedge_t *, args[0].integer);
 	    if (ep) {
-#ifdef USE_CGRAPH
-		ep = agnxtin(state->curgraph, ep);
-#else
-		ep = agnxtin(ep);
-#endif
+		ep = agnxtin(agroot(ep), ep);
 		v.integer = PTR2INT(ep);
 	    } else {
 		error(ERROR_WARNING, "NULL edge passed to nxtin()");
@@ -947,11 +933,10 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	    }
 	    break;
 	case F_nxtinsg:
-#ifdef USE_CGRAPH
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
 	    ep = INT2PTR(Agedge_t *, args[1].integer);
 	    if (!gp)
-		gp = state->curgraph;
+		gp = agroot(ep);
 	    if (ep) {
 		ep = agnxtin(gp, ep);
 		v.integer = PTR2INT(ep);
@@ -959,18 +944,11 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		error(ERROR_WARNING, "NULL edge passed to nxtin_sg()");
 		v.integer = 0;
 	    }
-#else
-		error(ERROR_FATAL, "nxtin_sg currently unsupported");
-#endif
 	    break;
 	case F_fstedge:
 	    np = INT2PTR(Agnode_t *, args[0].integer);
 	    if (np) {
-#ifdef USE_CGRAPH
-		ep = agfstedge(state->curgraph, np);
-#else
-		ep = agfstedge(np);
-#endif
+		ep = agfstedge(agroot(np), np);
 		v.integer = PTR2INT(ep);
 	    } else {
 		error(ERROR_WARNING, "NULL node passed to fstedge()");
@@ -978,11 +956,10 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	    }
 	    break;
 	case F_fstedgesg:
-#ifdef USE_CGRAPH
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
 	    np = INT2PTR(Agnode_t *, args[1].integer);
 	    if (!gp)
-		gp = state->curgraph;
+		gp = agroot(np);
 	    if (np) {
 		ep = agfstedge(gp, np);
 		v.integer = PTR2INT(ep);
@@ -990,9 +967,6 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		error(ERROR_WARNING, "NULL node passed to fstedge_sg()");
 		v.integer = 0;
 	    }
-#else
-		error(ERROR_FATAL, "fstedge_sg currently unsupported");
-#endif
 	    break;
 	case F_nxtedge:
 	    ep = INT2PTR(Agedge_t *, args[0].integer);
@@ -1004,21 +978,16 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		error(ERROR_WARNING, "NULL node passed to nxtedge()");
 		v.integer = 0;
 	    } else {
-#ifdef USE_CGRAPH
-		ep = agnxtedge(state->curgraph, ep, np);
-#else
-		ep = agnxtedge(ep, np);
-#endif
+		ep = agnxtedge(agroot(np), ep, np);
 		v.integer = PTR2INT(ep);
 	    }
 	    break;
 	case F_nxtedgesg:
-#ifdef USE_CGRAPH
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
 	    ep = INT2PTR(Agedge_t *, args[1].integer);
 	    np = INT2PTR(Agnode_t *, args[2].integer);
 	    if (!gp)
-		gp = state->curgraph;
+		gp = agroot(np);
 	    if (!ep) {
 		error(ERROR_WARNING, "NULL edge passed to nxtedge_sg()");
 		v.integer = 0;
@@ -1029,9 +998,6 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		ep = agnxtedge(gp, ep, np);
 		v.integer = PTR2INT(ep);
 	    }
-#else
-		error(ERROR_FATAL, "nxtedge_sg currently unsupported");
-#endif
 	    break;
 	case F_copy:
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
@@ -1191,18 +1157,26 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	case F_log:
 	    v.floating = log(args[0].floating);
 	    break;
+	case F_hasattr:
 	case F_get:
 	    objp = INT2PTR(Agobj_t *, args[0].integer);
+	    name = args[1].string;
 	    if (!objp) {
-		error(ERROR_WARNING, "NULL object passed to aget()");
-		v.string = 0;
-	    } else {
-		char* name = args[1].string;
-		if (name) v.string = agget(objp, name);
+		error(ERROR_FATAL, "NULL object passed to aget()/hasAttr()");
+	    } else if (!name) {
+		error(ERROR_FATAL, "NULL name passed to aget()/hasAttr()");
+	    }
+	    else {
+	        Agsym_t *gsym = agattrsym(objp, name);
+		if (sym->index == F_hasattr)
+		    v.integer = (gsym != NULL);
 		else {
-		    error(ERROR_WARNING, "NULL name passed to aget()");
-		    v.string = 0;
-		}
+		    if (!gsym) {
+	    		gsym = agattr(agroot(agraphof(objp)), AGTYPE(objp), name, "");
+	    		error(ERROR_WARNING, "Using value of uninitialized attribute \"%s\" of %s \"%s\" in aget()", name, kindOf (objp), nameOf(pgm, objp));
+		    }
+		    v.string = agxget(objp, gsym);
+        	}
             }
 	    break;
 	case F_set:
@@ -1252,25 +1226,44 @@ getval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 		v.integer = 0;
 	    }
 	    break;
+	case F_fstattr:
+	    gp = INT2PTR(Agraph_t *, args[0].integer);
+	    if (gp) {
+		char* kind = args[1].string;
+		if (!kind) {
+		    error(ERROR_FATAL,"NULL kind passed to fstAttr()");
+		}
+		else {
+		    v.string = nxtAttr (gp, kind, NULL);
+        	}
+	    } else {
+		error(ERROR_FATAL, "NULL graph passed to fstAttr()");
+	    }
+	    break;
+	case F_nxtattr:
+	case F_isattr:
 	case F_dget:
 	    gp = INT2PTR(Agraph_t *, args[0].integer);
 	    if (gp) {
 		char* kind = args[1].string;
 		char* name = args[2].string;
 		if (!name) {
-		    error(ERROR_WARNING, "NULL name passed to getDflt()");
-		    v.string = 0;
+		    error(ERROR_FATAL,"NULL name passed to %s", sym->name);
 		}
 		else if (!kind) {
-		    error(ERROR_WARNING, "NULL kind passed to getDflt()");
-		    v.string = 0;
+		    error(ERROR_FATAL,"NULL kind passed to %s", sym->name);
 		}
+		else if (sym->index == F_isattr) {
+		    v.integer = (agattr(gp, toKind (kind, sym->name), name, 0) != NULL);
+        	}
+		else if (sym->index == F_nxtattr) {
+		    v.string = nxtAttr (gp, kind, name);
+        	}
 		else {
 		    v.string = getDfltAttr(gp, kind, name);
         	}
 	    } else {
-		error(ERROR_WARNING, "NULL graph passed to node()");
-		v.string = 0;
+		error(ERROR_FATAL, "NULL graph passed to %s", sym->name);
 	    }
 	    break;
 	case F_canon:
@@ -1411,7 +1404,7 @@ setval(Expr_t * pgm, Exnode_t * x, Exid_t * sym, Exref_t * ref,
 	    break;
 	case V_travroot:
 	    np = INT2PTR(Agnode_t *, v.integer);
-	    if (!np || (agroot(agraphof(np)) == state->curgraph))
+	    if (!np || (agroot(np) == state->curgraph))
 		state->tvroot = np;
 	    else {
 		error(1, "cannot set $tvroot, node %s not in $G : ignored",
@@ -1570,14 +1563,20 @@ refval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
     Extype_t v;
     if (sym->lex == CONSTANT) {
 	switch (sym->index) {
-	case C_dfs:
-	    v.integer = TV_dfs;
+	case C_flat:
+	    v.integer = TV_flat;
+	    break;
+	case C_ne:
+	    v.integer = TV_ne;
+	    break;
+	case C_en:
+	    v.integer = TV_en;
 	    break;
 	case C_bfs:
 	    v.integer = TV_bfs;
 	    break;
-	case C_flat:
-	    v.integer = TV_flat;
+	case C_dfs:
+	    v.integer = TV_dfs;
 	    break;
 	case C_fwd:
 	    v.integer = TV_fwd;
@@ -1585,11 +1584,23 @@ refval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 	case C_rev:
 	    v.integer = TV_rev;
 	    break;
-	case C_ne:
-	    v.integer = TV_ne;
+	case C_postdfs:
+	    v.integer = TV_postdfs;
 	    break;
-	case C_en:
-	    v.integer = TV_en;
+	case C_postfwd:
+	    v.integer = TV_postfwd;
+	    break;
+	case C_postrev:
+	    v.integer = TV_postrev;
+	    break;
+	case C_prepostdfs:
+	    v.integer = TV_prepostdfs;
+	    break;
+	case C_prepostfwd:
+	    v.integer = TV_prepostfwd;
+	    break;
+	case C_prepostrev:
+	    v.integer = TV_prepostrev;
 	    break;
 	case C_null:
 	    v.integer = 0;
@@ -1611,9 +1622,9 @@ refval(Expr_t * pgm, Exnode_t * node, Exid_t * sym, Exref_t * ref,
 static void cvtError(Exid_t * xref, char *msg)
 {
     if (xref)
-	error(1, "%s: %s", xref->name, msg);
+	error(ERROR_FATAL, "%s: %s", xref->name, msg);
     else
-	error(1, "%s", msg);
+	error(ERROR_FATAL, "%s", msg);
 }
 #endif
 
@@ -1740,8 +1751,106 @@ binary(Expr_t * pg, Exnode_t * l, Exnode_t * ex, Exnode_t * r, int arg,
     return ret;
 }
 
+/* strToTvtype:
+ */
+static int
+strToTvtype (char* s)
+{
+    int rt = 0;
+    char* sfx;
+
+    if (!strncmp(s, "TV_", 3)) {
+	sfx = s + 3;
+	if (!strcmp(sfx, "flat")) {
+	    rt = TV_flat;
+	} else if (!strcmp(sfx, "ne")) {
+	    rt = TV_ne;
+	} else if (!strcmp(sfx, "en")) {
+	    rt = TV_en;
+	} else if (!strcmp(sfx, "bfs")) {
+	    rt = TV_bfs;
+	} else if (!strcmp(sfx, "dfs")) {
+	    rt = TV_dfs;
+	} else if (!strcmp(sfx, "fwd")) {
+	    rt = TV_fwd;
+	} else if (!strcmp(sfx, "rev")) {
+	    rt = TV_rev;
+	} else if (!strcmp(sfx, "postdfs")) {
+	    rt = TV_postdfs;
+	} else if (!strcmp(sfx, "postfwd")) {
+	    rt = TV_postfwd;
+	} else if (!strcmp(sfx, "postrev")) {
+	    rt = TV_postrev;
+	} else if (!strcmp(sfx, "prepostdfs")) {
+	    rt = TV_prepostdfs;
+	} else if (!strcmp(sfx, "prepostfwd")) {
+	    rt = TV_prepostfwd;
+	} else if (!strcmp(sfx, "prepostrev")) {
+	    rt = TV_prepostrev;
+	} else
+	    error(ERROR_FATAL, "illegal string \"%s\" for type tvtype_t", s);
+    } else
+	error(ERROR_FATAL, "illegal string \"%s\" for type tvtype_t", s);
+    return rt;
+}
+
+/* tvtypeToStr:
+ */
+static char*
+tvtypeToStr (int v)
+{
+    char* s = 0;
+
+    switch (v) {
+    case TV_flat:
+	s = "TV_flat";
+	break;
+    case TV_ne:
+	s = "TV_ne";
+	break;
+    case TV_en:
+	s = "TV_en";
+	break;
+    case TV_bfs:
+	s = "TV_bfs";
+	break;
+    case TV_dfs:
+	s = "TV_dfs";
+	break;
+    case TV_fwd:
+	s = "TV_fwd";
+	break;
+    case TV_rev:
+	s = "TV_rev";
+	break;
+    case TV_postdfs:
+	s = "TV_postdfs";
+	break;
+    case TV_postfwd:
+	s = "TV_postfwd";
+	break;
+    case TV_postrev:
+	s = "TV_postrev";
+	break;
+    case TV_prepostdfs:
+	s = "TV_prepostdfs";
+	break;
+    case TV_prepostfwd:
+	s = "TV_prepostfwd";
+	break;
+    case TV_prepostrev:
+	s = "TV_prepostrev";
+	break;
+    default:
+	exerror("Unexpected value %d for type tvtype_t",
+	    v);
+	break;
+    }
+    return s;
+}
+
 /* stringOf:
- * Convert value x of type string.
+ * Convert value x to type string.
  * Assume x does not have a built-in type
  * Return -1 if conversion cannot be done, 0 otherwise.
  * If arg is > 0, conversion unnecessary; just report possibility.
@@ -1754,27 +1863,8 @@ int stringOf(Expr_t * prog, register Exnode_t * x, int arg)
 	return 0;
 
     if (x->type == T_tvtyp) {
-	switch (x->data.constant.value.integer) {
-	case TV_flat:
-	    x->data.constant.value.string = "TV_flat";
-	    break;
-	case TV_dfs:
-	    x->data.constant.value.string = "TV_dfs";
-	    break;
-	case TV_bfs:
-	    x->data.constant.value.string = "TV_bfs";
-	    break;
-	case TV_fwd:
-	    x->data.constant.value.string = "TV_fwd";
-	    break;
-	case TV_rev:
-	    x->data.constant.value.string = "TV_rev";
-	    break;
-	default:
-	    exerror("Unexpected value %d for type tvtype_t",
-		    x->data.constant.value.integer);
-	    break;
-	}
+	x->data.constant.value.string = 
+	    tvtypeToStr (x->data.constant.value.integer);
     } else {
 	objp = INT2PTR(Agobj_t *, x->data.constant.value.integer);
 	if (!objp)
@@ -1842,28 +1932,10 @@ convert(Expr_t * prog, register Exnode_t * x, int type,
     } else if (type == STRING) {
 	if (x->type == T_tvtyp) {
 	    ret = 0;
-	    if (!arg)
-		switch (x->data.constant.value.integer) {
-		case TV_flat:
-		    x->data.constant.value.string = "TV_flat";
-		    break;
-		case TV_dfs:
-		    x->data.constant.value.string = "TV_dfs";
-		    break;
-		case TV_bfs:
-		    x->data.constant.value.string = "TV_bfs";
-		    break;
-		case TV_fwd:
-		    x->data.constant.value.string = "TV_fwd";
-		    break;
-		case TV_rev:
-		    x->data.constant.value.string = "TV_rev";
-		    break;
-		default:
-		    error(3, "Unexpected value %d for type tvtype_t",
-			  x->data.constant.value.integer);
-		    break;
-		}
+	    if (!arg) {
+		x->data.constant.value.string =
+		    tvtypeToStr (x->data.constant.value.integer);
+	    }
 	}
 #ifdef OLD
 	else {
@@ -1878,19 +1950,11 @@ convert(Expr_t * prog, register Exnode_t * x, int type,
     } else if ((type == T_tvtyp) && (x->type == INTEGER)) {
 	if (arg)
 	    ret = 0;
+	else if (validTVT(x->data.constant.value.integer))
+	    ret = 0;
 	else
-	    switch (x->data.constant.value.integer) {
-	    case TV_flat:
-	    case TV_dfs:
-	    case TV_bfs:
-	    case TV_fwd:
-	    case TV_rev:
-		break;
-	    default:
-		error(1, "Integer value %d not legal for type tvtype_t",
-		      x->data.constant.value.integer);
-		break;
-	    }
+	    error(ERROR_FATAL, "Integer value %d not legal for type tvtype_t",
+		x->data.constant.value.integer);
     }
     /* in case libexpr hands us the trivial case */
     else if (x->type == type) {
@@ -1901,28 +1965,9 @@ convert(Expr_t * prog, register Exnode_t * x, int type,
 	    if (arg)
 		ret = 0;
 	    else {
+		ret = 0;
 		s = x->data.constant.value.string;
-		if (!strncmp(s, "TV_", 3)) {
-		    if (!strcmp(s + 3, "flat")) {
-			x->data.constant.value.integer = TV_flat;
-			ret = 0;
-		    } else if (!strcmp(s + 3, "dfs")) {
-			x->data.constant.value.integer = TV_dfs;
-			ret = 0;
-		    } else if (!strcmp(s + 3, "bfs")) {
-			x->data.constant.value.integer = TV_bfs;
-			ret = 0;
-		    } else if (!strcmp(s + 3, "fwd")) {
-			x->data.constant.value.integer = TV_fwd;
-			ret = 0;
-		    } else if (!strcmp(s + 3, "rev")) {
-			x->data.constant.value.integer = TV_rev;
-			ret = 0;
-		    } else
-			error(ERROR_FATAL,
-			      "illegal string \"%s\" for type tvtype_t",
-			      s);
-		}
+		x->data.constant.value.integer = strToTvtype(s);
 	    }
 	}
     }
@@ -2291,18 +2336,12 @@ Agedge_t *openEdge(Agraph_t* g, Agnode_t * t, Agnode_t * h, char *key)
     root = sameG(t, h, "openEdge", "tail and head node");
     if (!root)
 	return 0;
-#ifdef USE_CGRAPH
     if (g && (root != agroot(g)))
 	return 0;
     else
 	g = root;
 
     ep = agedge(g, t, h, key, 1);
-#else
-    t = (Agnode_t *) agrebind(root, OBJ(t));
-    h = (Agnode_t *) agrebind(root, OBJ(h));
-    ep = agedge(t, h, key, 1);
-#endif
     if (ep && !aggetrec(ep, UDATA, 0))
 	agbindrec(ep, UDATA, sizeof(edata), 0);
     return ep;
