@@ -1,4 +1,4 @@
-/* $Id: gprstate.h,v 1.4 2008/01/09 22:36:19 erg Exp $ $Revision: 1.4 $ */
+/* $Id: gprstate.h,v 1.6 2008/12/15 18:26:25 erg Exp $ $Revision: 1.6 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -21,17 +21,17 @@ extern "C" {
 #ifndef GPRSTATE_H
 #define GPRSTATE_H
 
-#include <sfio.h>
-#ifdef USE_CGRAPH
-#include <cgraph.h>
-#else
-#include <agraph.h>
-#endif
-#include <ast.h>
-#include <vmalloc.h>
+#include "sfio.h"
+#include "cgraph.h"
+#include "ast.h"
+#include "vmalloc.h"
 
-    typedef enum { TV_flat, TV_bfs, TV_dfs, TV_fwd, TV_rev, TV_ne,
-	    TV_en } trav_type;
+    typedef enum { TV_flat, TV_ne, TV_en, 
+                   TV_bfs, 
+                   TV_dfs, TV_fwd, TV_rev,
+                   TV_postdfs, TV_postfwd, TV_postrev,
+                   TV_prepostdfs, TV_prepostfwd, TV_prepostrev,
+    } trav_type;
 
     typedef struct {
 	Agraph_t *curgraph;
