@@ -1,4 +1,4 @@
-/* $Id: queue.h,v 1.2 2008/01/09 22:36:19 erg Exp $ $Revision: 1.2 $ */
+/* $Id: queue.h,v 1.3 2008/08/07 17:17:46 ellson Exp $ $Revision: 1.3 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -21,11 +21,7 @@ extern "C" {
 #ifndef QUEUE_H
 #define QUEUE_H
 
-#ifdef USE_CGRAPH
-#include <cgraph.h>
-#else
-#include <agraph.h>
-#endif
+#include "cgraph.h"
 
     typedef Dt_t queue;
 
