@@ -1,4 +1,4 @@
-/* $Id: draw.h,v 1.6 2008/04/25 17:14:52 arif Exp $ $Revision: 1.6 $ */
+/* $Id: draw.h,v 1.12 2008/11/04 16:23:00 erg Exp $ $Revision: 1.12 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -26,6 +26,7 @@
 typedef struct {
     xdot_op op;
     void *obj;
+    void *font;		//pointer to font in view->fontset
     int size;
 } sdot_op;
 
@@ -35,6 +36,7 @@ typedef struct {
  */
 extern drawfunc_t OpFns[];
 extern void drawGraph(Agraph_t * g);
+void scanGraph(Agraph_t * g);
 void draw_selection_box(ViewInfo * view);
 void draw_magnifier(ViewInfo * view);
 void draw_fisheye_magnifier(ViewInfo * view);
@@ -43,5 +45,10 @@ extern void drawCircle(float x, float y, float radius, float zdepth);
 extern RGBColor GetRGBColor(char *color);
 extern void drawBorders(ViewInfo * view);
 void drawEllipse(float xradius, float yradius,int angle1,int angle2);
-int draw_node_hintbox(GLfloat x,GLfloat y,GLfloat fs,GLfloat pad,GLfloat z,GLfloat kts,GLfloat z_offset);
+int draw_node_hintbox(GLfloat x,GLfloat y,GLfloat z,GLfloat fs,char* text);
+
+
+void draw_sphere(float x,float y,float z,float r);
+
 #endif
+
