@@ -1,4 +1,4 @@
-/* $Id: menucallbacks.c,v 1.10 2008/04/25 18:37:53 erg Exp $ $Revision: 1.10 $ */
+/* $Id: menucallbacks.c,v 1.19 2008/11/11 15:25:53 arif Exp $ $Revision: 1.19 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -16,6 +16,7 @@
 
 #include "menucallbacks.h"
 #include "viewport.h"
+#include "topview.h"
 #include "tvnodes.h"
 #include "selection.h"
 #include "topviewsettings.h"
@@ -33,6 +34,7 @@ void mOpenSlot(GtkWidget * widget, gpointer user_data)
 
 
     filter = gtk_file_filter_new();
+    gtk_file_filter_add_pattern(filter, "*.gv");
     gtk_file_filter_add_pattern(filter, "*.dot");
     dialog = gtk_file_chooser_dialog_new("Open File",
 					 NULL,
@@ -156,6 +158,17 @@ void mShowHostSelectionSlot(GtkWidget * widget, gpointer user_data)
 
 }
 
+void mMenuPan(GtkWidget * widget, gpointer user_data)
+{
+	view->mouse.mouse_mode = MM_PAN;
+
+}
+void mMenuZoom(GtkWidget * widget, gpointer user_data)
+{
+	view->mouse.mouse_mode = MM_ZOOM;
+
+}
+
 
 //Graph
 void mNodeListSlot(GtkWidget * widget, gpointer user_data)
@@ -195,66 +208,74 @@ void mGraphPropertiesSlot(GtkWidget * widget, gpointer user_data)
 	gtk_widget_hide(glade_xml_get_widget(xml, "dlgOpenGraph"));
     }
 }
-void mClusterPropertiesSlot(GtkWidget * widget, gpointer user_data)
+
+void mNodeFindSlot(GtkWidget * widget, gpointer user_data)
 {
-    if (GD_selectedGraphsCount(view->g[view->activeGraph]) > 0) {
+	if((view->activeGraph !=-1) && (GD_TopView(view->g[view->activeGraph])))
+	{
+		gtk_dialog_set_response_sensitive((GtkDialog *)
+						  glade_xml_get_widget(xml,
+								       "findDialog"),
+						  1, 1);
+		gtk_dialog_set_response_sensitive((GtkDialog *)
+						  glade_xml_get_widget(xml,
+								       "findDialog"),
+						  2, 1);
+
+	
+		respond =
+			gtk_dialog_run((GtkDialog *)
+				   glade_xml_get_widget(xml, "findDialog"));
+
+		gtk_widget_hide(glade_xml_get_widget(xml, "findDialog"));
+		if (respond ==1)
+			select_with_regex(
+				gtk_entry_get_text((GtkEntry*) glade_xml_get_widget(xml,"findText"))
+				);
+	}
+
+}
+
+
+static void mPropertiesSlot(int count, gve_element element)
+{
+    if (count > 0) {
 	gtk_widget_hide(glade_xml_get_widget(xml, "frmObject"));
 	gtk_widget_show(glade_xml_get_widget(xml, "frmObject"));
-	load_object_properties(1, view->g[view->activeGraph]);
+	load_object_properties(element, view->g[view->activeGraph]);
     } else {
+	char buf[BUFSIZ];
+	sprintf (buf, "You need to select some %ss first!",element2s(element)); 
 	Dlg = (GtkMessageDialog *) gtk_message_dialog_new(NULL,
 							  GTK_DIALOG_MODAL,
 							  GTK_MESSAGE_QUESTION,
 							  GTK_BUTTONS_OK,
-							  "You need to select some clusters first!");
+							  buf);
 	respond = gtk_dialog_run((GtkDialog *) Dlg);
 	gtk_widget_hide((GtkWidget *) Dlg);
 
     }
 }
+void mClusterPropertiesSlot(GtkWidget * widget, gpointer user_data)
+{
+    mPropertiesSlot (GD_selectedGraphsCount(view->g[view->activeGraph]),GVE_CLUSTER);
+}
 void mNodePropertiesSlot(GtkWidget * widget, gpointer user_data)
 {
-    if (GD_selectedNodesCount(view->g[view->activeGraph]) > 0) {
-	gtk_widget_hide(glade_xml_get_widget(xml, "frmObject"));
-	gtk_widget_show(glade_xml_get_widget(xml, "frmObject"));
-	load_object_properties(2, view->g[view->activeGraph]);
-    } else {
-	Dlg = (GtkMessageDialog *) gtk_message_dialog_new(NULL,
-							  GTK_DIALOG_MODAL,
-							  GTK_MESSAGE_QUESTION,
-							  GTK_BUTTONS_OK,
-							  "You need to select some nodes first!");
-	respond = gtk_dialog_run((GtkDialog *) Dlg);
-	gtk_widget_hide((GtkWidget *) Dlg);
-
-    }
+    mPropertiesSlot (GD_selectedNodesCount(view->g[view->activeGraph]), GVE_NODE);
 }
 void mEdgePropertiesSlot(GtkWidget * widget, gpointer user_data)
 {
-    if (GD_selectedEdgesCount(view->g[view->activeGraph]) > 0) {
-	gtk_widget_hide(glade_xml_get_widget(xml, "frmObject"));
-	gtk_widget_show(glade_xml_get_widget(xml, "frmObject"));
-	load_object_properties(3, view->g[view->activeGraph]);
-    } else {
-	Dlg = (GtkMessageDialog *) gtk_message_dialog_new(NULL,
-							  GTK_DIALOG_MODAL,
-							  GTK_MESSAGE_QUESTION,
-							  GTK_BUTTONS_OK,
-							  "You need to select some Edges first!");
-	respond = gtk_dialog_run((GtkDialog *) Dlg);
-	gtk_widget_hide((GtkWidget *) Dlg);
-    }
-
+    mPropertiesSlot (GD_selectedEdgesCount(view->g[view->activeGraph]), GVE_EDGE);
 }
 
-
 void mShowCodeSlot(GtkWidget * widget, gpointer user_data)
 {
 }
-void mDotSlot(GtkWidget * widget, gpointer user_data)
+static void mSlot (GtkWidget * widget, gpointer user_data, gvk_layout layout, int doCursor)
 {
-    GdkCursor *cursor;
-    GdkWindow *w;
+    /* GdkCursor *cursor; */
+    /* GdkWindow *w; */
     Dlg = (GtkMessageDialog *) gtk_message_dialog_new(NULL,
 						      GTK_DIALOG_MODAL,
 						      GTK_MESSAGE_QUESTION,
@@ -263,73 +284,45 @@ void mDotSlot(GtkWidget * widget, gpointer user_data)
 
     respond = gtk_dialog_run((GtkDialog *) Dlg);
     if (respond == GTK_RESPONSE_YES)
-	do_graph_layout(view->g[view->activeGraph], GVK_DOT, 0);
+	do_graph_layout(view->g[view->activeGraph], layout, 0);
     gtk_object_destroy((GtkObject *) Dlg);
+	return;
 
-    cursor = gdk_cursor_new(GDK_HAND2);
-    w = (GdkWindow *) glade_xml_get_widget(xml, "frmMain");
-    gdk_window_set_cursor((GdkWindow *) view->drawing_area->window,
-			  cursor);
-    gdk_cursor_destroy(cursor);
+
+}
+
+void mDotSlot(GtkWidget * widget, gpointer user_data)
+{
+    mSlot (widget, user_data, GVK_DOT, 1);
 }
 
 void mNeatoSlot(GtkWidget * widget, gpointer user_data)
 {
-    Dlg = (GtkMessageDialog *) gtk_message_dialog_new(NULL,
-						      GTK_DIALOG_MODAL,
-						      GTK_MESSAGE_QUESTION,
-						      GTK_BUTTONS_YES_NO,
-						      "This will change the graph layout\n all your position changes will be lost\n Are you sure?");
-    respond = gtk_dialog_run((GtkDialog *) Dlg);
-    if (respond == GTK_RESPONSE_YES)
-	do_graph_layout(view->g[view->activeGraph], GVK_NEATO, 0);
-    gtk_object_destroy((GtkObject *) Dlg);
+    mSlot (widget, user_data, GVK_NEATO, 0);
 }
 
 void mTwopiSlot(GtkWidget * widget, gpointer user_data)
 {
-    Dlg = (GtkMessageDialog *) gtk_message_dialog_new(NULL,
-						      GTK_DIALOG_MODAL,
-						      GTK_MESSAGE_QUESTION,
-						      GTK_BUTTONS_YES_NO,
-						      "This will change the graph layout\n all your position changes will be lost\n Are you sure?");
-
-    respond = gtk_dialog_run((GtkDialog *) Dlg);
-    if (respond == GTK_RESPONSE_YES)
-	do_graph_layout(view->g[view->activeGraph], GVK_TWOPI, 0);
-    gtk_object_destroy((GtkObject *) Dlg);
-
+    mSlot (widget, user_data, GVK_TWOPI, 0);
 }
 
 void mCircoSlot(GtkWidget * widget, gpointer user_data)
 {
-    Dlg = (GtkMessageDialog *) gtk_message_dialog_new(NULL,
-						      GTK_DIALOG_MODAL,
-						      GTK_MESSAGE_QUESTION,
-						      GTK_BUTTONS_YES_NO,
-						      "This will change the graph layout\n all your position changes will be lost\n Are you sure?");
-
-    respond = gtk_dialog_run((GtkDialog *) Dlg);
-    if (respond == GTK_RESPONSE_YES)
-	do_graph_layout(view->g[view->activeGraph], GVK_CIRCO, 0);
-    gtk_object_destroy((GtkObject *) Dlg);
+    mSlot (widget, user_data, GVK_CIRCO, 0);
 }
 
 void mFdpSlot(GtkWidget * widget, gpointer user_data)
 {
-
-    Dlg = (GtkMessageDialog *) gtk_message_dialog_new(NULL,
-						      GTK_DIALOG_MODAL,
-						      GTK_MESSAGE_QUESTION,
-						      GTK_BUTTONS_YES_NO,
-						      "This will change the graph layout\n all your position changes will be lost\n Are you sure?");
-
-    respond = gtk_dialog_run((GtkDialog *) Dlg);
-    if (respond == GTK_RESPONSE_YES)
-	do_graph_layout(view->g[view->activeGraph], GVK_FDP, 0);
-    gtk_object_destroy((GtkObject *) Dlg);
+    mSlot (widget, user_data, GVK_FDP, 0);
 }
 
+void mSfdpSlot(GtkWidget * widget, gpointer user_data)
+{
+    mSlot (widget, user_data, GVK_SFDP, 0);
+}
+
+
+
 //select
 void mSelectAllSlot(GtkWidget * widget, gpointer user_data)
 {
@@ -397,3 +390,18 @@ void mAbout(GtkWidget * widget, gpointer user_data)
 void mHelp(GtkWidget * widget, gpointer user_data)
 {
 }
+
+
+
+void change_cursor(GdkCursorType C)
+{
+
+	GdkCursor *cursor;
+	GdkWindow * w;
+	cursor = gdk_cursor_new(C);
+    w = (GdkWindow *) glade_xml_get_widget(xml, "frmMain");
+    gdk_window_set_cursor((GdkWindow *) view->drawing_area->window,
+			  cursor);
+    gdk_cursor_destroy(cursor);
+	return;
+}
