@@ -1,4 +1,4 @@
-/* $Id: menucallbacks.h,v 1.3 2008/03/19 21:43:59 erg Exp $ $Revision: 1.3 $ */
+/* $Id: menucallbacks.h,v 1.7 2008/11/11 15:25:53 arif Exp $ $Revision: 1.7 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -34,10 +34,15 @@ _BB void mCopySlot(GtkWidget * widget, gpointer user_data);
 _BB void mPasteSlot(GtkWidget * widget, gpointer user_data);
 _BB void mDeleteSlot(GtkWidget * widget, gpointer user_data);
 _BB void mTopviewSettingsSlot(GtkWidget * widget, gpointer user_data);
+_BB void mNodeFindSlot(GtkWidget * widget, gpointer user_data);
 
 //view
 _BB void mShowToolBoxSlot(GtkWidget * widget, gpointer user_data);
 _BB void mShowHostSelectionSlot(GtkWidget * widget, gpointer user_data);
+_BB void mMenuPan(GtkWidget * widget, gpointer user_data);
+_BB void mMenuZoom(GtkWidget * widget, gpointer user_data);
+
+
 //Graph
 _BB void mNodeListSlot(GtkWidget * widget, gpointer user_data);
 _BB void mNewNodeSlot(GtkWidget * widget, gpointer user_data);
@@ -53,6 +58,7 @@ _BB void mNeatoSlot(GtkWidget * widget, gpointer user_data);
 _BB void mTwopiSlot(GtkWidget * widget, gpointer user_data);
 _BB void mCircoSlot(GtkWidget * widget, gpointer user_data);
 _BB void mFdpSlot(GtkWidget * widget, gpointer user_data);
+_BB void mSfdpSlot(GtkWidget * widget, gpointer user_data);
 
 //select
 _BB void mSelectAllSlot(GtkWidget * widget, gpointer user_data);
@@ -70,6 +76,7 @@ _BB void mSelectAreaXSlot(GtkWidget * widget, gpointer user_data);
 //help
 _BB void mAbout(GtkWidget * widget, gpointer user_data);
 _BB void mHelp(GtkWidget * widget, gpointer user_data);
+void change_cursor(GdkCursorType C);
 
 
 
