@@ -1,4 +1,4 @@
-/* $Id: toolboxcallbacks.h,v 1.2 2008/03/19 21:43:59 erg Exp $ $Revision: 1.2 $ */
+/* $Id: toolboxcallbacks.h,v 1.6 2008/11/07 18:09:50 arif Exp $ $Revision: 1.6 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -40,6 +40,7 @@ _BB void btnToolPan_clicked(GtkWidget * widget, gpointer user_data);
 _BB void btnToolZoom_clicked(GtkWidget * widget, gpointer user_data);
 _BB void btnToolZoomIn_clicked(GtkWidget * widget, gpointer user_data);
 _BB void btnToolZoomOut_clicked(GtkWidget * widget, gpointer user_data);
+_BB void btnToolZoomFit_clicked(GtkWidget * widget, gpointer user_data);
 _BB void btnToolFit_clicked(GtkWidget * widget, gpointer user_data);
 _BB void btnToolMove_clicked(GtkWidget * widget, gpointer user_data);
 _BB void btnToolAddNode_clicked(GtkWidget * widget, gpointer user_data);
@@ -48,7 +49,9 @@ _BB void btnToolFindNode_clicked(GtkWidget * widget, gpointer user_data);
 _BB void btnToolAddEdge_clicked(GtkWidget * widget, gpointer user_data);
 _BB void btnToolDeleteEdge_clicked(GtkWidget * widget, gpointer user_data);
 _BB void btnToolFindEdge_clicked(GtkWidget * widget, gpointer user_data);
+_BB void btnToolCursor_clicked(GtkWidget * widget, gpointer user_data);
 
+void write_to_console(char* text);
 
 
 #endif
