@@ -1,4 +1,4 @@
-/* $Id: main.c,v 1.12 2008/04/25 16:12:53 ellson Exp $ $Revision: 1.12 $ */
+/* $Id: main.c,v 1.22 2008/12/03 20:09:51 north Exp $ $Revision: 1.22 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -35,6 +35,12 @@
 #include "libintl.h"
 #endif
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#else
+#include "compat_getopt.h"
+#endif
+
 #ifdef G_OS_WIN32
 gchar *package_prefix;
 gchar *package_data_dir;
@@ -42,6 +48,7 @@ gchar *package_data_dir;
 gchar *package_locale_dir;
 static char* smyrnaDir;
 char* smyrnaGlade;
+unsigned char SmyrnaVerbose;
 
 char*
 smyrnaPath (char* suffix)
@@ -53,9 +60,61 @@ smyrnaPath (char* suffix)
     return buf;
 }
 
+static char *useString = "Usage: smyrn [-txv?] [-K<engine>] <file>\n\
+  -n         - use TopView mode\n\
+  -e         - use XDOT mode\n\
+  -K<engine> - layout graph using <engine>\n\
+  -v         - verbose\n\
+  -?         - print usage\n";
+
+static void usage(int v)
+{
+    printf(useString);
+    exit(v);
+}
+
+
+static char*
+parseArgs (int argc, char *argv[], ViewInfo* view)
+{
+    unsigned int c;
+
+    while ((c = getopt(argc, argv, ":K:txv?")) != -1) {
+	switch (c) {
+	case 'v':
+	    SmyrnaVerbose = 1;
+	    break;
+	case 't':
+	    view->dfltViewType = VT_TOPVIEW; 
+	    break;
+	case 'x':
+	    view->dfltViewType = VT_XDOT; 
+	    break;
+	case 'K':
+	    view->dfltEngine = s2layout (optarg);
+	    break;
+	case '?':
+	    if (optopt == '?')
+		usage(0);
+	    else
+		fprintf(stderr, "smyrna: option -%c unrecognized - ignored\n",
+			optopt);
+	    break;
+	}
+    }
+
+    if (optind < argc)
+	 return argv[optind];
+    else
+	return NULL;
+}
+extern int create_font_file(char* fontdescription,float gw,float gh);
+extern int load_png_font(char* file_name);
+
 int main(int argc, char *argv[])
 {
     GdkGLConfig *glconfig;
+    char* initFileName;
 
     smyrnaDir = getenv ("SMYRNA_PATH");
 #ifndef _WIN32
@@ -63,7 +122,7 @@ int main(int argc, char *argv[])
 	smyrnaDir = SMYRNA_PATH;
 #endif
 
-    load_attributes();
+	load_attributes();
 
 #ifdef G_OS_WIN32
     package_prefix =
@@ -71,6 +130,7 @@ int main(int argc, char *argv[])
     package_data_dir = g_build_filename(package_prefix, "share", NULL);
     package_locale_dir =
 	g_build_filename(package_prefix, "share", "locale", NULL);
+	add_pixmap_directory("C:/");
 #else
     if (smyrnaDir)
 	package_locale_dir = g_build_filename(smyrnaDir, "locale", NULL);
@@ -84,9 +144,9 @@ int main(int argc, char *argv[])
 #endif
     view = NEW(ViewInfo);
     init_viewport(view);
-
     gtk_set_locale();
     gtk_init(&argc, &argv);
+    initFileName = parseArgs (argc, argv, view);
 
 #ifdef _WIN32
 #define GTKTOPVIEW_ICONSDIR "C:\\Projects\\ATT\\GTK\\GTKTest2\\GUI\\images\\"
@@ -109,12 +169,14 @@ int main(int argc, char *argv[])
     glconfig = configure_gl();
     gladewidget = glade_xml_get_widget(xml, "vbox2");
     create_window(glconfig, gladewidget);
-	/*first arg is used as file name */
-    if (argc > 1)
-	add_graph_to_viewport_from_file(argv[1]);
+
+    if (initFileName)
+	add_graph_to_viewport_from_file(initFileName);
+	change_cursor(GDK_TOP_LEFT_ARROW);
 	gtk_main();
 
 
+
 #ifdef G_OS_WIN32
     g_free(package_prefix);
     g_free(package_data_dir);
