@@ -18,13 +18,14 @@
 
 #ifdef WIN32
 #include "windows.h"
+#include "gltexfont.h"
 //#define       SMYRNA_GLADE "C:/Projects/graphviz2/share/gui/smyrna.glade"
-#define	SMYRNA_ATTRS "c:/Projects/graphviz2/share/gui/attrs.txt"
+#define	SMYRNA_ATTRS "c:/graphviz-ms/share/gui/attrs.txt"
 #endif
 #ifdef _WIN32
-#define SMYRNA_GLADE "c:\\Projects\\graphviz2\\share\\gui\\smyrna.glade"
-#define GTKTOPVIEW_ATTRS "c:\\Projects\\ATT\\GTK\\GTKTest2\\attrs.txt"
-#define DEFAULT_ATTRIBUTES_TEMPLATE_DOT_FILE	"C:\\GTK\\2.0\\bin\\attr_template.dot"
+#define SMYRNA_GLADE "c:/graphviz-ms/share/gui/smyrna.glade"
+#define GTKTOPVIEW_ATTRS "c:/graphviz-ms/share/gui/attrs.txt"
+#define DEFAULT_ATTRIBUTES_TEMPLATE_DOT_FILE	"c:/graphviz-ms/share/gui/attr_template.dot"
 // #else
 // using -DGTKTOPVIEW_GLADE from Makefile.am and configure.ac
 // using -DGTKTOPVIEW_ATTRS from Makefile.am and configure.ac
@@ -36,10 +37,10 @@
 #endif
 
 #include <gtk/gtk.h>
-#include "cgraph.h"
 #include <GL/gl.h>
 #include <GL/glu.h>
 #include <gtk/gtkgl.h>
+#include "cgraph.h"
 #include "glcompset.h"
 #include "hier.h"
 
@@ -61,6 +62,7 @@
 #define MM_MOVE					10
 #define MM_MAGNIFIER			20
 #define MM_FISHEYE_MAGNIFIER	21
+#define GLOBAL_Z_OFFSET			0.001
 
 #define MAX_ZOOM	-0.000033
 #define MIN_ZOOM	-1500.00000
@@ -75,6 +77,12 @@
 #define NODE_ZOOM_LIMIT	-25.3
 #define NODE_CIRCLE_LIMIT	-7.3
 
+#define GL_DOTSIZE_CONSTANT -18
+
+
+#define SPHERE_SLICE_COUNT 6
+#define DOT_SIZE_CORRECTION_FAC 0.3
+
 typedef enum { nodshapedot,nodeshapecircle} node_shape;
 typedef enum { leftmousebutton,rightmousebutton,thirdmousebutton} clicked_mouse_button;
 typedef enum { MOUSE_ROTATE_X,MOUSE_ROTATE_Y,MOUSE_ROTATE_XY,MOUSE_ROTATE_Z} mouse_rotate_axis;
@@ -86,11 +94,13 @@ typedef struct {
     float A;			//Alpha
 } RGBColor;
 
+#define MAX_BTN_CNT 50
+
 typedef struct {
     GtkButton **gtkhostbtn;
     int gtkhostbtncount;
     GtkColorButton **gtkhostcolor;
-    int hostactive[50];		//temporary static, convert to dynamic,realloc
+    int hostactive[MAX_BTN_CNT];  //temporary static, convert to dynamic,realloc
     char **hostregex;
 } topviewdata;
 typedef struct {
@@ -206,21 +216,43 @@ typedef struct _mouse_attr {
 	clicked_mouse_button button;
 } mouse_attr;
 
+typedef enum { 
+    VT_NONE,
+    VT_XDOT,
+    VT_TOPVIEW,
+    VT_TOPFISH,
+} viewtype_t;
+
+typedef enum { 
+    GVE_NONE = -1, 
+    GVE_GRAPH,
+    GVE_CLUSTER,
+    GVE_NODE,
+    GVE_EDGE   /* keep last */
+} gve_element;
+
+typedef enum { 
+    GVK_NONE = -1, 
+    GVK_DOT,
+    GVK_NEATO,
+    GVK_TWOPI,
+    GVK_CIRCO,
+    GVK_FDP,   
+    GVK_SFDP/* keep last */
+} gvk_layout;
 
 typedef struct _attribute {
     char Type;
     char *Name;
     char *Default;
-    char ApplyTo[4];
-    char Engine[5];
+    char ApplyTo[GVE_EDGE+1];
+    char Engine[GVK_FDP+1];
     char **ComboValues;
     int ComboValuesCount;
     GtkWidget *attrWidget;
 
 } attribute;
 
-typedef enum { GVK_DOT,GVK_NEATO,GVK_TWOPI,GVK_CIRCO,GVK_FDP } gvk_layout;
-
 //bind this to cgraph g
 typedef struct _custom_graph_data {
     Agrec_t h;
@@ -303,10 +335,15 @@ typedef struct _custom_object_data	//has to be attached to every Node, Edge, Gra
 typedef struct _selection {
     int Active;			//0 there is no selection need to be applied
     char Type;			//0     single selection , 1 rectangle , 2 rectangleX 
-    float X, Y, W, H;		//selection boundries
+	int PickingType;	//0 normal, union,2 subtract 3 intersection
+	float X, Y, W, H;		//selection boundries
     int Anti;			//subtract selections if 1
     int AlreadySelected;	//for single selections to avoid selecting more than one object
     RGBColor SelectionColor;
+	float node_distance; //to get the closest node , this value is updated for each node, distance between selection coords and node coords, smallest gets to be selected
+	topview_node* single_selected_node; //pointer to selected node in a single node/edge selection cycle,after each node loop this value is checked and if it is in the limits that node is selected or deselected
+										//before the node/edge loop this value is nulled
+	topview_edge* single_selected_edge;//pointer to selected/picked edge
 
 } selection;
 typedef struct _magnifier {
@@ -358,6 +395,12 @@ typedef struct _ViewInfo
 	RGBColor selectedNodeColor;
 	/*selected edge color*/
 	RGBColor selectedEdgeColor;
+	/*default node alpha*/
+	float defaultnodealpha;
+	/*default edge alpha*/
+	float defaultedgealpha;
+	/*default node shape*/
+	int defaultnodeshape;
 
 	/*default line width*/
 	float LineWidth;
@@ -377,7 +420,7 @@ typedef struct _ViewInfo
 	/*randomize edge colors or use default edge color*/
 	int rndEdgeColor;
 	/*Font Size*/
-	int FontSize;
+	float FontSize;
 
 
 	float bdxLeft,bdyTop,bdzTop;	
@@ -450,8 +493,12 @@ typedef struct _ViewInfo
 	int drawnodes;
 	int drawedges;
 	int drawlabels;
-
-}ViewInfo;
+	viewtype_t dfltViewType;
+	gvk_layout dfltEngine;
+	GtkTextBuffer* consoleText;
+	float FontSizeConst;
+	fontset_t* fontset;
+} ViewInfo;
 
 extern ViewInfo *view;
 extern GtkMessageDialog *Dlg;
@@ -459,7 +506,11 @@ extern int respond;
 extern char* smyrnaPath (char* suffix);
 extern char* smyrnaGlade;
 
-extern void glexpose();
+extern void glexpose(void);
 
+extern char* layout2s (gvk_layout gvkl);
+extern gvk_layout s2layout (char* s);
+extern char* element2s (gve_element);
+extern unsigned char SmrynaVerbose;
 
 #endif
