@@ -1,4 +1,4 @@
-/* $Id: topview.c,v 1.80 2009/02/26 20:46:13 erg Exp $Revision: */
+/* $Id: topview.c,v 1.82 2009/03/11 16:28:19 arif Exp $Revision: */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -17,6 +17,9 @@
 #include "glTexFontTGA.h"
 #include "glTexFontDefs.h"
 #include "glTexFontInclude.h"
+#include "glcompbutton.h"
+#include "glcomppanel.h"
+#include "glcomplabel.h"
 #include "gltemplate.h"
 #include "topview.h"
 #include "math.h"
@@ -79,7 +82,7 @@ void preparetopview(Agraph_t * g, topview * t)
 	int maxlabelsize=0;
 	float maxedgelen,minedgelen,len,edgelength;
 	maxedgelen=0;
-	minedgelen=99999999;	//FIX ME if you have a giant graph or fix your graph
+	minedgelen=(float)99999999.00000;	//FIX ME if you have a giant graph or fix your graph
 	edgelength=0;
 
     ind = 0;
@@ -164,7 +167,7 @@ void preparetopview(Agraph_t * g, topview * t)
 			t->Nodes[ind].Label2 = '\0';
 
 		if (t->Nodes[ind].Label)
-			maxlabelsize = maxlabelsize + strlen(t->Nodes[ind].Label);
+			maxlabelsize = maxlabelsize + (int)strlen(t->Nodes[ind].Label);
 
 		for (e = agfstout(g, v); e; e = agnxtout(g, e)) 
 		{
@@ -199,7 +202,7 @@ void preparetopview(Agraph_t * g, topview * t)
 				t->Edges[ind2].y2 = b;
 				t->Edges[ind2].z2 = c;
 			}
-			len=pow(pow((t->Edges[ind2].x2-t->Edges[ind2].x1),2)+pow((t->Edges[ind2].y2-t->Edges[ind2].y1),2),0.5);
+			len=(float)pow(pow((t->Edges[ind2].x2-t->Edges[ind2].x1),2)+pow((t->Edges[ind2].y2-t->Edges[ind2].y1),2),0.5);
 			if (len > maxedgelen)
 				maxedgelen=len;
 			if (len < minedgelen)
@@ -268,7 +271,6 @@ void preparetopview(Agraph_t * g, topview * t)
 
 static float set_gl_dot_size(topview * t)
 {
-	GLfloat a;
 	static float dotsize;
 	static float prevdotsize;
 	if (view->active_camera==-1)
@@ -278,7 +280,7 @@ static float set_gl_dot_size(topview * t)
 
 //	dotsize=dotsize * DOT_SIZE_CORRECTION_FAC;
 	if (dotsize <=1.2)
-		dotsize=1.2;
+		dotsize=(float)1.2;
 	glPointSize((GLfloat)dotsize);
 	return dotsize;
 
@@ -642,13 +644,14 @@ static int draw_node_hint_boxes(void)
 	fontColorA(view->fontset->fonts[view->fontset->activefont],0, 0, 1, 1);
 
 
-	fontDrawString(view->fontset->fonts[view->fontset->activefont],
+	fontDrawString(
+		view->fontset->fonts[view->fontset->activefont],
 		       (view->Topview->picked_nodes[ind]->distorted_x),
 		        (view->Topview->picked_nodes[ind]->
-			      distorted_y+fs+fs/5.0 ),
+			      distorted_y+fs+fs/(GLfloat)5.0 ),
 		       fs *
 		       (float)strlen(agnameof
-			      (view->Topview->picked_nodes[ind]->Node)) / 2.0,
+			      (view->Topview->picked_nodes[ind]->Node)) / (GLfloat)2.0,
 		       agnameof(view->Topview->picked_nodes[ind]->Node));
     }
     return 1;
@@ -671,7 +674,7 @@ static int select_topview_node(topview_node * n)
 		|| 
 		(view->mouse.button== rightmousebutton))	//single selection or right click (picking)
 	{
-		float dist=pow((view->Selection.X-n->distorted_x),2)+pow((view->Selection.Y-n->distorted_y),2);
+		float dist=(float)pow((view->Selection.X-n->distorted_x),2)+(float)pow((view->Selection.Y-n->distorted_y),2);
 		if ((view->Selection.node_distance==-1) ||(dist < view->Selection.node_distance))
 		{
 				view->Selection.node_distance=dist;
@@ -834,8 +837,8 @@ int set_update_required(topview * t)
 
 float calculate_font_size(topview_node * v)
 {
-	double n;
-	n=(double)v->degree+(double)1.00;
+	float n;
+	n=(float)v->degree+(float)1.00;
 	return n;
 
 }
@@ -874,11 +877,11 @@ static int draw_topview_label(topview_node * v, float zdepth)
 		return 0;
 
 //	fs= 10;
-	fs= fs * 0.2;
+	fs= fs * (float)0.2;
 
 
 
-	fontSize(view->fontset->fonts[view->fontset->activefont],(int) fs);
+	fontSize(view->fontset->fonts[view->fontset->activefont],fs);
 	if ((log((float) v->degree) * -0.6 * view->zoom) > 0)
 	    fontColorA(view->fontset->fonts[view->fontset->activefont],(float) log((double) v->degree + (double) 1),
 		       view->penColor.G, view->penColor.B,
@@ -886,11 +889,11 @@ static int draw_topview_label(topview_node * v, float zdepth)
 		       (float) -0.4 * (float) view->zoom);
 	else
 	    fontColorA(view->fontset->fonts[view->fontset->activefont],(float) log((double) v->degree + (double) 1),
-		       view->penColor.G, view->penColor.B, 0.7);
+		       view->penColor.G, view->penColor.B, (float)0.7);
 
 //	fontColorA(0,0,0,1);
-	fontDrawString(view->fontset->fonts[view->fontset->activefont],(int) (v->distorted_x - ddx),
-		       (int) (v->distorted_y - ddy), (int) (fs * strlen(v->Label)*0.6),v->Label   );
+	fontDrawString(view->fontset->fonts[view->fontset->activefont],(v->distorted_x - ddx),
+		        (v->distorted_y - ddy),  (fs * strlen(v->Label)*(float)0.6),v->Label   );
 
 	return 1;
     } else
@@ -1264,16 +1267,16 @@ static void menu_click_fisheye_magnifier(void *p)
 
 static void menu_click_zoom_minus(void *p)
 {
-    if ((view->zoom - ZOOM_STEP) > MIN_ZOOM)
-	view->zoom = view->zoom - ZOOM_STEP;
+    if ((view->zoom - (float)ZOOM_STEP) > (float)MIN_ZOOM)
+	view->zoom = view->zoom - (float)ZOOM_STEP;
     else
-	view->zoom = MIN_ZOOM;
+	view->zoom = (float)MIN_ZOOM;
 }
 
 static void menu_click_zoom_plus(void *p)
 {
-    if ((view->zoom + ZOOM_STEP) < MAX_ZOOM)
-	view->zoom = view->zoom + ZOOM_STEP;
+    if ((view->zoom + (float)ZOOM_STEP) < (float)MAX_ZOOM)
+	view->zoom = view->zoom + (float)ZOOM_STEP;
     else
 	view->zoom = (float) MAX_ZOOM;
 
@@ -1441,11 +1444,11 @@ static glCompSet *glcreate_gl_topview_menu(void)
      */
 
     //small panel left bottom
-    p = glCompPanelNew(25, 25, 325, 40);
+    p = glCompPanelNew(25, 25, 325, 40,scientific_y);
     p->data = 0;
     glCompSetAddPanel(s, p);
 
-    b = glCompButtonNew(5, 7, 75, 25, "BROWSE", '\0', 0, 0);
+    b = glCompButtonNew(5, 7, 75, 25, "BROWSE", '\0', 0, 0,scientific_y);
     b->panel = p;
     b->groupid = 1;
     b->customptr = view;
@@ -1453,7 +1456,7 @@ static glCompSet *glcreate_gl_topview_menu(void)
     b->callbackfunc = menu_click_control;
 
 
-    b = glCompButtonNew(85, 7, 75, 25, "SHOW", '\0', 0, 0);
+    b = glCompButtonNew(85, 7, 75, 25, "SHOW", '\0', 0, 0,scientific_y);
     b->panel = p;
     b->customptr = view;
     b->groupid = 1;
@@ -1461,14 +1464,14 @@ static glCompSet *glcreate_gl_topview_menu(void)
     glCompSetAddButton(s, b);
 
 
-    b = glCompButtonNew(165, 7, 75, 25, "CAMERAS", '\0', 0, 0);
+    b = glCompButtonNew(165, 7, 75, 25, "CAMERAS", '\0', 0, 0,scientific_y);
     b->customptr = view;
     b->panel = p;
     b->groupid = 1;
     b->callbackfunc = menu_click_3d_view;
     glCompSetAddButton(s, b);
 
-    b = glCompButtonNew(245, 7, 75, 25, "HIDE", '\0', 0, 0);
+    b = glCompButtonNew(245, 7, 75, 25, "HIDE", '\0', 0, 0,scientific_y);
     b->color.R = 1;
     b->customptr = view;
     b->panel = p;
@@ -1476,14 +1479,18 @@ static glCompSet *glcreate_gl_topview_menu(void)
     b->callbackfunc = menu_click_hide;
     glCompSetAddButton(s, b);
 
+    //Fisheye control panel
+//	p = glCompPanelNew(	25, view->h-40, 165, 40,scientific_y);
+  //  p->data = 1;		//control panel
+  //  glCompSetAddPanel(s, p);
 
-    //control panel
-    p = glCompPanelNew(25, 75, 165, 277);
+	//control panel
+    p = glCompPanelNew(25, 75, 165, 277,scientific_y);
     p->data = 1;		//control panel
     glCompSetAddPanel(s, p);
 
     //view mode normal button
-    b = glCompButtonNew(5, 7, 75, 25, "NORMAL", '\0', 0, 0);
+    b = glCompButtonNew(5, 7, 75, 25, "NORMAL", '\0', 0, 0,scientific_y);
     b->color.R = 0;
     b->color.G = 1;
     b->color.B = (float) 0.1;
@@ -1494,7 +1501,7 @@ static glCompSet *glcreate_gl_topview_menu(void)
 
     glCompSetAddButton(s, b);
     //view mode fisheye button
-    b = glCompButtonNew(85, 7, 75, 25, "FISHEYE", '\0', 0, 0);
+    b = glCompButtonNew(85, 7, 75, 25, "FISHEYE", '\0', 0, 0,scientific_y);
     b->color.R = 0;
     b->color.G = 1;
     b->color.B = (float) 0.1;
@@ -1505,33 +1512,33 @@ static glCompSet *glcreate_gl_topview_menu(void)
     glCompSetAddButton(s, b);
 
     //rotate
-    b = glCompButtonNew(5, 197, 72, 72, "", smyrna_icon_rotate, 72, 72);
+    b = glCompButtonNew(5, 197, 72, 72, "", smyrna_icon_rotate, 72, 72,scientific_y);
     b->groupid = 3;
     b->customptr = view;
     b->panel = p;
     b->callbackfunc = menu_click_rotate;
     glCompSetAddButton(s, b);
 
-    b = glCompButtonNew(80, 251, 40, 20, "X", '\0', 0, 0);
+    b = glCompButtonNew(80, 251, 40, 20, "X", '\0', 0, 0,scientific_y);
     b->customptr = view;
     b->panel = p;
     b->groupid = 1;
     b->callbackfunc = menu_click_rotate_x;
     glCompSetAddButton(s, b);
-    b = glCompButtonNew(125, 251, 40, 20, "Y", '\0', 0, 0);
+    b = glCompButtonNew(125, 251, 40, 20, "Y", '\0', 0, 0,scientific_y);
     b->customptr = view;
     b->panel = p;
     b->groupid = 1;
     b->callbackfunc = menu_click_rotate_y;
     glCompSetAddButton(s, b);
-    b = glCompButtonNew(80, 231, 40, 20, "XY", '\0', 0, 0);
+    b = glCompButtonNew(80, 231, 40, 20, "XY", '\0', 0, 0,scientific_y);
     b->customptr = view;
     b->panel = p;
     b->groupid = 1;
     b->callbackfunc = menu_click_rotate_xy;
 
 	glCompSetAddButton(s, b);
-    b = glCompButtonNew(125, 231, 40, 20, "Z", '\0', 0, 0);
+    b = glCompButtonNew(125, 231, 40, 20, "Z", '\0', 0, 0,scientific_y);
     b->customptr = view;
     b->panel = p;
     b->groupid = 1;
@@ -1539,7 +1546,7 @@ static glCompSet *glcreate_gl_topview_menu(void)
     glCompSetAddButton(s, b);
 
 	//sanity button to center the drawing and fit it in the screen
-    b = glCompButtonNew(80, 201, 90, 20, "center", '\0', 0, 0);
+    b = glCompButtonNew(80, 201, 90, 20, "center", '\0', 0, 0,scientific_y);
     b->customptr = view;
     b->panel = p;
     b->groupid = 0;
@@ -1554,7 +1561,7 @@ static glCompSet *glcreate_gl_topview_menu(void)
 
     //pan button
     b = glCompButtonNew(5, 120, 72, 72, "adasasds", smyrna_icon_pan, 72,
-			72);
+			72,scientific_y);
     b->groupid = 3;
     b->customptr = view;
     b->panel = p;
@@ -1562,7 +1569,7 @@ static glCompSet *glcreate_gl_topview_menu(void)
     glCompSetAddButton(s, b);
     //zoom
     b = glCompButtonNew(85, 120, 72, 72, "adasasds", smyrna_icon_zoom, 72,
-			72);
+			72,scientific_y);
     b->groupid = 3;
     b->customptr = view;
     b->panel = p;
@@ -1570,7 +1577,7 @@ static glCompSet *glcreate_gl_topview_menu(void)
     glCompSetAddButton(s, b);
     //zoom +
     b = glCompButtonNew(85, 82, 36, 36, "adasasds", smyrna_icon_zoomplus,
-			36, 36);
+			36, 36,scientific_y);
     b->groupid = 0;
     b->customptr = view;
     b->panel = p;
@@ -1578,7 +1585,7 @@ static glCompSet *glcreate_gl_topview_menu(void)
     glCompSetAddButton(s, b);
     //zoom -
     b = glCompButtonNew(121, 82, 36, 36, "adasasds", smyrna_icon_zoomminus,
-			36, 36);
+			36, 36,scientific_y);
     b->groupid = 0;
     b->panel = p;
     b->customptr = view;
@@ -1586,19 +1593,19 @@ static glCompSet *glcreate_gl_topview_menu(void)
     glCompSetAddButton(s, b);
 
     b = glCompButtonNew(5, 45, 72, 72, "adasasds", smyrna_icon_fisheye, 72,
-			72);
+			72,scientific_y);
     b->groupid = 3;
     b->panel = p;
     b->customptr = view;
     b->callbackfunc = menu_click_fisheye_magnifier;
     glCompSetAddButton(s, b);
     //zoom percantage label
-    l = glCompLabelNew(100, 45, 24, "100");
+    l = glCompLabelNew(100, 45, 24, "100",scientific_y);
     l->panel = p;
     l->fontsizefactor = (float) 0.4;
     glCompSetAddLabel(s, l);
     view->Topview->customptr = l;
-    l = glCompLabelNew(93, 65, 20, "zoom");
+    l = glCompLabelNew(93, 65, 20, "zoom",scientific_y);
     l->panel = p;
     l->fontsizefactor = (float) 0.4;
     glCompSetAddLabel(s, l);
