@@ -1,4 +1,4 @@
-/* $Id: topview.h,v 1.15 2008/04/29 21:59:16 erg Exp $ $Revision: 1.15 $ */
+/* $Id: topview.h,v 1.18 2009/02/23 16:16:28 ellson Exp $ $Revision: 1.18 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -28,8 +28,10 @@ void cleartopview(topview * t);
 void preparetopview(Agraph_t * g, topview * t);
 void drawTopViewGraph(Agraph_t * g);
 int set_update_required(topview * t);
-int move_TVnodes();
+int move_TVnodes(void);
 void local_zoom(topview * t);
 void originate_distorded_coordinates(topview * t);
+float calcfontsize(float totaledgelength,int totallabelsize,int edgecount,int totalnodecount);
+void select_with_regex(char* exp);
 
 #endif
