@@ -1,4 +1,4 @@
-/* $Id: viewport.c,v 1.50 2009/03/02 15:44:54 arif Exp $ $Revision: 1.50 $ */
+/* $Id: viewport.c,v 1.53 2009/03/11 16:28:19 arif Exp $ $Revision: 1.53 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -225,7 +225,7 @@ set_viewport_settings_from_template(ViewInfo * view, Agraph_t * g)
     /*default line width */
     view->LineWidth =
 	(float) atof(get_attribute_value("defaultlinewidth", view, g));
-    view->FontSize = atoi(get_attribute_value("defaultfontsize", view, g));
+    view->FontSize = (float)atof(get_attribute_value("defaultfontsize", view, g));
 
     view->topviewusermode = atoi(get_attribute_value("usermode", view, g));
     get_attribute_value("defaultmagnifierwidth", view, g);
@@ -387,10 +387,10 @@ void init_viewport(ViewInfo * view)
 	view->timer=g_timer_new();
 	g_timer_stop(view->timer); 
 	view->active_frame=0;
-	view->total_frames=100;
-	view->frame_length=5;
+	view->total_frames=1500;
+	view->frame_length=1;
 	/*add a call back to the main()*/
-	g_timeout_add_full(G_PRIORITY_DEFAULT,100,gl_main_expose,NULL,NULL);
+	g_timeout_add_full((gint)G_PRIORITY_DEFAULT,(guint)100,gl_main_expose,NULL,NULL);
 	view->Topview->topviewmenu = '\0';
 	view->cameras='\0';;
 	view->camera_count=0;
@@ -620,8 +620,8 @@ static char* create_xdot_for_graph(Agraph_t * graph, int keeppos)
 {
     static char buf[BUFSIZ];
     static char* cmd = buf;
-    static int buflen = BUFSIZ;
-    int len;
+    static size_t buflen = BUFSIZ;
+    size_t len;
     int r = 0;
     FILE *output_file;
     char* fix;
