@@ -1,4 +1,4 @@
-/* $Id: convert.h,v 1.2 2008/01/09 20:50:35 erg Exp $ $Revision: 1.2 $ */
+/* $Id: convert.h,v 1.5 2008/08/07 17:17:46 ellson Exp $ $Revision: 1.5 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -25,21 +25,18 @@ extern "C" {
 #include "config.h"
 #endif
 
-#ifdef USE_CGRAPH
-#include <cgraph.h>
-#include <cghdr.h>
-#else
-#include <agraph.h>
-#endif
 #include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
 #include <string.h>
 
-    extern void dot_to_gxl(Agraph_t *, FILE *);
-#ifdef HAVE_LIBEXPAT
-    extern Agraph_t *gxl_to_dot(FILE *);
+#include "cgraph.h"
+#include "cghdr.h"
+
+    extern void gv_to_gxl(Agraph_t *, FILE *);
+#ifdef HAVE_EXPAT
+    extern Agraph_t *gxl_to_gv(FILE *);
 #endif
 
 #endif
