@@ -1,4 +1,4 @@
-/* $Id: cvtgxl.c,v 1.4 2007/08/08 23:00:58 erg Exp $ $Revision: 1.4 $ */
+/* $Id: cvtgxl.c,v 1.7 2008/08/04 20:43:06 ellson Exp $ $Revision: 1.7 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -19,24 +19,24 @@
  * Written by Emden R. Gansner and Krishnam Pericherla 
  */
 
-#include <convert.h>
-
+#include <ctype.h>
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #else
 #include "compat_getopt.h"
 #endif
 
-#include <ingraphs.h>
+#include "convert.h"
+#include "ingraphs.h"
 
-typedef enum { Unset, ToDot, ToGXL } mode;
+typedef enum { Unset, ToGV, ToGXL } mode;
 
 static FILE *outFile;
 static char *CmdName;
 static char **Files;
 static mode act = Unset;
 
-#ifdef HAVE_LIBEXPAT
+#ifdef HAVE_EXPAT
 static FILE *getFile(void)
 {
     FILE *rv = NULL;
@@ -83,7 +83,7 @@ static FILE *openFile(char *name, char *mode)
 
 static const char *use = "Usage: %s [-gd?] [-o<file>] [<graphs>]\n\
  -g        : convert to GXL\n\
- -d        : convert to dot\n\
+ -d        : convert to GV\n\
  -o<file>  : output to <file> (stdout)\n\
  -?        : usage\n";
 
@@ -113,28 +113,26 @@ static void checkInput(void)
     if (!ep)
 	return;
     ep++;
-    if (strcmp(ep, "dot") == 0)
+    if (strcmp(ep, "gv") == 0)
+	act = ToGXL;
+    else if (strcmp(ep, "dot") == 0)
 	act = ToGXL;
     else if (strcmp(ep, "gxl") == 0)
-	act = ToDot;
+	act = ToGV;
 }
 
 static void setAction(void)
 {
-    switch (*CmdName) {
-    case 'd':
-    case 'D':
+    if (tolower(CmdName[0]) == 'd') 
 	act = ToGXL;
-	break;
-    case 'g':
-    case 'G':
-	act = ToDot;
-	break;
-    default:
-	if (Files)
-	    checkInput();
-	break;
+    else if (tolower(CmdName[0]) == 'g') {
+	if (tolower(CmdName[1]) == 'v')
+	    act = ToGXL;
+	else
+	    act = ToGV;
     }
+    else if (Files)
+	checkInput();
 
     if (act == Unset) {
 	fprintf(stderr, "Cannot determine conversion type\n");
@@ -150,7 +148,7 @@ static void initargs(int argc, char **argv)
     while ((c = getopt(argc, argv, ":gdo:?")) != -1) {
 	switch (c) {
 	case 'd':
-	    act = ToDot;
+	    act = ToGV;
 	    break;
 	case 'g':
 	    act = ToGXL;
@@ -199,14 +197,14 @@ int main(int argc, char **argv)
 	    if (prev)
 		agclose(prev);
 	    prev = G;
-	    dot_to_gxl(G, outFile);
+	    gv_to_gxl(G, outFile);
 	    fflush(outFile);
 	}
     } else {
-#ifdef HAVE_LIBEXPAT
+#ifdef HAVE_EXPAT
 	FILE *inFile;
 	while ((inFile = getFile())) {
-	    while ((G = gxl_to_dot(inFile))) {
+	    while ((G = gxl_to_gv(inFile))) {
 		if (prev)
 		    agclose(prev);
 		prev = G;
@@ -215,7 +213,7 @@ int main(int argc, char **argv)
 	    }
 	}
 #else
-	fputs("cvtgxl: not configured for conversion from GXL to DOT\n",
+	fputs("cvtgxl: not configured for conversion from GXL to GV\n",
 	      stderr);
 	exit(1);
 
