@@ -1,4 +1,4 @@
-/* $Id: gvpack_builtins.c,v 1.1 2005/10/18 18:30:07 ellson Exp $ $Revision: 1.1 $ */
+/* $Id: gvpack_builtins.c,v 1.4 2009/02/12 17:47:23 ellson Exp $ $Revision: 1.4 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -18,8 +18,7 @@
 
 extern gvplugin_library_t gvplugin_neato_layout_LTX_library;
 
-const lt_symlist_t lt_preloaded_symbols[] = {
-    { "gvplugin_neato_layout_LTX_library", (void*)(&gvplugin_neato_layout_LTX_library) },
-    { 0, 0 }
+lt_symlist_t lt_preloaded_symbols[] = {
+	{ "gvplugin_neato_layout_LTX_library", (void*)(&gvplugin_neato_layout_LTX_library) },
+	{ 0, 0 }
 };
-
