@@ -1,4 +1,4 @@
-/* $Id: diffimg.c,v 1.13 2008/02/22 14:21:42 glenlow Exp $ $Revision: 1.13 $ */
+/* $Id: diffimg.c,v 1.14 2009/01/23 23:33:05 arif Exp $ $Revision: 1.14 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -35,12 +35,17 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h>
+#ifndef WIN32
+//#include <unistd.h>
+#endif
+
 #ifdef WIN32
 #define EX_USAGE		64
 #define EX_DATAERR		65
 #define EX_NOINPUT		66
 #define EX_UNAVAILABLE	69
+#define bool int
+#define false 0
 #else
 #include <sysexits.h>
 #endif
