@@ -1,4 +1,4 @@
-/* $Id: error.h,v 1.3 2007/07/23 18:45:29 ellson Exp $ $Revision: 1.3 $ */
+/* $Id: error.h,v 1.4 2008/10/31 17:14:54 erg Exp $ $Revision: 1.4 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -33,6 +33,7 @@ extern "C" {
 	int indent;
 	int line;
 	int warnings;
+	int trace;
 	char *file;
 	char *id;
     } Error_info_t;
@@ -57,6 +58,7 @@ extern "C" {
 
     extern Error_info_t error_info;
 
+    extern void setTraceLevel (int);
     extern void setErrorLine (int);
     extern void setErrorFileLine (char*, int);
     extern void setErrorId (char*);
