@@ -1,4 +1,4 @@
-/* $Id: pathpath.c,v 1.5 2008/03/14 04:10:39 ellson Exp $ $Revision: 1.5 $ */
+/* $Id: pathpath.c,v 1.6 2008/07/16 05:48:07 glenlow Exp $ $Revision: 1.6 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -44,7 +44,7 @@
 #ifdef WIN32
 #define environ _environ
 #else
-#ifdef HAVE__NSGETENVIRON
+#if defined(HAVE_CRT_EXTERNS_H) && defined(HAVE__NSGETENVIRON)
 #define environ (*_NSGetEnviron())
 #else
 extern char **environ;
