@@ -1,4 +1,4 @@
-/* $Id: cgraph.h,v 1.3 2007/11/09 23:39:28 erg Exp $ $Revision: 1.3 $ */
+/* $Id: cgraph.h,v 1.12 2009/01/21 16:47:00 arif Exp $ $Revision: 1.12 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -17,7 +17,11 @@
 #ifndef ATT_GRAPH_H
 #define ATT_GRAPH_H
 
-#include		<cdt.h>
+#include		"cdt.h"
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 #ifndef FALSE
 #define FALSE (0)
@@ -109,8 +113,8 @@ string name at its time of creation, or it is permissible to pass NIL(char*)
 for the name. */
 
 struct Agsubnode_s {		/* the node-per-graph-or-subgraph record */
+    Dtlink_t seq_link;		/* must be first */
     Dtlink_t id_link;
-    Dtlink_t seq_link;
     Agnode_t *node;		/* the object */
     Dtlink_t *in_id, *out_id;	/* by node/ID for random access */
     Dtlink_t *in_seq, *out_seq;	/* by node/sequence for serial access */
@@ -135,7 +139,8 @@ struct Agedgepair_s {
 
 struct Agdesc_s {		/* graph descriptor */
     unsigned directed:1;	/* if edges are asymmetric */
-    unsigned strict:1;		/* if and self, multi-edges forbidden */
+    unsigned strict:1;		/* if multi-edges forbidden */
+    unsigned no_loop:1;		/* if no loops */
     unsigned maingraph:1;	/* if this is the top level graph */
     unsigned flatlock:1;	/* if sets are flattened into lists in cdt */
     unsigned no_write:1;	/* if a temporary subgraph */
@@ -177,11 +182,24 @@ struct Agdisc_s {		/* user's discipline */
 };
 
 	/* default resource disciplines */
+#if !defined(_BLD_cgraph) && defined(GVDLL)
+#define extern  __declspec(dllimport)
+#endif
+
+/*visual studio*/
+#ifdef WIN32_DLL
+#ifndef CGRAPH_EXPORTS
+#define extern __declspec(dllimport)
+#endif
+#endif
+/*end visual studio*/
+
 extern Agmemdisc_t AgMemDisc;
 extern Agiddisc_t AgIdDisc;
 extern Agiodisc_t AgIoDisc;
 
 extern Agdisc_t AgDefaultDisc;
+#undef extern
 
 struct Agdstate_s {
     void *mem;
@@ -234,10 +252,10 @@ struct Agraph_s {
 
 #if _PACKAGE_ast
 /* fine control of object callbacks */
-#   if defined(_BLD_agraph) && defined(__EXPORT__)
+#   if defined(_BLD_cgraph) && defined(__EXPORT__)
 #	define extern  __EXPORT__
 #   endif
-#   if !defined(_BLD_agraph) && defined(__IMPORT__)
+#   if !defined(_BLD_cgraph) && defined(__IMPORT__)
 #	define extern  __IMPORT__
 #   endif
 #endif
@@ -257,6 +275,7 @@ extern int agwrite(Agraph_t * g, void *chan);
 extern int agisdirected(Agraph_t * g);
 extern int agisundirected(Agraph_t * g);
 extern int agisstrict(Agraph_t * g);
+extern int agissimple(Agraph_t * g);
 
 /* nodes */
 extern Agnode_t *agnode(Agraph_t * g, char *name, int createflag);
@@ -283,7 +302,9 @@ extern Agedge_t *agfstedge(Agraph_t * g, Agnode_t * n);
 extern Agedge_t *agnxtedge(Agraph_t * g, Agedge_t * e, Agnode_t * n);
 
 /* generic */
-extern Agraph_t *agraphof(void *);
+extern Agraph_t *agraphof(void* obj);
+extern Agraph_t *agroot(void* obj);
+extern int agcontains(Agraph_t *, void *);
 extern char *agnameof(void *);
 extern int agrelabel(void *obj, char *name);	/* scary */
 extern int agrelabel_node(Agnode_t * n, char *newname);
@@ -328,7 +349,7 @@ struct Agdatadict_s {		/* set of dictionaries per graph */
 extern Agsym_t *agattr(Agraph_t * g, int kind, char *name, char *value);
 extern Agsym_t *agattrsym(void *obj, char *name);
 extern Agsym_t *agnxtattr(Agraph_t * g, int kind, Agsym_t * attr);
-extern int      agcopyattr(void *old, void *new);
+extern int      agcopyattr(void *oldobj, void *newobj);
 
 extern void *agbindrec(void *obj, char *name, unsigned int size,
 		       int move_to_front);
@@ -348,7 +369,7 @@ extern int agsafeset(void* obj, char* name, char* value, char* def);
 extern Agraph_t *agsubg(Agraph_t * g, char *name, int cflag);	/* constructor */
 extern Agraph_t *agidsubg(Agraph_t * g, unsigned long id, int cflag);	/* constructor */
 extern Agraph_t *agfstsubg(Agraph_t * g), *agnxtsubg(Agraph_t * subg);
-extern Agraph_t *agparent(Agraph_t * g), *agroot(Agraph_t * g);
+extern Agraph_t *agparent(Agraph_t * g);
 
 /* set cardinality */
 extern int agnnodes(Agraph_t * g), agnedges(Agraph_t * g);
@@ -394,16 +415,52 @@ extern int agerrors(void);
 #define HEADPORT_ID		"headport"
 
 #if _PACKAGE_ast
-#   if !defined(_BLD_agraph) && defined(__IMPORT__)
+#   if !defined(_BLD_cgraph) && defined(__IMPORT__)
 #	define extern  __IMPORT__
 #   endif
 #endif
+#if !defined(_BLD_cgraph) && defined(GVDLL)
+#define extern  __declspec(dllimport)
+#endif
 
 extern Agdesc_t Agdirected, Agstrictdirected, Agundirected,
     Agstrictundirected;
 
+#undef extern
+
+/* fast graphs */
+void agflatten(Agraph_t * g, int flag);
+typedef Agsubnode_t	Agnoderef_t;
+typedef Dtlink_t	Agedgeref_t;
+
+#define AGHEADPOINTER(g)	((Agnoderef_t*)(g->n_seq->data->hh._head))
+#define AGRIGHTPOINTER(rep)  	((Agnoderef_t*)((rep)->seq_link.right?((void*)((rep)->seq_link.right) - offsetof(Agsubnode_t,seq_link)):0))
+#define AGLEFTPOINTER(rep)  	((Agnoderef_t*)((rep)->seq_link.hl._left?((void*)((rep)->seq_link.hl._left) - offsetof(Agsubnode_t,seq_link)):0))
+
+#define FIRSTNREF(g)	(agflatten(g,1), AGHEADPOINTER(g))
+
+#define NEXTNREF(g,rep)  	(AGRIGHTPOINTER(rep) == AGHEADPOINTER(g)?0:AGRIGHTPOINTER(rep))
+
+#define PREVNREF(g,rep)  	(((rep)==AGHEADPOINTER(g))?0:(AGLEFTPOINTER(rep)))
+
+#define LASTNREF(g)		(agflatten(g,1), AGHEADPOINTER(g)?AGLEFTPOINTER(AGHEADPOINTER(g)):0)
+#define NODEOF(rep)		((rep)->node)
+
+#define FIRSTOUTREF(g,sn)	(agflatten(g,1), (sn)->out_seq)
+#define LASTOUTREF(g,sn)	(agflatten(g,1), (Agedgeref_t*)dtlast(sn->out_seq))
+#define FIRSTINREF(g,sn)	(agflatten(g,1), (sn)->in_seq)
+#define NEXTEREF(g,rep)  	((rep)->right)
+#define PREVEREF(g,rep)  	((rep)->hl._left)
+/* this is expedient but a bit slimey because it "knows" that dict entries of both nodes
+and edges are embedded in main graph objects but allocated separately in subgraphs */
+#define AGSNMAIN(sn)        ((sn)==(&((sn)->node->mainsub)))
+#define EDGEOF(sn,rep)		(AGSNMAIN(sn)?((Agedge_t*)((unsigned char*)(rep) - offsetof(Agedge_t,seq_link))) : ((Dthold_t*)(rep))->obj)
+
 #undef extern
 #if _PACKAGE_ast
 _END_EXTERNS_
 #endif
+#ifdef __cplusplus
+}
+#endif
 #endif
