@@ -1,4 +1,4 @@
-/* $Id: edge.c,v 1.7 2009/01/23 23:33:05 arif Exp $ $Revision: 1.7 $ */
+/* $Id: edge.c,v 1.9 2009/03/11 17:27:54 north Exp $ $Revision: 1.9 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -91,12 +91,12 @@ Agedge_t *agnxtedge(Agraph_t * g, Agedge_t * e, Agnode_t * n)
 {
     Agedge_t *rv;
 
-    if (n == AGTAIL(e)) {
-	rv = agnxtout(g, e);
-	if (rv == NILedge)
-	    rv = agfstin(g, n);
+	if (AGTYPE(e) == AGOUTEDGE) {
+		rv = agnxtout(g, e);
+		if (rv == NILedge)
+			rv = agfstin(g, n);
     } else
-	rv = agnxtin(g, e);
+		rv = agnxtin(g, e);
     return rv;
 }
 
@@ -180,9 +180,11 @@ static void installedge(Agraph_t * g, Agedge_t * e)
 	sn = agsubrep(g, t);
 	ins(g->e_seq, &sn->out_seq, out);
 	ins(g->e_id, &sn->out_id, out);
-	sn = agsubrep(g, h);
-	ins(g->e_seq, &sn->in_seq, in);
-	ins(g->e_id, &sn->in_id, in);
+	if (t != h) {
+		sn = agsubrep(g, h);
+		ins(g->e_seq, &sn->in_seq, in);
+		ins(g->e_id, &sn->in_id, in);
+	}
 	g = agparent(g);
     }
 }
@@ -322,9 +324,11 @@ void agdeledgeimage(Agraph_t * g, Agedge_t * e, void *ignored)
     sn = agsubrep(g, t);
     del(g->e_seq, &sn->out_seq, out);
     del(g->e_id, &sn->out_id, out);
-    sn = agsubrep(g, h);
-    del(g->e_seq, &sn->in_seq, in);
-    del(g->e_id, &sn->in_id, in);
+	if (t != h) {
+		sn = agsubrep(g, h);
+		del(g->e_seq, &sn->in_seq, in);
+		del(g->e_id, &sn->in_id, in);
+	}
 #ifdef DEBUG
     for (e = agfstin(g,h); e; e = agnxtin(g,e))
 	assert(e != in);
