@@ -1,4 +1,4 @@
-/* $Id: write.c,v 1.8 2008/04/30 15:18:03 erg Exp $ $Revision: 1.8 $ */
+/* $Id: write.c,v 1.13 2009/02/11 17:06:39 arif Exp $ $Revision: 1.13 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -186,54 +186,55 @@ static void write_canonstr(Agraph_t * g, iochan_t * ofile, char *str)
 }
 
 static void write_dict(Agraph_t * g, iochan_t * ofile, char *name,
-		       Dict_t * dict)
+		       Dict_t * dict, int top)
 {
     int cnt = 0;
     Dict_t *view;
     Agsym_t *sym, *psym;
 
-    view = dtview(dict, NIL(Dict_t *));
+    if (!top) view = dtview(dict, NIL(Dict_t *));
+    else view = 0;
     for (sym = (Agsym_t *) dtfirst(dict); sym;
 	 sym = (Agsym_t *) dtnext(dict, sym)) {
-	if (EMPTY(sym->defval)) {	/* try to skip empty str (default) */
-	    if (view == NIL(Dict_t *))
-		continue;	/* no parent */
-	    psym = (Agsym_t *) dtsearch(view, sym);
-	    assert(psym);
-	    if (EMPTY(psym->defval))
-		continue;	/* also empty in parent */
-	}
-	if (cnt++ == 0) {
-	    indent(g, ofile);
-	    ioput(g, ofile, name);
-	    ioput(g, ofile, " [");
-	    Level++;
-	} else {
-	    ioput(g, ofile, ",\n");
-	    indent(g, ofile);
-	}
-	write_canonstr(g, ofile, sym->name);
-	ioput(g, ofile, "=");
-	write_canonstr(g, ofile, sym->defval);
+		if (EMPTY(sym->defval)) {	/* try to skip empty str (default) */
+			if (view == NIL(Dict_t *))
+			continue;	/* no parent */
+			psym = (Agsym_t *) dtsearch(view, sym);
+			assert(psym);
+			if (EMPTY(psym->defval))
+			continue;	/* also empty in parent */
+		}
+		if (cnt++ == 0) {
+			indent(g, ofile);
+			ioput(g, ofile, name);
+			ioput(g, ofile, " [");
+			Level++;
+		} else {
+			ioput(g, ofile, ",\n");
+			indent(g, ofile);
+		}
+		write_canonstr(g, ofile, sym->name);
+		ioput(g, ofile, "=");
+		write_canonstr(g, ofile, sym->defval);
     }
     if (cnt > 0) {
-	Level--;
-	if (cnt > 1) {
-	    ioput(g, ofile, "\n");
-	    indent(g, ofile);
+		Level--;
+		if (cnt > 1) {
+			ioput(g, ofile, "\n");
+			indent(g, ofile);
+		}
+		ioput(g, ofile, "];\n");
 	}
-	ioput(g, ofile, "];\n");
-    }
     dtview(dict, view);		/* restore previous view */
 }
 
-static void write_dicts(Agraph_t * g, iochan_t * ofile)
+static void write_dicts(Agraph_t * g, iochan_t * ofile, int top)
 {
     Agdatadict_t *def;
-    if ((def = agdatadict(g))) {
-	write_dict(g, ofile, "graph", def->dict.g);
-	write_dict(g, ofile, "node", def->dict.n);
-	write_dict(g, ofile, "edge", def->dict.e);
+    if ((def = agdatadict(g,FALSE))) {
+		write_dict(g, ofile, "graph", def->dict.g, top);
+		write_dict(g, ofile, "node", def->dict.n, top);
+		write_dict(g, ofile, "edge", def->dict.e, top);
     }
 }
 
@@ -274,7 +275,7 @@ static void write_hdr(Agraph_t * g, iochan_t * ofile, int top)
     ioput(g, ofile, sep);
     ioput(g, ofile, "{\n");
     Level++;
-    write_dicts(g, ofile);
+    write_dicts(g, ofile, top);
 	AGATTRWF(g) = TRUE;
 }
 
@@ -305,9 +306,9 @@ static int irrelevant_subgraph(Agraph_t * g)
 		&& strcmp(sdata->str[i], pdata->str[i]))
 		return FALSE;
     }
-    dd = agdatadict(g);
-    if ((dtsize(dd->dict.n) > 0) || (dtsize(dd->dict.e) > 0))
-	return FALSE;
+    dd = agdatadict(g,FALSE);
+	if (!dd) return TRUE;
+    if ((dtsize(dd->dict.n) > 0) || (dtsize(dd->dict.e) > 0)) return FALSE;
     return TRUE;
 }
 
@@ -401,37 +402,33 @@ static void write_nondefault_attrs(void *obj, iochan_t * ofile,
     int cnt = 0;
 
     if ((AGTYPE(obj) == AGINEDGE) || (AGTYPE(obj) == AGOUTEDGE)) {
-	if (write_edge_name(obj, ofile, FALSE))
-	    cnt++;
-    }
+		if (write_edge_name(obj, ofile, FALSE)) cnt++;
+	}
     data = agattrrec(obj);
     g = agraphof(obj);
-    if (data)
-	for (sym = (Agsym_t *) dtfirst(defdict); sym;
-	     sym = (Agsym_t *) dtnext(defdict, sym)) {
+    if (data) for (sym = (Agsym_t *) dtfirst(defdict); sym;
+	  sym = (Agsym_t *) dtnext(defdict, sym)) {
 	    if ((AGTYPE(obj) == AGINEDGE) || (AGTYPE(obj) == AGOUTEDGE)) {
-		if (Tailport && (sym->id == Tailport->id))
-		    continue;
-		if (Headport && (sym->id == Headport->id))
-		    continue;
+			if (Tailport && (sym->id == Tailport->id)) continue;
+			if (Headport && (sym->id == Headport->id)) continue;
 	    }
 	    if (data->str[sym->id] != sym->defval) {
-		if (cnt++ == 0) {
-		    indent(g, ofile);
-		    ioput(g, ofile, " [");
-		    Level++;
-		} else {
-		    ioput(g, ofile, ",\n");
-		    indent(g, ofile);
-		}
-		write_canonstr(g, ofile, sym->name);
-		ioput(g, ofile, "=");
-		write_canonstr(g, ofile, data->str[sym->id]);
+			if (cnt++ == 0) {
+				indent(g, ofile);
+				ioput(g, ofile, " [");
+				Level++;
+			} else {
+				ioput(g, ofile, ",\n");
+				indent(g, ofile);
+			}
+			write_canonstr(g, ofile, sym->name);
+			ioput(g, ofile, "=");
+			write_canonstr(g, ofile, data->str[sym->id]);
 	    }
 	}
     if (cnt > 0) {
-	ioput(g, ofile, "]");
-	Level--;
+		ioput(g, ofile, "]");
+		Level--;
     }
     AGATTRWF((Agobj_t *) obj) = TRUE;
 }
@@ -548,23 +545,26 @@ static void write_edge(Agedge_t * e, iochan_t * ofile, Dict_t * d)
 
 static void write_body(Agraph_t * g, iochan_t * ofile)
 {
-    Agnode_t *n;
+    Agnode_t *n, *prev;
     Agedge_t *e;
     Agdatadict_t *dd;
     /* int                  has_attr; */
 
     /* has_attr = (agattrrec(g) != NIL(Agattr_t*)); */
     write_subgs(g, ofile);
-    dd = agdatadict(g);
+    dd = agdatadict(agroot(g),FALSE);
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	if (write_node_test(g, n, AGSEQ(n)))
-	    write_node(n, ofile, dd->dict.n);
-	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
-	    if (write_node_test(g, e->node, AGSEQ(n)))
-		write_node(e->node, ofile, dd->dict.n);
-	    if (write_edge_test(g, e))
-		write_edge(e, ofile, dd->dict.e);
-	}
+		if (write_node_test(g, n, AGSEQ(n)))
+			write_node(n, ofile, dd? dd->dict.n : 0);
+		prev = n;
+		for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
+			if ((prev != aghead(e)) && write_node_test(g, aghead(e), AGSEQ(n)))  {
+				write_node(aghead(e), ofile, dd? dd->dict.n : 0);
+				prev = aghead(e);
+			}
+			if (write_edge_test(g, e))
+				write_edge(e, ofile, dd? dd->dict.e : 0);
+		}
     }
 }
 
