@@ -1,4 +1,4 @@
-/* $Id: arith.h,v 1.12 2008/05/15 14:51:09 ellson Exp $ $Revision: 1.12 $ */
+/* $Id: arith.h,v 1.14 2008/09/12 19:35:28 ellson Exp $ $Revision: 1.14 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -50,6 +50,10 @@ extern "C" {
 #endif
 #define ABS(a)		((a) >= 0 ? (a) : -(a))
 
+#define AVG(a,b)	((a + b) / 2)
+#define SGN(a)		(((a)<0)? -1 : 1)
+#define CMP(a,b)	(((a)<(b)) ? -1 : (((a)>(b)) ? 1 : 0))
+
 #ifndef INT_MAX
 #define	INT_MAX		((int)(~(unsigned)0 >> 1))
 #endif
