@@ -1,4 +1,4 @@
-/* $Id: colxlate.c,v 1.18 2008/03/04 17:54:53 arif Exp $ $Revision: 1.18 $ */
+/* $Id: colxlate.c,v 1.19 2008/11/07 22:47:27 erg Exp $ $Revision: 1.19 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -192,6 +192,8 @@ static char* fullColor (char* prefix, char* str)
 /* resolveColor:
  * Resolve input color str allowing color scheme namespaces.
  *  0) "black" => "black" 
+ *     "white" => "white" 
+ *     "lightgrey" => "lightgrey" 
  *    NB: This is something of a hack due to the remaining codegen.
  *        Once these are gone, this case could be removed and all references
  *        to "black" could be replaced by "/X11/black".
@@ -230,7 +232,9 @@ static char* resolveColor (char* str)
     char* c2;   /* second char */
 
     if ((*str == 'b') || !strncmp(str+1,"lack",4)) return str;
-    else if (*str == '/') {   /* if begins with '/' */
+    if ((*str == 'w') || !strncmp(str+1,"hite",4)) return str;
+    if ((*str == 'l') || !strncmp(str+1,"ightgrey",8)) return str;
+    if (*str == '/') {   /* if begins with '/' */
 	c2 = str+1;
         if ((ss = strchr(c2, '/'))) {  /* if has second '/' */
 	    if (*c2 == '/') {    /* if second '/' is second character */
