@@ -1,4 +1,4 @@
-/* $Id: diagen.c,v 1.25 2008/04/23 18:05:06 ellson Exp $ $Revision: 1.25 $ */
+/* $Id: diagen.c,v 1.33 2009/02/12 17:47:24 ellson Exp $ $Revision: 1.33 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -243,6 +243,20 @@ static pointf diapt(point p)
     return rv;
 }
 
+static pointf diaptf(pointf p)
+{
+    pointf rv;
+
+    if (Rot == 0) {
+	rv.x = PB.LL.x + p.x * Scale + Offset.x;
+	rv.y = PB.UR.y - 1 - p.y * Scale - Offset.y;
+    } else {
+	rv.x = PB.UR.x - 1 - p.y * Scale - Offset.x;
+	rv.y = PB.UR.y - 1 - p.x * Scale - Offset.y;
+    }
+    return rv;
+}
+
 static void dia_grstyle(context_t * cp)
 {
     if (strcmp(cp->pencolor, DEFAULT_COLOR)) {
@@ -290,8 +304,7 @@ static void dia_comment(char *str)
 }
 
 static void
-dia_begin_job(FILE * ofp, graph_t * g, char **lib, char *user,
-	      char *info[], point pages)
+dia_begin_job(FILE * ofp, graph_t * g, const char **lib, char *info[], point pages)
 {
 #if HAVE_LIBZ
     int fd;
@@ -591,7 +604,7 @@ static void dia_ellipse(point p, int rx, int ry, int filled)
 
     switch (Obj) {
     case NODE:
-	nodeId = Curnode->id;
+	nodeId = AGID(Curnode);
 	break;
     default:
 	nodeId = -1;
@@ -677,7 +690,7 @@ int box_connection(node_t * n, pointf p)
     double xsize, ysize, mindist2 = 0.0, dist2;
     polygon_t *poly;
     pointf P, *vertices;
-    static point *A;
+    static pointf *A;
     static int A_size;
 
     poly = (polygon_t *) ND_shape_info(n);
@@ -687,30 +700,27 @@ int box_connection(node_t * n, pointf p)
 
     if (A_size < sides) {
 	A_size = sides + 5;
-	A = ALLOC(A_size, A, point);
+	A = ALLOC(A_size, A, pointf);
     }
 
-    xsize = ((ND_lw_i(n) + ND_rw_i(n)) / POINTS(ND_width(n))) * 16.0;
-    ysize = ((ND_ht_i(n)) / POINTS(ND_height(n))) * 16.0;
+    xsize = (ND_lw(n) + ND_rw(n)) / POINTS(ND_width(n));
+    ysize = (ND_ht(n)) / POINTS(ND_height(n));
 
     for (j = 0; j < peripheries; j++) {
 	for (i = 0; i < sides; i++) {
 	    P = vertices[i + j * sides];
-/* simple rounding produces random results around .5 
- * this trick should clip off the random part. 
- * (note xsize/ysize prescaled by 16.0 above) */
-	    A[i].x = ROUND(P.x * xsize) / 16;
-	    A[i].y = ROUND(P.y * ysize) / 16;
+	    A[i].x = P.x * xsize;
+	    A[i].y = P.y * ysize;
 	    if (sides > 2) {
-		A[i].x += ND_coord_i(n).x;
-		A[i].y += ND_coord_i(n).y;
+		A[i].x += ND_coord(n).x;
+		A[i].y += ND_coord(n).y;
 	    }
 	}
     }
 
     z = 0;
     while (z < i) {
-	dist2 = DIST2(p, diapt(A[z]));
+	dist2 = DIST2(p, diaptf(A[z]));
 	if (z == 0) {
 	    mindist2 = dist2;
 	    conn = 0;
@@ -724,8 +734,8 @@ int box_connection(node_t * n, pointf p)
 
     z = 0;
     while (z < i) {
-	P.x = (diapt(A[z]).x + diapt(A[z + 1]).x) / 2;
-	P.y = (diapt(A[z]).y + diapt(A[z + 1]).y) / 2;
+	P.x = (diaptf(A[z]).x + diaptf(A[z + 1]).x) / 2;
+	P.y = (diaptf(A[z]).y + diaptf(A[z + 1]).y) / 2;
 	dist2 = DIST2(p, P);
 	if (dist2 < mindist2) {
 	    mindist2 = dist2;
@@ -742,9 +752,7 @@ static void
 dia_bezier(point * A, int n, int arrow_at_start, int arrow_at_end, int filled)
 {
     int i, conn_h, conn_t;
-    pointf p, firstp = { 0, 0 }, llp = {
-    0, 0}, urp = {
-    0, 0};
+    pointf p, firstp = { 0, 0 }, llp = { 0, 0}, urp = { 0, 0};
     node_t *head, *tail;
     char *shape_t;
     pointf cp_h, cp_t;
@@ -781,8 +789,8 @@ dia_bezier(point * A, int n, int arrow_at_start, int arrow_at_end, int filled)
     if (Curedge) {
         conn_h = conn_t = -1;
 
-        head = Curedge->head;
-        tail = Curedge->tail;
+        head = aghead(Curedge);
+        tail = agtail(Curedge);
 
         shape_t = ND_shape(tail)->name;
 
@@ -813,23 +821,26 @@ dia_bezier(point * A, int n, int arrow_at_start, int arrow_at_end, int filled)
         }
     
         dia_fputs("      <dia:attribute name=\"conn_endpoints\">\n");
-        dia_printf("        <dia:point val=\"%g,%g\"/>\n", diapt(A[0]).x,
-	           diapt(A[0]).y);
-        dia_printf("        <dia:point val=\"%g,%g\"/>\n", diapt(A[n - 1]).x,
-	           diapt(A[n - 1]).y);
+        dia_printf("        <dia:point val=\"%g,%g\"/>\n",
+		diapt(A[0]).x, diapt(A[0]).y);
+        dia_printf("        <dia:point val=\"%g,%g\"/>\n",
+		diapt(A[n - 1]).x, diapt(A[n - 1]).y);
         dia_fputs("      </dia:attribute>\n");
         dia_fputs("      <dia:connections>\n");
     
+/* FIXME !!! - What is this crap!   It should just user the arrow vector. */
         if ((strcmp(shape_t, "ellipse") == 0)
 	    || (strcmp(shape_t, "circle") == 0)
 	    || (strcmp(shape_t, "doublecircle") == 0)) {
-	    cp_h = diapt(ND_coord_i(head));
-	    if (AG_IS_DIRECTED(Rootgraph))
+	    cp_h = diaptf(ND_coord(head));
+	    if (agisdirected(Rootgraph))
+
 	        conn_h = ellipse_connection(cp_h, diapt(A[n - 1]));
 	    else
 	        conn_h = ellipse_connection(cp_h, diapt(A[0]));
+        } else if (strcmp(shape_t, "record") == 0) { 
         } else {
-	    if (AG_IS_DIRECTED(Rootgraph))
+	    if (agisdirected(Rootgraph))
 	        conn_h = box_connection(head, diapt(A[n - 1]));
 	    else
 	        conn_h = box_connection(head, diapt(A[0]));
@@ -838,13 +849,14 @@ dia_bezier(point * A, int n, int arrow_at_start, int arrow_at_end, int filled)
         if ((strcmp(shape_t, "ellipse") == 0)
 	    || (strcmp(shape_t, "circle") == 0)
 	    || (strcmp(shape_t, "doublecircle") == 0)) {
-	    cp_t = diapt(ND_coord_i(tail));
-	    if (AG_IS_DIRECTED(Rootgraph))
+	    cp_t = diaptf(ND_coord(tail));
+	    if (agisdirected(Rootgraph))
 	        conn_t = ellipse_connection(cp_t, diapt(A[0]));
 	    else
 	        conn_t = ellipse_connection(cp_t, diapt(A[n - 1]));
+        } else if (strcmp(shape_t, "record") == 0) {
         } else {
-	    if (AG_IS_DIRECTED(Rootgraph))
+	    if (agisdirected(Rootgraph))
 	        conn_t = box_connection(tail, diapt(A[0]));
 	    else
 	        conn_t = box_connection(tail, diapt(A[n - 1]));
@@ -853,17 +865,17 @@ dia_bezier(point * A, int n, int arrow_at_start, int arrow_at_end, int filled)
         if (arrow_at_start) {
 	    dia_printf
 	        ("        <dia:connection handle=\"0\" to=\"%d\" connection=\"%d\"/>\n",
-	         head->id, conn_h);
+	         AGID(head), conn_h);
 	    dia_printf
 	        ("        <dia:connection handle=\"%d\" to=\"%d\" connection=\"%d\"/>\n",
-	         (n - 1), tail->id, conn_t);
+	         (n - 1), AGID(tail), conn_t);
         } else {
 	    dia_printf
 	        ("        <dia:connection handle=\"0\" to=\"%d\" connection=\"%d\"/>\n",
-	         tail->id, conn_t);
+	         AGID(tail), conn_t);
 	    dia_printf
 	        ("        <dia:connection handle=\"%d\" to=\"%d\" connection=\"%d\"/>\n",
-	         (n - 1), head->id, conn_h);
+	         (n - 1), AGID(head), conn_h);
         }
     
         dia_fputs("      </dia:connections>\n");
@@ -889,7 +901,7 @@ static void dia_polygon(point * A, int n, int filled)
     case NODE:
 	dia_printf
 	    ("    <dia:object type=\"Standard - Polygon\" version=\"0\" id=\"%d\">\n",
-	     Curnode->id);
+	     AGID(Curnode));
 	break;
     case EDGE:
 	return;
@@ -897,7 +909,7 @@ static void dia_polygon(point * A, int n, int filled)
     case CLST:
 	dia_printf
 	    ("    <dia:object type=\"Standard - Polygon\" version=\"0\" id=\"%s\">\n",
-	     Curgraph->name);
+	     agnameof(Curgraph));
 	break;
     default:
 	dia_printf
