@@ -1,4 +1,4 @@
-/* $Id: hpglgen.c,v 1.16 2006/12/07 22:49:35 erg Exp $ $Revision: 1.16 $ */
+/* $Id: hpglgen.c,v 1.18 2009/02/12 17:47:24 ellson Exp $ $Revision: 1.18 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -397,8 +397,7 @@ static void hpgl_reset(void)
 }
 
 static void
-hpgl_begin_job(FILE * ofp, graph_t * g, char **lib, char *user,
-	       char *info[], point pages)
+hpgl_begin_job(FILE * ofp, graph_t * g, const char **lib, char *info[], point pages)
 {
     /* Pages = pages; */
     N_pages = pages.x * pages.y;
