@@ -1,4 +1,4 @@
-/* $Id: logic.h,v 1.5 2006/01/28 00:20:45 ellson Exp $ $Revision: 1.5 $ */
+/* $Id: logic.h,v 1.8 2008/06/27 17:53:03 erg Exp $ $Revision: 1.8 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -25,20 +25,24 @@ extern "C" {
 #include <stdbool.h>
 #endif
 
+#ifndef NOT
 #define NOT(v) (!(v))
+#endif
 
+#if 0
 /* HAVE_BOOL only tested the CC compiler, we know C++ must define bool */
 #if ! defined HAVE_BOOL && ! defined __cplusplus
 typedef unsigned char bool;
 #define false 0
 #define true NOT(false)
 #endif
+#endif
 
 #ifndef FALSE
-#define	FALSE false
+#define	FALSE (0)
 #endif
 #ifndef TRUE
-#define TRUE true
+#define TRUE (!FALSE)
 #endif
 
 #ifndef NOTUSED
