@@ -1,4 +1,4 @@
-/* $Id: memory.h,v 1.3 2005/10/18 18:50:26 ellson Exp $ $Revision: 1.3 $ */
+/* $Id: memory.h,v 1.5 2009/01/21 16:47:00 arif Exp $ $Revision: 1.5 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -38,17 +38,32 @@ extern "C" {
 #define NEW(t)           (t*)zmalloc(sizeof(t))
 #define N_NEW(n,t)       (t*)zmalloc((n)*sizeof(t))
 #define GNEW(t)          (t*)gmalloc(sizeof(t))
+
 #define N_GNEW(n,t)      (t*)gmalloc((n)*sizeof(t))
+#define N_GGNEW(n,t)      (t*)malloc((n)*sizeof(t))
 #define ALLOC(size,ptr,type) (ptr? (type*)grealloc(ptr,(size)*sizeof(type)):(type*)gmalloc((size)*sizeof(type)))
 #define RALLOC(size,ptr,type) ((type*)grealloc(ptr,(size)*sizeof(type)))
 #define ZALLOC(size,ptr,type,osize) (ptr? (type*)zrealloc(ptr,size,sizeof(type),osize):(type*)zmalloc((size)*sizeof(type)))
 #endif
+#ifdef GVDLL
+#define extern __declspec(dllexport)
+#else
+#define extern
+#endif
 
+/*visual studio*/
+#ifdef WIN32_DLL
+#ifndef GVC_EXPORTS
+#define extern __declspec(dllimport)
+#endif
+#endif
+/*end visual studio*/
 
     extern void *zmalloc(size_t);
     extern void *zrealloc(void *, size_t, size_t, size_t);
     extern void *gmalloc(size_t);
-    extern void *grealloc(void *, size_t);
+	extern void *grealloc(void *, size_t);
+#undef extern
 
 #ifdef __cplusplus
 }
