@@ -1,4 +1,4 @@
-/* $Id: ns.c,v 1.12 2007/06/25 17:49:29 erg Exp $ $Revision: 1.12 $ */
+/* $Id: ns.c,v 1.18 2008/10/15 12:57:08 ellson Exp $ $Revision: 1.18 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -29,7 +29,7 @@ static int x_val(edge_t * e, node_t * v, int dir);
 static void check_cycles(graph_t * g);
 #endif
 
-#define LENGTH(e)		(ND_rank(e->head) - ND_rank(e->tail))
+#define LENGTH(e)		(ND_rank(aghead(e)) - ND_rank(agtail(e)))
 #define SLACK(e)		(LENGTH(e) - ED_minlen(e))
 #define SEQ(a,b,c)		(((a) <= (b)) && ((b) <= (c)))
 #define TREE_EDGE(e)	(ED_tree_index(e) >= 0)
@@ -50,17 +50,17 @@ static void add_tree_edge(edge_t * e)
 	abort();
     ED_tree_index(e) = Tree_edge.size;
     Tree_edge.list[Tree_edge.size++] = e;
-    if (ND_mark(e->tail) == FALSE)
-	Tree_node.list[Tree_node.size++] = e->tail;
-    if (ND_mark(e->head) == FALSE)
-	Tree_node.list[Tree_node.size++] = e->head;
-    n = e->tail;
+    if (ND_mark(agtail(e)) == FALSE)
+	Tree_node.list[Tree_node.size++] = agtail(e);
+    if (ND_mark(aghead(e)) == FALSE)
+	Tree_node.list[Tree_node.size++] = aghead(e);
+    n = agtail(e);
     ND_mark(n) = TRUE;
     ND_tree_out(n).list[ND_tree_out(n).size++] = e;
     ND_tree_out(n).list[ND_tree_out(n).size] = NULL;
     if (ND_out(n).list[ND_tree_out(n).size - 1] == 0)
 	abort();
-    n = e->head;
+    n = aghead(e);
     ND_mark(n) = TRUE;
     ND_tree_in(n).list[ND_tree_in(n).size++] = e;
     ND_tree_in(n).list[ND_tree_in(n).size] = NULL;
@@ -77,14 +77,14 @@ static void exchange_tree_edges(edge_t * e, edge_t * f)
     Tree_edge.list[ED_tree_index(e)] = f;
     ED_tree_index(e) = -1;
 
-    n = e->tail;
+    n = agtail(e);
     i = --(ND_tree_out(n).size);
     for (j = 0; j <= i; j++)
 	if (ND_tree_out(n).list[j] == e)
 	    break;
     ND_tree_out(n).list[j] = ND_tree_out(n).list[i];
     ND_tree_out(n).list[i] = NULL;
-    n = e->head;
+    n = aghead(e);
     i = --(ND_tree_in(n).size);
     for (j = 0; j <= i; j++)
 	if (ND_tree_in(n).list[j] == e)
@@ -92,10 +92,10 @@ static void exchange_tree_edges(edge_t * e, edge_t * f)
     ND_tree_in(n).list[j] = ND_tree_in(n).list[i];
     ND_tree_in(n).list[i] = NULL;
 
-    n = f->tail;
+    n = agtail(f);
     ND_tree_out(n).list[ND_tree_out(n).size++] = f;
     ND_tree_out(n).list[ND_tree_out(n).size] = NULL;
-    n = f->head;
+    n = aghead(f);
     ND_tree_in(n).list[ND_tree_in(n).size++] = f;
     ND_tree_in(n).list[ND_tree_in(n).size] = NULL;
 }
@@ -120,29 +120,29 @@ void init_rank(void)
 	ND_rank(v) = 0;
 	ctr++;
 	for (i = 0; (e = ND_in(v).list[i]); i++)
-	    ND_rank(v) = MAX(ND_rank(v), ND_rank(e->tail) + ED_minlen(e));
+	    ND_rank(v) = MAX(ND_rank(v), ND_rank(agtail(e)) + ED_minlen(e));
 	for (i = 0; (e = ND_out(v).list[i]); i++) {
-	    if (--(ND_priority(e->head)) <= 0)
-		enqueue(Q, e->head);
+	    if (--(ND_priority(aghead(e))) <= 0)
+		enqueue(Q, aghead(e));
 	}
     }
     if (ctr != N_nodes) {
 	agerr(AGERR, "trouble in init_rank\n");
 	for (v = GD_nlist(G); v; v = ND_next(v))
 	    if (ND_priority(v))
-		agerr(AGPREV, "\t%s %d\n", v->name, ND_priority(v));
+		agerr(AGPREV, "\t%s %d\n", agnameof(v), ND_priority(v));
     }
     free_queue(Q);
 }
 
 static node_t *incident(edge_t * e)
 {
-    if (ND_mark(e->tail)) {
-	if (ND_mark(e->head) == FALSE)
-	    return e->tail;
+    if (ND_mark(agtail(e))) {
+	if (ND_mark(aghead(e)) == FALSE)
+	    return agtail(e);
     } else {
-	if (ND_mark(e->head))
-	    return e->head;
+	if (ND_mark(aghead(e)))
+	    return aghead(e);
     }
     return NULL;
 }
@@ -154,7 +154,7 @@ static edge_t *leave_edge(void)
 
     j = S_i;
     while (S_i < Tree_edge.size) {
-	if ((f = Tree_edge.list[S_i])->u.cutvalue < 0) {
+	if (ED_cutvalue(f = Tree_edge.list[S_i]) < 0) {
 	    if (rv) {
 		if (ED_cutvalue(rv) > ED_cutvalue(f))
 		    rv = f;
@@ -168,7 +168,7 @@ static edge_t *leave_edge(void)
     if (j > 0) {
 	S_i = 0;
 	while (S_i < j) {
-	    if ((f = Tree_edge.list[S_i])->u.cutvalue < 0) {
+	    if (ED_cutvalue(f = Tree_edge.list[S_i]) < 0) {
 		if (rv) {
 		    if (ED_cutvalue(rv) > ED_cutvalue(f))
 			rv = f;
@@ -193,19 +193,19 @@ static void dfs_enter_outedge(node_t * v)
 
     for (i = 0; (e = ND_out(v).list[i]); i++) {
 	if (TREE_EDGE(e) == FALSE) {
-	    if (!SEQ(Low, ND_lim(e->head), Lim)) {
+	    if (!SEQ(Low, ND_lim(aghead(e)), Lim)) {
 		slack = SLACK(e);
 		if ((slack < Slack) || (Enter == NULL)) {
 		    Enter = e;
 		    Slack = slack;
 		}
 	    }
-	} else if (ND_lim(e->head) < ND_lim(v))
-	    dfs_enter_outedge(e->head);
+	} else if (ND_lim(aghead(e)) < ND_lim(v))
+	    dfs_enter_outedge(aghead(e));
     }
     for (i = 0; (e = ND_tree_in(v).list[i]) && (Slack > 0); i++)
-	if (ND_lim(e->tail) < ND_lim(v))
-	    dfs_enter_outedge(e->tail);
+	if (ND_lim(agtail(e)) < ND_lim(v))
+	    dfs_enter_outedge(agtail(e));
 }
 
 static void dfs_enter_inedge(node_t * v)
@@ -215,19 +215,19 @@ static void dfs_enter_inedge(node_t * v)
 
     for (i = 0; (e = ND_in(v).list[i]); i++) {
 	if (TREE_EDGE(e) == FALSE) {
-	    if (!SEQ(Low, ND_lim(e->tail), Lim)) {
+	    if (!SEQ(Low, ND_lim(agtail(e)), Lim)) {
 		slack = SLACK(e);
 		if ((slack < Slack) || (Enter == NULL)) {
 		    Enter = e;
 		    Slack = slack;
 		}
 	    }
-	} else if (ND_lim(e->tail) < ND_lim(v))
-	    dfs_enter_inedge(e->tail);
+	} else if (ND_lim(agtail(e)) < ND_lim(v))
+	    dfs_enter_inedge(agtail(e));
     }
     for (i = 0; (e = ND_tree_out(v).list[i]) && (Slack > 0); i++)
-	if (ND_lim(e->head) < ND_lim(v))
-	    dfs_enter_inedge(e->head);
+	if (ND_lim(aghead(e)) < ND_lim(v))
+	    dfs_enter_inedge(aghead(e));
 }
 
 static edge_t *enter_edge(edge_t * e)
@@ -236,11 +236,11 @@ static edge_t *enter_edge(edge_t * e)
     int outsearch;
 
     /* v is the down node */
-    if (ND_lim(e->tail) < ND_lim(e->head)) {
-	v = e->tail;
+    if (ND_lim(agtail(e)) < ND_lim(aghead(e))) {
+	v = agtail(e);
 	outsearch = FALSE;
     } else {
-	v = e->head;
+	v = aghead(e);
 	outsearch = TRUE;
     }
     Enter = NULL;
@@ -260,16 +260,16 @@ static int treesearch(node_t * v)
     edge_t *e;
 
     for (i = 0; (e = ND_out(v).list[i]); i++) {
-	if ((ND_mark(e->head) == FALSE) && (SLACK(e) == 0)) {
+	if ((ND_mark(aghead(e)) == FALSE) && (SLACK(e) == 0)) {
 	    add_tree_edge(e);
-	    if ((Tree_edge.size == N_nodes - 1) || treesearch(e->head))
+	    if ((Tree_edge.size == N_nodes - 1) || treesearch(aghead(e)))
 		return TRUE;
 	}
     }
     for (i = 0; (e = ND_in(v).list[i]); i++) {
-	if ((ND_mark(e->tail) == FALSE) && (SLACK(e) == 0)) {
+	if ((ND_mark(agtail(e)) == FALSE) && (SLACK(e) == 0)) {
 	    add_tree_edge(e);
-	    if ((Tree_edge.size == N_nodes - 1) || treesearch(e->tail))
+	    if ((Tree_edge.size == N_nodes - 1) || treesearch(agtail(e)))
 		return TRUE;
 	}
     }
@@ -287,7 +287,7 @@ static int tight_tree(void)
 	ND_tree_in(n).size = ND_tree_out(n).size = 0;
     }
     for (i = 0; i < Tree_edge.size; i++)
-	Tree_edge.list[i]->u.tree_index = -1;
+	ED_tree_index(Tree_edge.list[i]) = -1;
 
     Tree_node.size = Tree_edge.size = 0;
     for (n = GD_nlist(G); n && (Tree_edge.size == 0); n = ND_next(n))
@@ -324,10 +324,10 @@ static int feasible_tree(void)
 	if (e) {
 	    delta = SLACK(e);
 	    if (delta) {
-		if (incident(e) == e->head)
+		if (incident(e) == aghead(e))
 		    delta = -delta;
 		for (i = 0; i < Tree_node.size; i++)
-		    Tree_node.list[i]->u.rank += delta;
+		    ND_rank(Tree_node.list[i]) += delta;
 	    }
 	} else {
 #ifdef DEBUG
@@ -355,7 +355,7 @@ static node_t *treeupdate(node_t * v, node_t * w, int cutvalue, int dir)
 
     while (!SEQ(ND_low(v), ND_lim(w), ND_lim(v))) {
 	e = ND_par(v);
-	if (v == e->tail)
+	if (v == agtail(e))
 	    d = dir;
 	else
 	    d = NOT(dir);
@@ -363,10 +363,10 @@ static node_t *treeupdate(node_t * v, node_t * w, int cutvalue, int dir)
 	    ED_cutvalue(e) += cutvalue;
 	else
 	    ED_cutvalue(e) -= cutvalue;
-	if (ND_lim(e->tail) > ND_lim(e->head))
-	    v = e->tail;
+	if (ND_lim(agtail(e)) > ND_lim(aghead(e)))
+	    v = agtail(e);
 	else
-	    v = e->head;
+	    v = aghead(e);
     }
     return v;
 }
@@ -379,10 +379,10 @@ static void rerank(node_t * v, int delta)
     ND_rank(v) -= delta;
     for (i = 0; (e = ND_tree_out(v).list[i]); i++)
 	if (e != ND_par(v))
-	    rerank(e->head, delta);
+	    rerank(aghead(e), delta);
     for (i = 0; (e = ND_tree_in(v).list[i]); i++)
 	if (e != ND_par(v))
-	    rerank(e->tail, delta);
+	    rerank(agtail(e), delta);
 }
 
 /* e is the tree edge that is leaving and f is the nontree edge that
@@ -398,25 +398,25 @@ update(edge_t * e, edge_t * f)
     /* "for (v = in nodes in tail side of e) do ND_rank(v) -= delta;" */
     if (delta > 0) {
 	int s;
-	s = ND_tree_in(e->tail).size + ND_tree_out(e->tail).size;
+	s = ND_tree_in(agtail(e)).size + ND_tree_out(agtail(e)).size;
 	if (s == 1)
-	    rerank(e->tail, delta);
+	    rerank(agtail(e), delta);
 	else {
-	    s = ND_tree_in(e->head).size + ND_tree_out(e->head).size;
+	    s = ND_tree_in(aghead(e)).size + ND_tree_out(aghead(e)).size;
 	    if (s == 1)
-		rerank(e->head, -delta);
+		rerank(aghead(e), -delta);
 	    else {
-		if (ND_lim(e->tail) < ND_lim(e->head))
-		    rerank(e->tail, delta);
+		if (ND_lim(agtail(e)) < ND_lim(aghead(e)))
+		    rerank(agtail(e), delta);
 		else
-		    rerank(e->head, -delta);
+		    rerank(aghead(e), -delta);
 	    }
 	}
     }
 
     cutvalue = ED_cutvalue(e);
-    lca = treeupdate(f->tail, f->head, cutvalue, 1);
-    if (treeupdate(f->head, f->tail, cutvalue, 0) != lca)
+    lca = treeupdate(agtail(f), aghead(f), cutvalue, 1);
+    if (treeupdate(aghead(f), agtail(f), cutvalue, 0) != lca)
 	abort();
     ED_cutvalue(f) = -cutvalue;
     ED_cutvalue(e) = 0;
@@ -469,10 +469,10 @@ static void LR_balance(void)
 	    delta = SLACK(f);
 	    if (delta <= 1)
 		continue;
-	    if (ND_lim(e->tail) < ND_lim(e->head))
-		rerank(e->tail, delta / 2);
+	    if (ND_lim(agtail(e)) < ND_lim(aghead(e)))
+		rerank(agtail(e), delta / 2);
 	    else
-		rerank(e->head, -delta / 2);
+		rerank(aghead(e), -delta / 2);
 	}
     }
     freeTreeList (G);
@@ -502,11 +502,11 @@ static void TB_balance(void)
 	high = Maxrank;
 	for (i = 0; (e = ND_in(n).list[i]); i++) {
 	    inweight += ED_weight(e);
-	    low = MAX(low, ND_rank(e->tail) + ED_minlen(e));
+	    low = MAX(low, ND_rank(agtail(e)) + ED_minlen(e));
 	}
 	for (i = 0; (e = ND_out(n).list[i]); i++) {
 	    outweight += ED_weight(e);
-	    high = MIN(high, ND_rank(e->head) - ED_minlen(e));
+	    high = MIN(high, ND_rank(aghead(e)) - ED_minlen(e));
 	}
 	if (low < 0)
 	    low = 0;		/* vnodes can have ranks < 0 */
@@ -554,7 +554,7 @@ static int init_graph(graph_t * g)
 	    ED_cutvalue(e) = 0;
 	    ED_tree_index(e) = -1;
 	    if (feasible
-		&& (ND_rank(e->head) - ND_rank(e->tail) < ED_minlen(e)))
+		&& (ND_rank(aghead(e)) - ND_rank(agtail(e)) < ED_minlen(e)))
 		feasible = FALSE;
 	}
 	ND_tree_in(n).list = N_NEW(i + 1, edge_t *);
@@ -566,6 +566,27 @@ static int init_graph(graph_t * g)
     return feasible;
 }
 
+/* graphSize:
+ * Compute no. of nodes and edges in the graph
+ */
+static void
+graphSize (graph_t * g, int* nn, int* ne)
+{
+    int i, nnodes, nedges;
+    node_t *n;
+    edge_t *e;
+   
+    nnodes = nedges = 0;
+    for (n = GD_nlist(g); n; n = ND_next(n)) {
+	nnodes++;
+	for (i = 0; (e = ND_out(n).list[i]); i++) {
+	    nedges++;
+	}
+    }
+    *nn = nnodes;
+    *ne = nedges;
+}
+
 /* rank:
  * Apply network simplex to rank the nodes in a graph.
  * Uses ED_minlen as the internode constraint: if a->b with minlen=ml,
@@ -587,8 +608,13 @@ int rank(graph_t * g, int balance, int maxiter)
 #ifdef DEBUG
     check_cycles(g);
 #endif
-    if (Verbose)
+    if (Verbose) {
+	int nn, ne;
+	graphSize (g, &nn, &ne);
+	fprintf(stderr, "%s %d nodes %d edges maxiter=%d balance=%d\n", ns,
+	    nn, ne, maxiter, balance);
 	start_timer();
+    }
     feasible = init_graph(g);
     if (!feasible)
 	init_rank();
@@ -648,11 +674,11 @@ static void x_cutval(edge_t * f)
     int i, sum, dir;
 
     /* set v to the node on the side of the edge already searched */
-    if (ND_par(f->tail) == f) {
-	v = f->tail;
+    if (ND_par(agtail(f)) == f) {
+	v = agtail(f);
 	dir = 1;
     } else {
-	v = f->head;
+	v = aghead(f);
 	dir = -1;
     }
 
@@ -669,10 +695,10 @@ static int x_val(edge_t * e, node_t * v, int dir)
     node_t *other;
     int d, rv, f;
 
-    if (e->tail == v)
-	other = e->head;
+    if (agtail(e) == v)
+	other = aghead(e);
     else
-	other = e->tail;
+	other = agtail(e);
     if (!(SEQ(ND_low(v), ND_lim(other), ND_lim(v)))) {
 	f = 1;
 	rv = ED_weight(e);
@@ -685,12 +711,12 @@ static int x_val(edge_t * e, node_t * v, int dir)
 	rv -= ED_weight(e);
     }
     if (dir > 0) {
-	if (e->head == v)
+	if (aghead(e) == v)
 	    d = 1;
 	else
 	    d = -1;
     } else {
-	if (e->tail == v)
+	if (agtail(e) == v)
 	    d = 1;
 	else
 	    d = -1;
@@ -709,10 +735,10 @@ static void dfs_cutval(node_t * v, edge_t * par)
 
     for (i = 0; (e = ND_tree_out(v).list[i]); i++)
 	if (e != par)
-	    dfs_cutval(e->head, e);
+	    dfs_cutval(aghead(e), e);
     for (i = 0; (e = ND_tree_in(v).list[i]); i++)
 	if (e != par)
-	    dfs_cutval(e->tail, e);
+	    dfs_cutval(agtail(e), e);
     if (par)
 	x_cutval(par);
 }
@@ -727,10 +753,10 @@ static int dfs_range(node_t * v, edge_t * par, int low)
     ND_low(v) = low;
     for (i = 0; (e = ND_tree_out(v).list[i]); i++)
 	if (e != par)
-	    lim = dfs_range(e->head, e, lim);
+	    lim = dfs_range(aghead(e), e, lim);
     for (i = 0; (e = ND_tree_in(v).list[i]); i++)
 	if (e != par)
-	    lim = dfs_range(e->tail, e, lim);
+	    lim = dfs_range(agtail(e), e, lim);
     ND_lim(v) = lim;
     return lim + 1;
 }
@@ -749,11 +775,11 @@ void tchk(void)
 	for (i = 0; (e = ND_tree_out(n).list[i]); i++) {
 	    e_cnt++;
 	    if (SLACK(e) > 0)
-		printf("not a tight tree %x", e);
+		fprintf(stderr, "not a tight tree %lx", (unsigned long int)e);
 	}
     }
     if ((n_cnt != Tree_node.size) || (e_cnt != Tree_edge.size))
-	printf("something missing\n");
+	fprintf(stderr, "something missing\n");
 }
 
 void check_cutvalues(void)
@@ -781,7 +807,7 @@ int check_ranks(void)
     for (n = GD_nlist(G); n; n = ND_next(n)) {
 	for (i = 0; (e = ND_out(n).list[i]); i++) {
 	    cost += (ED_weight(e)) * abs(LENGTH(e));
-	    if (ND_rank(e->head) - ND_rank(e->tail) - ED_minlen(e) < 0)
+	    if (ND_rank(aghead(e)) - ND_rank(agtail(e)) - ED_minlen(e) < 0)
 		abort();
 	}
     }
@@ -805,13 +831,13 @@ void checktree(void)
 	if (i != ND_tree_in(v).size)
 	    abort();
     }
-    printf("%d %d %d\n", Tree_edge.size, n, m);
+    fprintf(stderr, "%d %d %d\n", Tree_edge.size, n, m);
 }
 
 void check_fast_node(node_t * n)
 {
     node_t *nptr;
-    nptr = GD_nlist(n->graph);
+    nptr = GD_nlist(agraphof(n));
     while (nptr && nptr != n)
 	nptr = ND_next(nptr);
     assert(nptr != NULL);
@@ -828,17 +854,21 @@ static node_t *checkdfs(node_t * n)
     ND_mark(n) = TRUE;
     ND_onstack(n) = TRUE;
     for (i = 0; (e = ND_out(n).list[i]); i++) {
-	w = e->head;
+	w = aghead(e);
 	if (ND_onstack(w)) {
-	    fprintf(stderr, "cycle: last edge %x %s(%x) %s(%x)\n",
-		e,n->name,n,w->name,w);
+	    fprintf(stderr, "cycle: last edge %lx %s(%lx) %s(%lx)\n",
+		(unsigned long int)e,
+	       	agnameof(n), (unsigned long int)n,
+		agnameof(w), (unsigned long int)w);
 	    return w;
 	}
 	else {
 	    if (ND_mark(w) == FALSE) {
 		x = checkdfs(w);
 		if (x) {
-		    fprintf(stderr,"unwind %x %s(%x)\n",e,n->name,n);
+		    fprintf(stderr,"unwind %lx %s(%lx)\n",
+			(unsigned long int)e,
+			agnameof(n), (unsigned long int)n);
 		    if (x != n) return x;
 		    fprintf(stderr,"unwound to root\n");
 		    fflush(stderr);
