@@ -1,4 +1,4 @@
-/* $Id: pointset.h,v 1.3 2008/01/03 23:24:28 ellson Exp $ $Revision: 1.3 $ */
+/* $Id: pointset.h,v 1.6 2009/01/21 16:47:00 arif Exp $ $Revision: 1.6 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -14,21 +14,30 @@
 *              AT&T Research, Florham Park NJ             *
 **********************************************************/
 
+#ifndef _POINTSET_H
+#define _POINTSET_H 1
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-
-
-#ifndef _POINTSET_H
-#define _POINTSET_H 1
-
-#include "render.h"
-
     typedef Dict_t PointSet;
     typedef Dict_t PointMap;
+#ifdef GVDLL
+#define extern __declspec(dllexport)
+#else
+#define extern
+#endif
 
-    extern PointSet *newPS(void);
+/*visual studio*/
+#ifdef WIN32_DLL
+#ifndef GVC_EXPORTS
+#define extern __declspec(dllimport)
+#endif
+#endif
+/*end visual studio*/
+
+	extern PointSet *newPS(void);
     extern void freePS(PointSet *);
     extern void insertPS(PointSet *, point);
     extern void addPS(PointSet *, int, int);
@@ -41,9 +50,10 @@ extern "C" {
     extern void clearPM(PointMap *);
     extern void freePM(PointMap *);
     extern int insertPM(PointMap *, int, int, int);
-#endif
-
 
+#undef extern
 #ifdef __cplusplus
 }
 #endif
+
+#endif /* _POINTSET_H */
