@@ -1,4 +1,4 @@
-/* $Id: postproc.c,v 1.13 2007/12/12 03:31:34 erg Exp $ $Revision: 1.13 $ */
+/* $Id: postproc.c,v 1.27 2009/01/06 22:58:41 arif Exp $ $Revision: 1.27 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -17,26 +17,25 @@
 
 #include "render.h"
 
-
 static int Rankdir;
 static boolean Flip;
-static point Offset;
+static pointf Offset;
 
 static void place_flip_graph_label(graph_t * g);
 
 #define M1 \
 "/pathbox {\n\
-    /Y exch %d sub def\n\
-    /X exch %d sub def\n\
-    /y exch %d sub def\n\
-    /x exch %d sub def\n\
+    /Y exch %.5g sub def\n\
+    /X exch %.5g sub def\n\
+    /y exch %.5g sub def\n\
+    /x exch %.5g sub def\n\
     newpath x y moveto\n\
     X y lineto\n\
     X Y lineto\n\
     x Y lineto\n\
     closepath stroke\n \
 } def\n\
-/dbgstart { gsave %d %d translate } def\n\
+/dbgstart { gsave %.5g %.5g translate } def\n\
 /arrowlength 10 def\n\
 /arrowwidth arrowlength 2 div def\n\
 /arrowhead {\n\
@@ -68,12 +67,13 @@ static void place_flip_graph_label(graph_t * g);
     X Y moveto\n\
     x y makearrow\n\
 } def\n"
+
 #define M2 \
 "/pathbox {\n\
-    /X exch neg %d sub def\n\
-    /Y exch %d sub def\n\
-    /x exch neg %d sub def\n\
-    /y exch %d sub def\n\
+    /X exch neg %.5g sub def\n\
+    /Y exch %.5g sub def\n\
+    /x exch neg %.5g sub def\n\
+    /y exch %.5g sub def\n\
     newpath x y moveto\n\
     X y lineto\n\
     X Y lineto\n\
@@ -81,9 +81,9 @@ static void place_flip_graph_label(graph_t * g);
     closepath stroke\n\
 } def\n"
 
-static point map_point(point p)
+static pointf map_point(pointf p)
 {
-    p = ccwrotatep(p, Rankdir*90);
+    p = ccwrotatepf(p, Rankdir*90);
     p.x -= Offset.x;
     p.y -= Offset.y;
     return p;
@@ -96,8 +96,8 @@ static void map_edge(edge_t * e)
 
     if (ED_spl(e) == NULL) {
 	if ((Concentrate == FALSE) || (ED_edge_type(e) != IGNORED))
-	    agerr(AGERR, "lost %s %s edge\n", e->tail->name,
-		  e->head->name);
+	    agerr(AGERR, "lost %s %s edge\n",agnameof(agtail(e)),
+		  agnameof(aghead(e)));
 	return;
     }
     for (j = 0; j < ED_spl(e)->size; j++) {
@@ -110,61 +110,39 @@ static void map_edge(edge_t * e)
 	    ED_spl(e)->list[j].ep = map_point(ED_spl(e)->list[j].ep);
     }
     if (ED_label(e))
-	ED_label(e)->p = map_point(ED_label(e)->p);
+	ED_label(e)->pos = map_point(ED_label(e)->pos);
     /* vladimir */
     if (ED_head_label(e))
-	ED_head_label(e)->p = map_point(ED_head_label(e)->p);
+	ED_head_label(e)->pos = map_point(ED_head_label(e)->pos);
     if (ED_tail_label(e))
-	ED_tail_label(e)->p = map_point(ED_tail_label(e)->p);
+	ED_tail_label(e)->pos = map_point(ED_tail_label(e)->pos);
 }
 
 void translate_bb(graph_t * g, int rankdir)
 {
     int c;
-    box bb, new_bb;
+    boxf bb, new_bb;
 
     bb = GD_bb(g);
     if (rankdir == RANKDIR_LR || rankdir == RANKDIR_BT) {
-	new_bb.LL = map_point(pointof(bb.LL.x, bb.UR.y));
-	new_bb.UR = map_point(pointof(bb.UR.x, bb.LL.y));
+	new_bb.LL = map_point(pointfof(bb.LL.x, bb.UR.y));
+	new_bb.UR = map_point(pointfof(bb.UR.x, bb.LL.y));
     } else {
-	new_bb.LL = map_point(pointof(bb.LL.x, bb.LL.y));
-	new_bb.UR = map_point(pointof(bb.UR.x, bb.UR.y));
+	new_bb.LL = map_point(pointfof(bb.LL.x, bb.LL.y));
+	new_bb.UR = map_point(pointfof(bb.UR.x, bb.UR.y));
     }
     GD_bb(g) = new_bb;
     if (GD_label(g)) {
-	GD_label(g)->p = map_point(GD_label(g)->p);
+	GD_label(g)->pos = map_point(GD_label(g)->pos);
     }
     for (c = 1; c <= GD_n_cluster(g); c++)
 	translate_bb(GD_clust(g)[c], rankdir);
 }
 
-void dot_nodesize(node_t * n, boolean flip)
-{
-    double x, y;
-    int ps;
-
-    if (flip == FALSE) {
-	x = ND_width(n);
-	y = ND_height(n);
-    } else {
-	y = ND_width(n);
-	x = ND_height(n);
-    }
-    ps = POINTS(x) / 2;
-    if (ps < 1)
-	ps = 1;
-    ND_lw_i(n) = ND_rw_i(n) = ps;
-    if (x == y)
-	ND_ht_i(n) = 2*ps;
-    else
-	ND_ht_i(n) = POINTS(y);
-}
-
 /* translate_drawing:
  * Translate and/or rotate nodes, spline points, and bbox info if
- * Offset is non-trivial.
- * Also, if Rankdir, reset ND_lw, ND_rw, and ND_ht to correct value.
+ * necessary. Also, if Rankdir (!= RANKDIR_BT), reset ND_lw, ND_rw, 
+ * and ND_ht to correct value.
  */
 static void translate_drawing(graph_t * g)
 {
@@ -174,16 +152,13 @@ static void translate_drawing(graph_t * g)
 
     if (!shift && !Rankdir) return;
     for (v = agfstnode(g); v; v = agnxtnode(g, v)) {
-	if (Rankdir) dot_nodesize(v, FALSE);
-	if (shift) {
-	    ND_coord_i(v) = map_point(ND_coord_i(v));
-	    if (State == GVSPLINES)
-		for (e = agfstout(g, v); e; e = agnxtout(g, e))
-		    map_edge(e);
-	}
+	if (Rankdir) gv_nodesize(v, FALSE);
+	ND_coord(v) = map_point(ND_coord(v));
+	if (State == GVSPLINES)
+	    for (e = agfstout(g, v); e; e = agnxtout(g, e))
+		map_edge(e);
     }
-    if (shift)
-	translate_bb(g, GD_rankdir(g));
+    translate_bb(g, GD_rankdir(g));
 }
 
 /* place_root_label:
@@ -192,9 +167,9 @@ static void translate_drawing(graph_t * g)
  * flipped drawing has been transposed, so we don't have
  * to worry about switching x and y.
  */
-static void place_root_label(graph_t * g, point d)
+static void place_root_label(graph_t * g, pointf d)
 {
-    point p;
+    pointf p;
 
     if (GD_label_pos(g) & LABEL_AT_RIGHT) {
 	p.x = GD_bb(g).UR.x - d.x / 2;
@@ -210,7 +185,7 @@ static void place_root_label(graph_t * g, point d)
 	p.y = GD_bb(g).LL.y + d.y / 2;
     }
 
-    GD_label(g)->p = p;
+    GD_label(g)->pos = p;
     GD_label(g)->set = TRUE;
 }
 
@@ -223,9 +198,8 @@ static void place_root_label(graph_t * g, point d)
  */
 void dotneato_postprocess(Agraph_t * g)
 {
-    int diff;
-    pointf dimen;
-    point d = { 0, 0 };
+    double diff;
+    pointf dimen = {0., 0.};
 
     Rankdir = GD_rankdir(g);
     Flip = GD_flip(g);
@@ -237,36 +211,35 @@ void dotneato_postprocess(Agraph_t * g)
     if (GD_label(g) && !GD_label(g)->set) {
 	dimen = GD_label(g)->dimen;
 	PAD(dimen);
-	PF2P(dimen, d);
 	if (Flip) {
 	    if (GD_label_pos(g) & LABEL_AT_TOP) {
-		GD_bb(g).UR.x += d.y;
+		GD_bb(g).UR.x += dimen.y;
 	    } else {
-		GD_bb(g).LL.x -= d.y;
+		GD_bb(g).LL.x -= dimen.y;
 	    }
 
-	    if (d.x > GD_bb(g).UR.y - GD_bb(g).LL.y) {
-		diff = d.x - (GD_bb(g).UR.y - GD_bb(g).LL.y);
-		diff = diff / 2;
+	    if (dimen.x > (GD_bb(g).UR.y - GD_bb(g).LL.y)) {
+		diff = dimen.x - (GD_bb(g).UR.y - GD_bb(g).LL.y);
+		diff = diff / 2.;
 		GD_bb(g).LL.y -= diff;
 		GD_bb(g).UR.y += diff;
 	    }
 	} else {
 	    if (GD_label_pos(g) & LABEL_AT_TOP) {
 		if (Rankdir == RANKDIR_TB)
-		    GD_bb(g).UR.y += d.y;
+		    GD_bb(g).UR.y += dimen.y;
 		else
-		    GD_bb(g).LL.y -= d.y;
+		    GD_bb(g).LL.y -= dimen.y;
 	    } else {
 		if (Rankdir == RANKDIR_TB)
-		    GD_bb(g).LL.y -= d.y;
+		    GD_bb(g).LL.y -= dimen.y;
 		else
-		    GD_bb(g).UR.y += d.y;
+		    GD_bb(g).UR.y += dimen.y;
 	    }
 
-	    if (d.x > GD_bb(g).UR.x - GD_bb(g).LL.x) {
-		diff = d.x - (GD_bb(g).UR.x - GD_bb(g).LL.x);
-		diff = diff / 2;
+	    if (dimen.x > (GD_bb(g).UR.x - GD_bb(g).LL.x)) {
+		diff = dimen.x - (GD_bb(g).UR.x - GD_bb(g).LL.x);
+		diff = diff / 2.;
 		GD_bb(g).LL.x -= diff;
 		GD_bb(g).UR.x += diff;
 	    }
@@ -277,18 +250,18 @@ void dotneato_postprocess(Agraph_t * g)
 	Offset = GD_bb(g).LL;
 	break;
     case RANKDIR_LR:
-	Offset = pointof(-GD_bb(g).UR.y, GD_bb(g).LL.x);
+	Offset = pointfof(-GD_bb(g).UR.y, GD_bb(g).LL.x);
 	break;
     case RANKDIR_BT:
-	Offset = pointof(GD_bb(g).LL.x, -GD_bb(g).UR.y);
+	Offset = pointfof(GD_bb(g).LL.x, -GD_bb(g).UR.y);
 	break;
     case RANKDIR_RL:
-	Offset = pointof(GD_bb(g).LL.y, GD_bb(g).LL.x);
+	Offset = pointfof(GD_bb(g).LL.y, GD_bb(g).LL.x);
 	break;
     }
     translate_drawing(g);
     if (GD_label(g) && !GD_label(g)->set)
-	place_root_label(g, d);
+	place_root_label(g, dimen);
 
     if (Show_boxes) {
 	char buf[BUFSIZ];
@@ -301,88 +274,36 @@ void dotneato_postprocess(Agraph_t * g)
     }
 }
 
-#if 0
-void osize_label(textlabel_t * label, int *b, int *t, int *l, int *r)
-{
-    point pt, sz2;
-    pointf dimen;
-
-    dimen = label->dimen;
-    PAD(dimen);
-    sz2 = cvt2pt(label->dimen);
-    sz2.x /= 2;
-    sz2.y /= 2;
-    pt = add_points(label->p, sz2);
-    if (*r < pt.x)
-	*r = pt.x;
-    if (*t < pt.y)
-	*t = pt.y;
-    pt = sub_points(label->p, sz2);
-    if (*l > pt.x)
-	*l = pt.x;
-    if (*b > pt.y)
-	*b = pt.y;
-}
-#endif
-
 /* place_flip_graph_label:
  * Put cluster labels recursively in the flip case.
  */
 static void place_flip_graph_label(graph_t * g)
 {
     int c;
-    point p, d;
-#ifdef OLD
-    int maxx, minx;
-    int maxy, miny;
-    pointf dimen;
-#endif
+    pointf p, d;
 
+#ifndef WITH_CGRAPH
     if ((g != g->root) && (GD_label(g)) && !GD_label(g)->set) {
+#else /* WITH_CGRAPH */
+    if ((g != agroot(g)) && (GD_label(g)) && !GD_label(g)->set) {
+#endif /* WITH_CGRAPH */
 
 	if (GD_label_pos(g) & LABEL_AT_TOP) {
 	    d = GD_border(g)[RIGHT_IX];
 	    p.x = GD_bb(g).UR.x - d.x / 2;
-#ifdef OLD
-	    maxx = GD_bb(g).UR.x + d.y;
-	    GD_bb(g).UR.x = maxx;
-	    if (GD_bb(g->root).UR.x < maxx)
-		GD_bb(g->root).UR.x = maxx;
-#endif
 	} else {
 	    d = GD_border(g)[LEFT_IX];
 	    p.x = GD_bb(g).LL.x + d.x / 2;
-#ifdef OLD
-	    minx = GD_bb(g).LL.x - d.y;
-	    GD_bb(g).LL.x = minx;
-	    if (GD_bb(g->root).LL.x > minx)
-		GD_bb(g->root).LL.x = minx;
-#endif
 	}
 
 	if (GD_label_pos(g) & LABEL_AT_RIGHT) {
 	    p.y = GD_bb(g).LL.y + d.y / 2;
-#ifdef OLD
-	    maxy = p.y + d.x / 2;
-	    if (GD_bb(g->root).UR.y < maxy)
-		GD_bb(g->root).UR.y = maxy;
-#endif
 	} else if (GD_label_pos(g) & LABEL_AT_LEFT) {
 	    p.y = GD_bb(g).UR.y - d.y / 2;
-#ifdef OLD
-	    miny = p.y - d.x / 2;
-	    if (GD_bb(g->root).LL.y > miny)
-		GD_bb(g->root).LL.y = miny;
-#endif
 	} else {
 	    p.y = (GD_bb(g).LL.y + GD_bb(g).UR.y) / 2;
-#ifdef OLD
-	    maxy = p.y + d.x / 2;
-	    miny = p.y - d.x / 2;
-#endif
 	}
-
-	GD_label(g)->p = p;
+	GD_label(g)->pos = p;
 	GD_label(g)->set = TRUE;
     }
 
@@ -399,12 +320,13 @@ static void place_flip_graph_label(graph_t * g)
 void place_graph_label(graph_t * g)
 {
     int c;
-#ifdef OLD
-    int minx, maxx;
-#endif
-    point p, d;
+    pointf p, d;
 
+#ifndef WITH_CGRAPH
     if ((g != g->root) && (GD_label(g)) && !GD_label(g)->set) {
+#else /* WITH_CGRAPH */
+    if ((g != agroot(g)) && (GD_label(g)) && !GD_label(g)->set) {
+#endif /* WITH_CGRAPH */
 	if (GD_label_pos(g) & LABEL_AT_TOP) {
 	    d = GD_border(g)[TOP_IX];
 	    p.y = GD_bb(g).UR.y - d.y / 2;
@@ -415,38 +337,12 @@ void place_graph_label(graph_t * g)
 
 	if (GD_label_pos(g) & LABEL_AT_RIGHT) {
 	    p.x = GD_bb(g).UR.x - d.x / 2;
-#ifdef OLD
-	    minx = p.x - d.x / 2;
-	    if (GD_bb(g).LL.x > minx)
-		GD_bb(g).LL.x = minx;
-	    if (GD_bb(g->root).LL.x > minx)
-		GD_bb(g->root).LL.x = minx;
-#endif
 	} else if (GD_label_pos(g) & LABEL_AT_LEFT) {
 	    p.x = GD_bb(g).LL.x + d.x / 2;
-#ifdef OLD
-	    maxx = p.x + d.x / 2;
-	    if (GD_bb(g).UR.x < maxx)
-		GD_bb(g).UR.x = maxx;
-	    if (GD_bb(g->root).UR.x < maxx)
-		GD_bb(g->root).UR.x = maxx;
-#endif
 	} else {
 	    p.x = (GD_bb(g).LL.x + GD_bb(g).UR.x) / 2;
-#ifdef OLD
-	    maxx = p.x + d.x / 2;
-	    minx = p.x - d.x / 2;
-	    if (GD_bb(g).UR.x < maxx)
-		GD_bb(g).UR.x = maxx;
-	    if (GD_bb(g).LL.x > minx)
-		GD_bb(g).LL.x = minx;
-	    if (GD_bb(g->root).UR.x < maxx)
-		GD_bb(g->root).UR.x = maxx;
-	    if (GD_bb(g->root).LL.x > minx)
-		GD_bb(g->root).LL.x = minx;
-#endif
 	}
-	GD_label(g)->p = p;
+	GD_label(g)->pos = p;
 	GD_label(g)->set = TRUE;
     }
 
