@@ -1,4 +1,4 @@
-/* $Id: routespl.c,v 1.13 2007/10/12 00:40:59 erg Exp $ $Revision: 1.13 $ */
+/* $Id: routespl.c,v 1.23 2008/10/15 12:57:08 ellson Exp $ $Revision: 1.23 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -35,23 +35,20 @@ static int nedges, nboxes; /* total no. of edges and boxes used in routing */
 
 static int routeinit;
 /* static data used across multiple edges */
-static point *ps;             /* final spline points */
+static pointf *ps;             /* final spline points */
 static int maxpn;             /* size of ps[] */
 static Ppoint_t *polypoints;  /* vertices of polygon defined by boxes */
 static int polypointn;        /* size of polypoints[] */
 static Pedge_t *edges;        /* polygon edges passed to Proutespline */
 static int edgen;             /* size of edges[] */
 
-static void checkpath(int, box*, path*);
+static void checkpath(int, boxf*, path*);
 static void mkspacep(int size);
 static void printpath(path * pp);
-#ifdef OBSOLETE
-static int append(path * path, int bi, point p0, point p1, int);
-#endif
 #ifdef DEBUG
-static void printboxes(int boxn, box* boxes)
+static void printboxes(int boxn, boxf* boxes)
 {
-    point ll, ur;
+    pointf ll, ur;
     int bi;
     char buf[BUFSIZ];
     int newcnt = Show_cnt + boxn;
@@ -59,7 +56,7 @@ static void printboxes(int boxn, box* boxes)
     Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
     for (bi = 0; bi < boxn; bi++) {
 	ll = boxes[bi].LL, ur = boxes[bi].UR;
-	sprintf(buf, "%d %d %d %d pathbox", ll.x, ll.y, ur.x, ur.y);
+	sprintf(buf, "%d %d %d %d pathbox", (int)ll.x, (int)ll.y, (int)ur.x, (int)ur.y);
 	Show_boxes[bi+1+Show_cnt] = strdup (buf);
     }
     Show_cnt = newcnt;
@@ -89,6 +86,17 @@ static void psprintpoint(point p)
 	    p.x, p.y);
     fprintf(stderr, "grestore\n");
 }
+static void psprintpointf(pointf p)
+{
+    fprintf(stderr, "gsave\n");
+    fprintf(stderr,
+	    "newpath %.5g %.5g moveto %.5g %.5g 2 0 360 arc closepath fill stroke\n",
+	    p.x, p.y, p.x, p.y);
+    fprintf(stderr, "/Times-Roman findfont 4 scalefont setfont\n");
+    fprintf(stderr, "%.5g %.5g moveto (\\(%.5g,%.5g\\)) show\n", p.x + 5, p.y + 5,
+	    p.x, p.y);
+    fprintf(stderr, "grestore\n");
+}
 
 static void psprintspline(Ppolyline_t spl)
 {
@@ -160,11 +168,11 @@ static void psprintpoly(Ppoly_t p)
     Show_boxes[Show_cnt+1] = NULL;
 }
 
-static void psprintboxes(int boxn, box* boxes)
+static void psprintboxes(int boxn, boxf* boxes)
 {
     char buf[BUFSIZ];
     int newcnt = Show_cnt + 5*boxn + 3;
-    point ll, ur;
+    pointf ll, ur;
     int bi, li;
 
     Show_boxes = ALLOC(newcnt+2,Show_boxes,char*);
@@ -173,13 +181,13 @@ static void psprintboxes(int boxn, box* boxes)
     Show_boxes[li++] = strdup ("gsave 0 1 0 setrgbcolor");
     for (bi = 0; bi < boxn; bi++) {
 	ll = boxes[bi].LL, ur = boxes[bi].UR;
-	sprintf(buf, "newpath\n%d %d moveto", ll.x, ll.y);
+	sprintf(buf, "newpath\n%d %d moveto", (int)ll.x, (int)ll.y);
 	Show_boxes[li++] = strdup (buf);
-	sprintf(buf, "%d %d lineto", ll.x, ur.y);
+	sprintf(buf, "%d %d lineto", (int)ll.x, (int)ur.y);
 	Show_boxes[li++] = strdup (buf);
-	sprintf(buf, "%d %d lineto", ur.x, ur.y);
+	sprintf(buf, "%d %d lineto", (int)ur.x, (int)ur.y);
 	Show_boxes[li++] = strdup (buf);
-	sprintf(buf, "%d %d lineto", ur.x, ll.y);
+	sprintf(buf, "%d %d lineto", (int)ur.x, (int)ll.y);
 	Show_boxes[li++] = strdup (buf);
 	Show_boxes[li++] = strdup ("closepath stroke");
     }
@@ -212,21 +220,6 @@ static int debugleveln(edge_t* realedge, int i)
 }
 #endif  /* DEBUG */
 
-#ifdef OBSOLETE
-static point mkpt(int x, int y)
-{
-    point rv;
-    rv.x = x;
-    rv.y = y;
-    return rv;
-}
-
-static int pteq(point p, point q)
-{
-    return ((p.x == q.x) && (p.y == q.y));
-}
-#endif
-
 /* routesplinesinit:
  * Data initialized once until matching call to routeplineterm
  * Allows recursive calls to dot
@@ -235,14 +228,7 @@ void
 routesplinesinit()
 {
     if (++routeinit > 1) return;
-#ifdef UNUSED
-    if (!(bs = N_GNEW(BINC, box))) {
-	agerr(AGERR, "cannot allocate bs\n");
-	abort();
-    }
-    maxbn = BINC;
-#endif
-    if (!(ps = N_GNEW(PINC, point))) {
+    if (!(ps = N_GNEW(PINC, pointf))) {
 	agerr(AGERR, "cannot allocate ps\n");
 	abort();
     }
@@ -276,7 +262,7 @@ void routesplinesterm()
 		nedges, nboxes, elapsed_sec());
 }
 
-static point *_routesplines(path * pp, int *npoints, int polyline)
+static pointf *_routesplines(path * pp, int *npoints, int polyline)
 {
     Ppoly_t poly;
     Ppolyline_t pl, spl;
@@ -284,10 +270,10 @@ static point *_routesplines(path * pp, int *npoints, int polyline)
     Ppoint_t eps[2];
     Pvector_t evs[2];
     int edgei, prev, next;
-    point sp[4];
+    pointf sp[4];
     int pi, bi, si;
     double t;
-    box *boxes;
+    boxf *boxes;
     int boxn;
     edge_t* realedge;
     int flip;
@@ -329,14 +315,14 @@ static point *_routesplines(path * pp, int *npoints, int polyline)
     if ((boxn > 1) && (boxes[0].LL.y > boxes[1].LL.y)) {
         flip = 1;
 	for (bi = 0; bi < boxn; bi++) {
-	    int v = boxes[bi].UR.y;
+	    double v = boxes[bi].UR.y;
 	    boxes[bi].UR.y = -1*boxes[bi].LL.y;
 	    boxes[bi].LL.y = -v;
 	}
     }
     else flip = 0;
 
-    if (realedge->tail != realedge->head) {
+    if (agtail(realedge) != aghead(realedge)) {
 	/* I assume that the path goes either down only or
 	   up - right - down */
 	for (bi = 0, pi = 0; bi < boxn; bi++) {
@@ -411,40 +397,9 @@ static point *_routesplines(path * pp, int *npoints, int polyline)
 		polypoints[pi++].y = boxes[bi].LL.y;
 	    }
 	}
-    } 
+    }
     else {
-#ifdef OBSOLETE
-	/* new, more generalized approach for self-edges.  We do not
-	   assume any monotonicity about the box path, only that it
-	   is simply connected.  We build up the constraint poly by
-	   walking the box path from one end to the other and back
-	   in the recursive function append(). A better approach to all
-	   of this might be to dispense with the box paths altogether
-	   and just compute the constraint poly directly, but this
-	   needs to be done as part of a more thorough overhaul. */
-	point p0, p1;
-	box b0, b1;
-	b0 = pp->boxes[0];
-	b1 = pp->boxes[1];
-	/* determine 'starting' segment (side of b0) for box path search */
-	if (b0.UR.x == b1.LL.x) {
-	    p0 = b0.LL;
-	    p1 = mkpt(b0.LL.x, b0.UR.y);
-	} else if (b0.LL.y == b1.UR.y) {
-	    p0 = mkpt(b0.LL.x, b0.UR.y);
-	    p1 = b0.UR;
-	} else if (b0.LL.x == b1.UR.x) {
-	    p0 = b0.UR;
-	    p1 = mkpt(b0.UR.x, b0.LL.y);
-	} else if (b0.UR.y == b1.LL.y) {
-	    p0 = mkpt(b0.UR.x, b0.LL.y);
-	    p1 = b0.LL;
-	} else
-	    abort();
-	pi = append(pp, 0, p0, p1, 0);
-#else
 	abort();
-#endif
     }
 
     if (flip) {
@@ -510,14 +465,13 @@ static point *_routesplines(path * pp, int *npoints, int polyline)
 	boxes[bi].UR.x = INT_MIN;
     }
     for (splinepi = 0; splinepi < spl.pn; splinepi++) {
-	ps[splinepi].x = spl.ps[splinepi].x;
-	ps[splinepi].y = spl.ps[splinepi].y;
+	ps[splinepi] = spl.ps[splinepi];
     }
 REDO:
     for (splinepi = 0; splinepi + 3 < spl.pn; splinepi += 3) {
 	int num_div = delta * boxn;
 	for (si = 0; si <= num_div; si++) {
-	    t = si / ((double)num_div);
+	    t = si / (double)num_div;
 	    sp[0] = ps[splinepi];
 	    sp[1] = ps[splinepi + 1];
 	    sp[2] = ps[splinepi + 2];
@@ -535,7 +489,10 @@ REDO:
 	    sp[0].x = sp[0].x + t * (sp[1].x - sp[0].x);
 	    sp[0].y = sp[0].y + t * (sp[1].y - sp[0].y);
 	    for (bi = 0; bi < boxn; bi++) {
-		if (sp[0].y <= boxes[bi].UR.y && sp[0].y >= boxes[bi].LL.y) {
+/* this tested ok on 64bit machines, but on 32bit we need this FUDGE
+ *     or graphs/directed/records.gv fails */
+#define FUDGE .0001
+		if (sp[0].y <= boxes[bi].UR.y+FUDGE && sp[0].y >= boxes[bi].LL.y-FUDGE) {
 		    if (boxes[bi].LL.x > sp[0].x)
 			boxes[bi].LL.x = sp[0].x;
 		    if (boxes[bi].UR.x < sp[0].x)
@@ -550,6 +507,8 @@ REDO:
      * valid values. cf. bug 456. Would making sp[] pointfs help?
      */
     for (bi = 0; bi < boxn; bi++) {
+	/* these fp equality tests are used only to detect if the
+	 * values have been changed since initialization - ok */
 	if ((boxes[bi].LL.x == INT_MAX) || (boxes[bi].UR.x == INT_MIN)) {
 	    delta *= 2;
 	    goto REDO;
@@ -569,12 +528,12 @@ REDO:
     return ps;
 }
 
-point *routesplines(path * pp, int *npoints)
+pointf *routesplines(path * pp, int *npoints)
 {
     return _routesplines (pp, npoints, 0);
 }
 
-point *routepolylines(path * pp, int *npoints)
+pointf *routepolylines(path * pp, int *npoints)
 {
     return _routesplines (pp, npoints, 1);
 }
@@ -601,9 +560,9 @@ static int overlap(int i0, int i1, int j0, int j1)
  * fixing all the bugs, at least try to engineer around them!
  * in postmodern CS, we could call this "self-healing code."
  */
-static void checkpath(int boxn, box* boxes, path* thepath)
+static void checkpath(int boxn, boxf* boxes, path* thepath)
 {
-    box *ba, *bb;
+    boxf *ba, *bb;
     int bi, i, errs, l, r, d, u;
     int xoverlap, yoverlap;
 
@@ -611,9 +570,9 @@ static void checkpath(int boxn, box* boxes, path* thepath)
     /* remove degenerate boxes. */
     i = 0;
     for (bi = 0; bi < boxn; bi++) {
-	if (boxes[bi].LL.y == boxes[bi].UR.y)
+	if (ABS(boxes[bi].LL.y - boxes[bi].UR.y) < .01)
 	    continue;
-	if (boxes[bi].LL.x == boxes[bi].UR.x)
+	if (ABS(boxes[bi].LL.x - boxes[bi].UR.x) < .01)
 	    continue;
 	if (i != bi)
 	    boxes[i] = boxes[bi];
@@ -762,167 +721,11 @@ static void mkspacep(int size)
 {
     if (size > maxpn) {
 	int newmax = maxpn + (size / PINC + 1) * PINC;
-	ps = RALLOC(newmax, ps, point);
+	ps = RALLOC(newmax, ps, pointf);
 	maxpn = newmax;
     }
 }
 
-#ifdef OBSOLETE
-/* new code to create poly from box list
- * given that we entered the box b on segment p0,p1 (p0==p1 allowed) 
- * then add successive points to the constraint poly
- */
-
-#define BOXLEFT 0
-#define BOXTOP 1
-#define BOXRIGHT 2
-#define BOXBOTTOM 3
-static box B;
-
-static int sideofB(point p, box B)
-{
-    if (p.x == B.LL.x)
-	return BOXLEFT;
-    if (p.y == B.UR.y)
-	return BOXTOP;
-    if (p.x == B.UR.x)
-	return BOXRIGHT;
-    if (p.y == B.LL.y)
-	return BOXBOTTOM;
-    abort();
-    return 0;
-}
-
-static int appendpt(point p, int polysz)
-{
-    polypoints[polysz].x = p.x;
-    polypoints[polysz].y = p.y;
-    return (polysz+1);
-}
-
-static int cmpf(const void *pp0, const void *pp1)
-{
-    point p0, p1;
-    int s0, s1;
-
-    p0 = *(point *) pp0;
-    p1 = *(point *) pp1;
-    s0 = sideofB(p0, B);
-    s1 = sideofB(p1, B);
-
-    if (s0 != s1)
-	return s1 - s0;
-    switch (s0) {
-    case BOXLEFT:
-	return p1.y - p0.y;
-    case BOXTOP:
-	return p1.x - p0.x;
-    case BOXRIGHT:
-	return p0.y - p1.y;
-    case BOXBOTTOM:
-	return p0.x - p1.x;
-    default:
-	abort();
-    }
-    return 0;			/* not reached */
-}
-
-/* append:
- */
-static int 
-append(path * path, int bi, point p0, point p1, int polysz)
-{
-    point v[8];		/* worst case 4 corners + 2 segs * 2 points each */
-    point w[8];
-    box b = path->boxes[bi];
-    box bb;
-    int i, i0, npw, delta;
-    point q0 = { 0, 0 }, q1 = { 0, 0}, r;
-    int pn;
-
-    /* v = 4 corners of b, p0 and p1 */
-    pn = 0;
-    v[pn++] = b.LL;
-    v[pn++] = mkpt(b.UR.x, b.LL.y);
-    v[pn++] = b.UR;
-    v[pn++] = mkpt(b.LL.x, b.UR.y);
-    v[pn++] = p0;
-    v[pn++] = p1;
-
-    if (bi + 1 < path->nbox) {
-	bb = path->boxes[bi + 1];
-	/* determine points q0,q1 where b and bb touch and append to v */
-	if (b.UR.x == bb.LL.x) {
-	    q0.x = q1.x = b.UR.x;
-	    q0.y = MIN(b.UR.y, bb.UR.y);
-	    q1.y = MAX(b.LL.y, bb.LL.y);
-	} else if (b.LL.x == bb.UR.x) {
-	    q0.x = q1.x = b.LL.x;
-	    q0.y = MIN(b.UR.y, bb.UR.y);
-	    q1.y = MAX(b.LL.y, bb.LL.y);
-	} else if (b.UR.y == bb.LL.y) {
-	    q0.y = q1.y = b.UR.y;
-	    q0.x = MIN(b.UR.x, bb.UR.x);
-	    q1.x = MAX(b.LL.x, bb.LL.x);
-	} else if (b.LL.y == bb.UR.y) {
-	    q0.y = q1.y = b.LL.y;
-	    q0.x = MIN(b.UR.x, bb.UR.x);
-	    q1.x = MAX(b.LL.x, bb.LL.x);
-	} else
-	    abort();
-	v[pn++] = q0;
-	v[pn++] = q1;
-    }
-
-    /* sort v so that the cyclic order is p0, all other points, p1  */
-    B = b;
-    qsort(v, pn, sizeof(v[0]), cmpf);
-
-    /* eliminate duplicates and record i0 = index of p0 in w */
-    w[0] = v[0];
-    npw = 1;
-    i0 = -1;
-    for (i = 0; i < pn; i++) {
-	if (pteq(w[npw - 1], p0))
-	    i0 = npw - 1;
-	if (!pteq(v[i], w[npw - 1]))
-	    w[npw++] = v[i];
-    }
-
-    i = i0;
-    if (bi == 0)
-	polysz = appendpt(p0, polysz);
-    if (pteq(p1, w[(i0 + 1) % npw]))
-	delta = -1;
-    else if (pteq(p1, w[(i0 - 1 + npw) % npw]))
-	delta = 1;
-    else
-	abort();
-    do {
-	i = (i + delta + npw) % npw;	/* go to the next point in order */
-	r = w[i];		/* call it r */
-
-	/* append r to current poly, except p0 and p1 are special cases */
-	if ((bi == 0) || (!pteq(r, p0) && !pteq(r, p1)))
-	    polysz = appendpt(r, polysz);
-	if (pteq(r, p1))
-	    break;
-	if (bi + 1 < path->nbox) {	/* recur when we hit the next box */
-	    if (pteq(r, q0)) {
-		polysz = append(path, bi + 1, q0, q1, polysz);
-		polysz = appendpt(q1, polysz);	/* assumes q1 != p0 and p1 */
-		i += delta;	/* skip q1 */
-	    } else if (pteq(r, q1)) {
-		polysz = append(path, bi + 1, q1, q0, polysz);
-		polysz = appendpt(q0, polysz);
-		i += delta;
-	    }
-	}
-    } while (i != i0);
-    return polysz;
-}
-#endif
-
 static void printpath(path * pp)
 {
     int bi;
@@ -935,13 +738,13 @@ static void printpath(path * pp)
 #endif
     fprintf(stderr, "%d boxes:\n", pp->nbox);
     for (bi = 0; bi < pp->nbox; bi++)
-	fprintf(stderr, "%d (%d, %d), (%d, %d)\n", bi, pp->boxes[bi].LL.x,
-		pp->boxes[bi].LL.y, pp->boxes[bi].UR.x,
-		pp->boxes[bi].UR.y);
-    fprintf(stderr, "start port: (%d, %d), tangent angle: %.3f, %s\n",
+	fprintf(stderr, "%d (%.5g, %.5g), (%.5g, %.5g)\n", bi,
+		pp->boxes[bi].LL.x, pp->boxes[bi].LL.y,
+	       	pp->boxes[bi].UR.x, pp->boxes[bi].UR.y);
+    fprintf(stderr, "start port: (%.5g, %.5g), tangent angle: %.5g, %s\n",
 	    pp->start.p.x, pp->start.p.y, pp->start.theta,
 	    pp->start.constrained ? "constrained" : "not constrained");
-    fprintf(stderr, "end port: (%d, %d), tangent angle: %.3f, %s\n",
+    fprintf(stderr, "end port: (%.5g, %.5g), tangent angle: %.5g, %s\n",
 	    pp->end.p.x, pp->end.p.y, pp->end.theta,
 	    pp->end.constrained ? "constrained" : "not constrained");
 }
