@@ -1,4 +1,4 @@
-/* $Id: strcasecmp.c,v 1.3 2005/07/20 02:39:16 ellson Exp $ $Revision: 1.3 $ */
+/* $Id: strcasecmp.c,v 1.4 2009/01/22 21:10:26 arif Exp $ $Revision: 1.4 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -18,7 +18,6 @@
 #include "config.h"
 #endif
 
-#ifndef HAVE_STRCASECMP
 
 #include <string.h>
 #include <ctype.h>
@@ -36,4 +35,3 @@ int strcasecmp(const char *s1, const char *s2)
     return tolower(*(unsigned char *) s1) - tolower(*(unsigned char *) s2);
 }
 
-#endif				/* HAVE_STRCASECMP */
