@@ -1,4 +1,4 @@
-/* $Id: textpara.h,v 1.10 2008/03/06 21:40:02 ellson Exp $ $Revision: 1.10 $ */
+/* $Id: textpara.h,v 1.11 2008/08/15 21:37:36 ellson Exp $ $Revision: 1.11 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -40,7 +40,7 @@ extern "C" {
 	void *layout;
 	void (*free_layout) (void *layout);   /* FIXME - this is ugly */
 	double fontsize, width, height, yoffset_layout, yoffset_centerline;
-	char just;
+	char just;	/* 'l' 'n' 'r' */
     } textpara_t;
 
 #ifdef __cplusplus
