@@ -1,4 +1,4 @@
-/* $Id: compound.c,v 1.3 2005/06/22 19:20:51 erg Exp $ $Revision: 1.3 $ */
+/* $Id: compound.c,v 1.9 2008/10/23 02:18:13 ellson Exp $ $Revision: 1.9 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -20,25 +20,12 @@
 
 #include	"dot.h"
 
-
-/* midPt:
- * Return midpoint between two given points.
+/* pf2s:
+ * Convert a pointf to its string representation.
  */
-static point midPt(point p, point q)
+static char *pf2s(pointf p, char *buf)
 {
-    point v;
-
-    v.x = (p.x + q.x) / 2;
-    v.y = (p.y + q.y) / 2;
-    return v;
-}
-
-/* p2s:
- * Convert a point to its string representation.
- */
-static char *p2s(point p, char *buf)
-{
-    sprintf(buf, "(%d,%d)", p.x, p.y);
+    sprintf(buf, "(%.5g,%.5g)", p.x, p.y);
     return buf;
 }
 
@@ -46,16 +33,18 @@ static char *p2s(point p, char *buf)
  * the box bp. Assume cp is outside the box, and pp is
  * on or in the box. 
  */
-static point boxIntersect(point pp, point cp, box * bp)
+static pointf boxIntersectf(pointf pp, pointf cp, boxf * bp)
 {
-    point ipp;
+    pointf ipp;
     double ppx = pp.x;
     double ppy = pp.y;
     double cpx = cp.x;
     double cpy = cp.y;
-    point ll = bp->LL;
-    point ur = bp->UR;
+    pointf ll;
+    pointf ur;
 
+    ll = bp->LL;
+    ur = bp->UR;
     if (cp.x < ll.x) {
 	ipp.x = ll.x;
 	ipp.y = pp.y + (int) ((ipp.x - ppx) * (ppy - cpy) / (ppx - cpx));
@@ -86,21 +75,20 @@ static point boxIntersect(point pp, point cp, box * bp)
 	char ppbuf[100], cpbuf[100], llbuf[100], urbuf[100];
 
 	agerr(AGERR,
-	      "segment [%s,%s] does not intersect box ll=%s,ur=%s\n",
-	      p2s(pp, ppbuf), p2s(cp, cpbuf), p2s(ll, llbuf), p2s(ur,
-								  urbuf));
+		"segment [%s,%s] does not intersect box ll=%s,ur=%s\n",
+		pf2s(pp, ppbuf), pf2s(cp, cpbuf),
+		pf2s(ll, llbuf), pf2s(ur, urbuf));
 	assert(0);
     }
     return ipp;
 }
 
-/* inBox:
+/* inBoxf:
  * Returns true if p is on or in box bb
  */
-static int inBox(point p, box * bb)
+static int inBoxf(pointf p, boxf * bb)
 {
-    return ((p.x >= bb->LL.x) && (p.x <= bb->UR.x) &&
-	    (p.y >= bb->LL.y) && (p.y <= bb->UR.y));
+    return INSIDE(p, *bb);
 }
 
 /* getCluster:
@@ -114,7 +102,11 @@ static graph_t *getCluster(graph_t * g, char *cluster_name)
 
     if (!cluster_name || (*cluster_name == '\0'))
 	return NULL;
+#ifndef WITH_CGRAPH
     sg = agfindsubg(g, cluster_name);
+#else
+    sg = agsubg(g, cluster_name, 0);
+#endif
     if (sg == NULL)
 	agerr(AGWARN, "cluster named %s not found\n", cluster_name);
     return sg;
@@ -123,58 +115,54 @@ static graph_t *getCluster(graph_t * g, char *cluster_name)
 /* The following functions are derived from pp. 411-415 (pp. 791-795)
  * of Graphics Gems. In the code there, they use a SGN function to
  * count crossings. This doesn't seem to handle certain special cases,
- * as when the last point is on the line. It certainly doesn't work
- * for use; see bug 145. We need to use ZSGN instead.
+ * as when the last point is on the line. It certainly didn't work
+ * for us when we used int values; see bug 145. We needed to use CMP instead.
+ *
+ * Possibly unnecessary with double values, but harmless.
  */
-#define SGN(a,b)          (((a)<(b)) ? -1 : 1)
-#define ZSGN(a,b)         (((a)<(b)) ? -1 : (a)>(b) ? 1 : 0)
 
 /* countVertCross:
  * Return the number of times the Bezier control polygon crosses
  * the vertical line x = xcoord.
  */
-static int countVertCross(pointf * pts, int xcoord)
+static int countVertCross(pointf * pts, double xcoord)
 {
     int i;
     int sign, old_sign;
     int num_crossings = 0;
 
-    sign = old_sign = ZSGN(pts[0].x, xcoord);
+    sign = CMP(pts[0].x, xcoord);
     if (sign == 0)
 	num_crossings++;
     for (i = 1; i <= 3; i++) {
-	sign = ZSGN(pts[i].x, xcoord);
+	old_sign = sign;
+	sign = CMP(pts[i].x, xcoord);
 	if ((sign != old_sign) && (old_sign != 0))
 	    num_crossings++;
-	old_sign = sign;
     }
-
     return num_crossings;
-
 }
 
 /* countHorzCross:
  * Return the number of times the Bezier control polygon crosses
  * the horizontal line y = ycoord.
  */
-static int countHorzCross(pointf * pts, int ycoord)
+static int countHorzCross(pointf * pts, double ycoord)
 {
     int i;
     int sign, old_sign;
     int num_crossings = 0;
 
-    sign = old_sign = ZSGN(pts[0].y, ycoord);
+    sign = CMP(pts[0].y, ycoord);
     if (sign == 0)
 	num_crossings++;
     for (i = 1; i <= 3; i++) {
-	sign = ZSGN(pts[i].y, ycoord);
+	old_sign = sign;
+	sign = CMP(pts[i].y, ycoord);
 	if ((sign != old_sign) && (old_sign != 0))
 	    num_crossings++;
-	old_sign = sign;
     }
-
     return num_crossings;
-
 }
 
 /* findVertical:
@@ -187,7 +175,7 @@ static int countHorzCross(pointf * pts, int ycoord)
  */
 static double
 findVertical(pointf * pts, double tmin, double tmax,
-	     int xcoord, int ymin, int ymax)
+	     double xcoord, double ymin, double ymax)
 {
     pointf Left[4];
     pointf Right[4];
@@ -197,8 +185,8 @@ findVertical(pointf * pts, double tmin, double tmax,
     if (no_cross == 0)
 	return -1.0;
 
-    /* if 1 crossing and on the line x = xcoord */
-    if ((no_cross == 1) && (ROUND(pts[3].x) == xcoord)) {
+    /* if 1 crossing and on the line x == xcoord (within 1 point) */
+    if ((no_cross == 1) && (ROUND(pts[3].x) == ROUND(xcoord))) {
 	if ((ymin <= pts[3].y) && (pts[3].y <= ymax)) {
 	    return tmax;
 	} else
@@ -225,7 +213,7 @@ findVertical(pointf * pts, double tmin, double tmax,
  */
 static double
 findHorizontal(pointf * pts, double tmin, double tmax,
-	       int ycoord, int xmin, int xmax)
+	       double ycoord, double xmin, double xmax)
 {
     pointf Left[4];
     pointf Right[4];
@@ -235,8 +223,8 @@ findHorizontal(pointf * pts, double tmin, double tmax,
     if (no_cross == 0)
 	return -1.0;
 
-    /* if 1 crossing and on the line y = ycoord */
-    if ((no_cross == 1) && (ROUND(pts[3].y) == ycoord)) {
+    /* if 1 crossing and on the line y == ycoord (within 1 point) */
+    if ((no_cross == 1) && (ROUND(pts[3].y) == ROUND(ycoord))) {
 	if ((xmin <= pts[3].x) && (pts[3].x <= xmax)) {
 	    return tmax;
 	} else
@@ -251,31 +239,25 @@ findHorizontal(pointf * pts, double tmin, double tmax,
 	return t;
     return findHorizontal(Right, (tmin + tmax) / 2.0, tmax, ycoord, xmin,
 			  xmax);
-
 }
 
-#define P2PF(p, pf) (pf.x = p.x, pf.y = p.y)
-#define PF2P(pf, p) (p.x = ROUND (pf.x), p.y = ROUND (pf.y))
-
-/* splineIntersect:
+/* splineIntersectf:
  * Given four spline control points and a box,
  * find the shortest portion of the spline from
- * ipts[0] to the intersection with the box, if any.
- * If an intersection is found, the four points are stored in ipts[0..3]
- * with ipts[3] being on the box, and 1 is returned. Otherwise, ipts
+ * pts[0] to the intersection with the box, if any.
+ * If an intersection is found, the four points are stored in pts[0..3]
+ * with pts[3] being on the box, and 1 is returned. Otherwise, pts
  * is left unchanged and 0 is returned.
  */
-static int splineIntersect(point * ipts, box * bb)
+static int splineIntersectf(pointf * pts, boxf * bb)
 {
     double tmin = 2.0;
     double t;
-    pointf pts[4];
     pointf origpts[4];
     int i;
 
     for (i = 0; i < 4; i++) {
-	P2PF(ipts[i], origpts[i]);
-	pts[i] = origpts[i];
+	origpts[i] = pts[i];
     }
 
     t = findVertical(pts, 0.0, 1.0, bb->LL.x, bb->LL.y, bb->UR.y);
@@ -303,13 +285,9 @@ static int splineIntersect(point * ipts, box * bb)
     }
 
     if (tmin < 2.0) {
-	for (i = 0; i < 4; i++) {
-	    PF2P(pts[i], ipts[i]);
-	}
 	return 1;
     } else
 	return 0;
-
 }
 
 /* makeCompoundEdge:
@@ -328,11 +306,11 @@ static void makeCompoundEdge(graph_t * g, edge_t * e)
     int starti = 0, endi = 0;	/* index of first and last control point */
     node_t *head;
     node_t *tail;
-    box *bb;
+    boxf *bb;
     int i, j;
     int size;
-    point pts[4];
-    point p;
+    pointf pts[4];
+    pointf p;
     int fixed;
 
     /* find head and tail target clusters, if defined */
@@ -345,14 +323,14 @@ static void makeCompoundEdge(graph_t * g, edge_t * e)
     /* at present, we only handle single spline case */
     if (ED_spl(e)->size > 1) {
 	agerr(AGWARN, "%s -> %s: spline size > 1 not supported\n",
-	      e->tail->name, e->head->name);
+	      agnameof(agtail(e)), agnameof(aghead(e)));
 	return;
     }
     bez = ED_spl(e)->list;
     size = bez->size;
 
-    head = e->head;
-    tail = e->tail;
+    head = aghead(e);
+    tail = agtail(e);
 
     /* allocate new Bezier */
     nbez = GNEW(bezier);
@@ -371,42 +349,41 @@ static void makeCompoundEdge(graph_t * g, edge_t * e)
     fixed = 0;
     if (lh) {
 	bb = &(GD_bb(lh));
-	if (!inBox(ND_coord_i(head), bb)) {
+	if (!inBoxf(ND_coord(head), bb)) {
 	    agerr(AGWARN, "%s -> %s: head not inside head cluster %s\n",
-		  e->tail->name, e->head->name, agget(e, "lhead"));
+		  agnameof(agtail(e)), agnameof(aghead(e)), agget(e, "lhead"));
 	} else {
 	    /* If first control point is in bb, degenerate case. Spline
 	     * reduces to four points between the arrow head and the point 
 	     * where the segment between the first control point and arrow head 
 	     * crosses box.
 	     */
-	    if (inBox(bez->list[0], bb)) {
-		if (inBox(ND_coord_i(tail), bb)) {
+	    if (inBoxf(bez->list[0], bb)) {
+		if (inBoxf(ND_coord(tail), bb)) {
 		    agerr(AGWARN,
 			  "%s -> %s: tail is inside head cluster %s\n",
-			  e->tail->name, e->head->name, agget(e, "lhead"));
+			  agnameof(agtail(e)), agnameof(aghead(e)), agget(e, "lhead"));
 		} else {
 		    assert(bez->sflag);	/* must be arrowhead on tail */
-		    p = boxIntersect(bez->list[0], bez->sp, bb);
+		    p = boxIntersectf(bez->list[0], bez->sp, bb);
 		    bez->list[3] = p;
-		    bez->list[1] = midPt(p, bez->sp);
-		    bez->list[0] = midPt(bez->list[1], bez->sp);
-		    bez->list[2] = midPt(bez->list[1], p);
+		    bez->list[1] = mid_pointf(p, bez->sp);
+		    bez->list[0] = mid_pointf(bez->list[1], bez->sp);
+		    bez->list[2] = mid_pointf(bez->list[1], p);
 		    if (bez->eflag)
-			endi =
-			    arrowEndClip(e, bez->list,
+			endi = arrowEndClip(e, bez->list,
 					 starti, 0, nbez, bez->eflag);
 		    endi += 3;
 		    fixed = 1;
 		}
 	    } else {
 		for (endi = 0; endi < size - 1; endi += 3) {
-		    if (splineIntersect(&(bez->list[endi]), bb))
+		    if (splineIntersectf(&(bez->list[endi]), bb))
 			break;
 		}
 		if (endi == size - 1) {	/* no intersection */
 		    assert(bez->eflag);
-		    nbez->ep = boxIntersect(bez->ep, bez->list[endi], bb);
+		    nbez->ep = boxIntersectf(bez->ep, bez->list[endi], bb);
 		} else {
 		    if (bez->eflag)
 			endi =
@@ -432,42 +409,38 @@ static void makeCompoundEdge(graph_t * g, edge_t * e)
     fixed = 0;
     if (lt) {
 	bb = &(GD_bb(lt));
-	if (!inBox(ND_coord_i(tail), bb)) {
+	if (!inBoxf(ND_coord(tail), bb)) {
 	    agerr(AGWARN, "%s -> %s: tail not inside tail cluster %s\n",
-		  e->tail->name, head->name, agget(e, "ltail"));
+		agnameof(agtail(e)), agnameof(aghead(e)), agget(e, "ltail"));
 	} else {
 	    /* If last control point is in bb, degenerate case. Spline
 	     * reduces to four points between arrow head, and the point
 	     * where the segment between the last control point and the 
 	     * arrow head crosses box.
 	     */
-	    if (inBox(bez->list[endi], bb)) {
-		if (inBox(ND_coord_i(head), bb)) {
+	    if (inBoxf(bez->list[endi], bb)) {
+		if (inBoxf(ND_coord(head), bb)) {
 		    agerr(AGWARN,
-			  "%s -> %s: head is inside tail cluster %s\n",
-			  e->tail->name, e->head->name, agget(e, "ltail"));
+			"%s -> %s: head is inside tail cluster %s\n",
+		  	agnameof(agtail(e)), agnameof(aghead(e)), agget(e, "ltail"));
 		} else {
 		    assert(bez->eflag);	/* must be arrowhead on head */
-		    p = boxIntersect(bez->list[endi], nbez->ep, bb);
+		    p = boxIntersectf(bez->list[endi], nbez->ep, bb);
 		    starti = endi - 3;
 		    bez->list[starti] = p;
-		    bez->list[starti + 2] = midPt(p, nbez->ep);
-		    bez->list[starti + 3] =
-			midPt(bez->list[starti + 2], nbez->ep);
-		    bez->list[starti + 1] =
-			midPt(bez->list[starti + 2], p);
+		    bez->list[starti + 2] = mid_pointf(p, nbez->ep);
+		    bez->list[starti + 3] = mid_pointf(bez->list[starti + 2], nbez->ep);
+		    bez->list[starti + 1] = mid_pointf(bez->list[starti + 2], p);
 		    if (bez->sflag)
-			starti =
-			    arrowStartClip(e, bez->list,
-					   starti, endi - 3, nbez,
-					   bez->sflag);
+			starti = arrowStartClip(e, bez->list, starti,
+				endi - 3, nbez, bez->sflag);
 		    fixed = 1;
 		}
 	    } else {
 		for (starti = endi; starti > 0; starti -= 3) {
 		    for (i = 0; i < 4; i++)
 			pts[i] = bez->list[starti - i];
-		    if (splineIntersect(pts, bb)) {
+		    if (splineIntersectf(pts, bb)) {
 			for (i = 0; i < 4; i++)
 			    bez->list[starti - i] = pts[i];
 			break;
@@ -476,14 +449,12 @@ static void makeCompoundEdge(graph_t * g, edge_t * e)
 		if (starti == 0) {
 		    assert(bez->sflag);
 		    nbez->sp =
-			boxIntersect(bez->sp, bez->list[starti], bb);
+			boxIntersectf(bez->sp, bez->list[starti], bb);
 		} else {
 		    starti -= 3;
 		    if (bez->sflag)
-			starti =
-			    arrowStartClip(e, bez->list,
-					   starti, endi - 3, nbez,
-					   bez->sflag);
+			starti = arrowStartClip(e, bez->list, starti,
+				endi - 3, nbez, bez->sflag);
 		}
 		fixed = 1;
 	    }
@@ -498,13 +469,12 @@ static void makeCompoundEdge(graph_t * g, edge_t * e)
     /* complete Bezier, free garbage and attach new Bezier to edge 
      */
     nbez->size = endi - starti + 1;
-    nbez->list = N_GNEW(nbez->size, point);
+    nbez->list = N_GNEW(nbez->size, pointf);
     for (i = 0, j = starti; i < nbez->size; i++, j++)
 	nbez->list[i] = bez->list[j];
     free(bez->list);
     free(bez);
     ED_spl(e)->list = nbez;
-
 }
 
 /* dot_compoundEdges:
