@@ -1,4 +1,4 @@
-/* $Id: dotinit.c,v 1.10 2008/04/19 16:00:36 erg Exp $ $Revision: 1.10 $ */
+/* $Id: dotinit.c,v 1.18 2009/01/19 22:32:43 arif Exp $ $Revision: 1.18 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -15,13 +15,18 @@
 **********************************************************/
 
 
-#include    "dot.h"
+#include <time.h>
+#include "dot.h"
+#include "aspect.h"
 
 static void 
 dot_init_node(node_t * n)
 {
+#ifdef WITH_CGRAPH
+    agbindrec(n, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);	//graph custom data
+#endif /* WITH_CGRAPH */
     common_init_node(n);
-    dot_nodesize(n, GD_flip(n->graph));
+    gv_nodesize(n, GD_flip(agraphof(n)));
     alloc_elist(4, ND_in(n));
     alloc_elist(4, ND_out(n));
     alloc_elist(2, ND_flat_in(n));
@@ -34,12 +39,14 @@ static void
 dot_init_edge(edge_t * e)
 {
     char *tailgroup, *headgroup;
-
+#ifdef WITH_CGRAPH
+    agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);	//graph custom data
+#endif /* WITH_CGRAPH */
     common_init_edge(e);
 
     ED_weight(e) = late_double(e, E_weight, 1.0, 0.0);
-    tailgroup = late_string(e->tail, N_group, "");
-    headgroup = late_string(e->head, N_group, "");
+    tailgroup = late_string(agtail(e), N_group, "");
+    headgroup = late_string(aghead(e), N_group, "");
     ED_count(e) = ED_xpenalty(e) = 1;
     if (tailgroup[0] && (tailgroup == headgroup)) {
 	ED_xpenalty(e) = CL_CROSS;
@@ -92,7 +99,11 @@ dot_cleanup_node(node_t * n)
     free_label(ND_label(n));
     if (ND_shape(n))
 	ND_shape(n)->fns->freefn(n);
+#ifndef WITH_CGRAPH
     memset(&(n->u), 0, sizeof(Agnodeinfo_t));
+#else /* WITH_CGRAPH */
+	agclean(agraphof(n), AGNODE,"Agnodeinfo_t");	
+#endif /* WITH_CGRAPH */
 }
 
 static void free_virtual_edge_list(node_t * n)
@@ -141,12 +152,17 @@ dot_cleanup_graph(graph_t * g)
     }
 
     free_list(GD_comp(g));
-    if ((g == g->root) && GD_rank(g)) {
+    if ((g == agroot(g)) && GD_rank(g)) {
 	for (i = GD_minrank(g); i <= GD_maxrank(g); i++)
 	    free(GD_rank(g)[i].v);
 	free(GD_rank(g));
     }
-    if (g != g->root) memset(&(g->u), 0, sizeof(Agraphinfo_t));
+    if (g != agroot(g)) 
+#ifndef WITH_CGRAPH
+	memset(&(g->u), 0, sizeof(Agraphinfo_t));
+#else /* WITH_CGRAPH */
+	agclean(g,AGRAPH,"Agraphinfo_t");
+#endif /* WITH_CGRAPH */
 }
 
 /* delete the layout (but retain the underlying graph) */
@@ -199,18 +215,64 @@ dumpRanks (graph_t * g)
 }
 #endif
 
+#define MIN_AR 1.0
+#define MAX_AR 20.0
+#define DEF_PASSES 5
+
+static aspect_t*
+setAspect (Agraph_t * g, aspect_t* adata)
+{
+    double rv;
+    char *p;
+    int r, passes = DEF_PASSES;
+
+    p = agget (g, "aspect");
+
+    if (!p || ((r = sscanf (p, "%lf,%d", &rv, &passes)) <= 0)) {
+	adata->nextIter = 0;
+	adata->badGraph = 0;
+	return NULL;
+    }
+    
+    if (rv < MIN_AR) rv = MIN_AR;
+    else if (rv > MAX_AR) rv = MAX_AR;
+    adata->targetAR = rv;
+    adata->nextIter = -1;
+    adata->nPasses = passes;
+    adata->badGraph = 0;
+
+    if (Verbose) 
+        fprintf(stderr, "Target AR = %g\n", adata->targetAR);
+
+    return adata;
+}
+
 void dot_layout(Agraph_t * g)
 {
+    aspect_t aspect;
+    aspect_t* asp;
+
     setEdgeType (g, ET_SPLINE);
+    asp = setAspect (g, &aspect);
+
     dot_init_node_edge(g);
-    dot_rank(g);
-    dot_mincross(g);
-    /* dumpRanks (g); */
-    dot_position(g);
-    /* dumpRanks (g); */
+
+    do {
+        dot_rank(g, asp);
+	if (aspect.badGraph) {
+	    agerr(AGWARN, "dot does not support the aspect attribute for disconnected graphs or graphs with clusters\n");
+	    asp = NULL;
+	    aspect.nextIter = 0;
+	}
+        dot_mincross(g,  (asp != NULL));
+        dot_position(g, asp);
+	aspect.nPasses--;
+    } while (aspect.nextIter && aspect.nPasses);
+
     dot_sameports(g);
     dot_splines(g);
     if (mapbool(agget(g, "compound")))
 	dot_compoundEdges(g);
     dotneato_postprocess(g);
+
 }
