@@ -1,4 +1,4 @@
-/* $Id: mincross.c,v 1.12 2008/06/20 14:57:39 erg Exp $ $Revision: 1.12 $ */
+/* $Id: mincross.c,v 1.17 2008/10/21 19:48:31 ellson Exp $ $Revision: 1.17 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -25,9 +25,9 @@
 #include "dot.h"
 
 /* #define DEBUG */
-#define MARK(v)			((v)->u.mark)
-#define saveorder(v)	((v)->u.coord.x)
-#define flatindex(v)	((v)->u.low)
+#define MARK(v)		(ND_mark(v))
+#define saveorder(v)	(ND_coord(v)).x
+#define flatindex(v)	ND_low(v)
 
 	/* forward declarations */
 static boolean medians(graph_t * g, int r0, int r1);
@@ -40,8 +40,8 @@ static void init_mincross(graph_t * g);
 static void merge2(graph_t * g);
 static void init_mccomp(graph_t * g, int c);
 static void cleanup2(graph_t * g, int nc);
-static int mincross_clust(graph_t * par, graph_t * g);
-static int mincross(graph_t * g, int startpass, int endpass);
+static int mincross_clust(graph_t * par, graph_t * g, int);
+static int mincross(graph_t * g, int startpass, int endpass, int);
 static void init_mincross(graph_t * g);
 static void mincross_step(graph_t * g, int pass);
 static void mincross_options(graph_t * g);
@@ -72,7 +72,7 @@ static boolean ReMincross;
  * Note that nodes are not placed into GD_rank(g) until mincross()
  * is called.
  */
-void dot_mincross(graph_t * g)
+void dot_mincross(graph_t * g, int doBalance)
 {
     int c, nc;
     char *s;
@@ -81,14 +81,14 @@ void dot_mincross(graph_t * g)
 
     for (nc = c = 0; c < GD_comp(g).size; c++) {
 	init_mccomp(g, c);
-	nc += mincross(g, 0, 2);
+	nc += mincross(g, 0, 2, doBalance);
     }
 
     merge2(g);
 
     /* run mincross on contents of each cluster */
     for (c = 1; c <= GD_n_cluster(g); c++) {
-	nc += mincross_clust(g, GD_clust(g)[c]);
+	nc += mincross_clust(g, GD_clust(g)[c], doBalance);
 #ifdef DEBUG
 	check_vlists(GD_clust(g)[c]);
 	check_order();
@@ -99,7 +99,7 @@ void dot_mincross(graph_t * g)
 	&& (!(s = agget(g, "remincross")) || (mapbool(s)))) {
 	mark_lowclusters(g);
 	ReMincross = TRUE;
-	nc = mincross(g, 2, 2);
+	nc = mincross(g, 2, 2, doBalance);
 #ifdef DEBUG
 	for (c = 1; c <= GD_n_cluster(g); c++)
 	    check_vlists(GD_clust(g)[c]);
@@ -127,8 +127,7 @@ static void free_matrix(adjmatrix_t * p)
 
 #define ELT(M,i,j)		(M->data[((i)*M->ncols)+(j)])
 
-static void 
-init_mccomp(graph_t * g, int c)
+static void init_mccomp(graph_t * g, int c)
 {
     int r;
 
@@ -145,11 +144,10 @@ static int betweenclust(edge_t * e)
 {
     while (ED_to_orig(e))
 	e = ED_to_orig(e);
-    return (ND_clust(e->tail) != ND_clust(e->head));
+    return (ND_clust(agtail(e)) != ND_clust(aghead(e)));
 }
 
-static void 
-do_ordering(graph_t * g, int outflag)
+static void do_ordering(graph_t * g, int outflag)
 {
     int i, ne;
     node_t *n, *u, *v;
@@ -177,11 +175,11 @@ do_ordering(graph_t * g, int outflag)
 	for (ne = 1; (f = sortlist[ne]); ne++) {
 	    e = sortlist[ne - 1];
 	    if (outflag) {
-		u = e->head;
-		v = f->head;
+		u = aghead(e);
+		v = aghead(f);
 	    } else {
-		u = e->tail;
-		v = f->tail;
+		u = agtail(e);
+		v = agtail(f);
 	    }
 	    if (find_flat_edge(u, v))
 		continue;
@@ -195,8 +193,7 @@ do_ordering(graph_t * g, int outflag)
 /* ordered_edges:
  * handle case where graph specifies edge ordering
  */
-static void 
-ordered_edges(graph_t * g)
+static void ordered_edges(graph_t * g)
 {
     char *ordering;
 
@@ -208,9 +205,10 @@ ordered_edges(graph_t * g)
 	else if (ordering[0])
 	    agerr(AGERR, "ordering '%s' not recognized.\n", ordering);
     }
-
-    else {
+    else
+    {
 	/* search meta-graph to find subgraphs that may be ordered */
+#ifndef WITH_CGRAPH
 	graph_t *mg, *subg;
 	node_t *mm, *mn;
 	edge_t *me;
@@ -221,14 +219,18 @@ ordered_edges(graph_t * g)
 	    mn = me->head;
 	    subg = agusergraph(mn);
 	    /* clusters are processed by seperate calls to ordered_edges */
+#else /* WITH_CGRAPH */
+	graph_t *subg;
+
+	for (subg = agfstsubg(g); subg; subg = agnxtsubg(subg)) {
+#endif /* WITH_CGRAPH */
 	    if (!is_cluster(subg))
 		ordered_edges(subg);
 	}
     }
 }
 
-static int 
-mincross_clust(graph_t * par, graph_t * g)
+static int mincross_clust(graph_t * par, graph_t * g, int doBalance)
 {
     int c, nc;
 
@@ -236,17 +238,16 @@ mincross_clust(graph_t * par, graph_t * g)
     ordered_edges(g);
     flat_breakcycles(g);
     flat_reorder(g);
-    nc = mincross(g, 2, 2);
+    nc = mincross(g, 2, 2, doBalance);
 
     for (c = 1; c <= GD_n_cluster(g); c++)
-	nc += mincross_clust(g, GD_clust(g)[c]);
+	nc += mincross_clust(g, GD_clust(g)[c], doBalance);
 
     save_vlist(g);
     return nc;
 }
 
-static int 
-left2right(graph_t * g, node_t * v, node_t * w)
+static int left2right(graph_t * g, node_t * v, node_t * w)
 {
     adjmatrix_t *M;
     int rv;
@@ -282,51 +283,49 @@ left2right(graph_t * g, node_t * v, node_t * w)
     return rv;
 }
 
-static int 
-in_cross(node_t * v, node_t * w)
+static int in_cross(node_t * v, node_t * w)
 {
     register edge_t **e1, **e2;
     register int inv, cross = 0, t;
 
     for (e2 = ND_in(w).list; *e2; e2++) {
-	register int cnt = (*e2)->u.xpenalty;
-	inv = ((*e2)->tail)->u.order;
+	register int cnt = ED_xpenalty(*e2);		
+		
+	inv = ND_order((agtail(*e2)));
 
 	for (e1 = ND_in(v).list; *e1; e1++) {
-	    t = ((*e1)->tail)->u.order - inv;
+	    t = ND_order(agtail(*e1)) - inv;
 	    if ((t > 0)
 		|| ((t == 0)
-		    && ((*e1)->u.tail_port.p.x > (*e2)->u.tail_port.p.x)))
-		cross += (*e1)->u.xpenalty * cnt;
+		    && (  ED_tail_port(*e1).p.x > ED_tail_port(*e2).p.x)))
+		cross += ED_xpenalty(*e1) * cnt;
 	}
     }
     return cross;
 }
 
-static int
-out_cross(node_t * v, node_t * w)
+static int out_cross(node_t * v, node_t * w)
 {
     register edge_t **e1, **e2;
     register int inv, cross = 0, t;
 
     for (e2 = ND_out(w).list; *e2; e2++) {
-	register int cnt = (*e2)->u.xpenalty;
-	inv = ((*e2)->head)->u.order;
+	register int cnt = ED_xpenalty(*e2);
+	inv = ND_order(aghead(*e2));
 
 	for (e1 = ND_out(v).list; *e1; e1++) {
-	    t = ((*e1)->head)->u.order - inv;
+	    t = ND_order(aghead(*e1)) - inv;
 	    if ((t > 0)
 		|| ((t == 0)
-		    && ((*e1)->u.head_port.p.x > (*e2)->u.head_port.p.x)))
-		cross += (*e1)->u.xpenalty * cnt;
+		    && ((ED_head_port(*e1)).p.x > (ED_head_port(*e2)).p.x)))
+		cross += ((ED_xpenalty(*e1)) * cnt);
 	}
     }
     return cross;
 
 }
 
-static void 
-exchange(node_t * v, node_t * w)
+static void exchange(node_t * v, node_t * w)
 {
     int vi, wi, r;
 
@@ -339,8 +338,149 @@ exchange(node_t * v, node_t * w)
     GD_rank(Root)[r].v[vi] = w;
 }
 
-static int 
-transpose_step(graph_t * g, int r, int reverse)
+static void balanceNodes(graph_t * g, int r, node_t * v, node_t * w)
+{
+    node_t *s;			/* separator node */
+    int sepIndex;
+    int nullType;		/* type of null nodes */
+    int cntDummy = 0, cntOri = 0;
+    int k = 0, m = 0, k1 = 0, m1 = 0, i = 0;
+
+    /* we only consider v and w of different types */
+    if (ND_node_type(v) == ND_node_type(w))
+	return;
+
+    /* count the number of dummy and original nodes */
+    for (i = 0; i < GD_rank(g)[r].n; i++) {
+	if (ND_node_type(GD_rank(g)[r].v[i]) == NORMAL)
+	    cntOri++;
+	else
+	    cntDummy++;
+    }
+
+    if (cntOri < cntDummy) {
+	if (ND_node_type(v) == NORMAL)
+	    s = v;
+	else
+	    s = w;
+    } else {
+	if (ND_node_type(v) == NORMAL)
+	    s = w;
+	else
+	    s = v;
+    }
+
+    /* get the separator node index */
+    for (i = 0; i < GD_rank(g)[r].n; i++) {
+	if (GD_rank(g)[r].v[i] == s)
+	    sepIndex = i;
+    }
+
+    nullType = (ND_node_type(s) == NORMAL) ? VIRTUAL : NORMAL;
+
+    /* count the number of null nodes to the left and 
+     * right of the separator node 
+     */
+    for (i = sepIndex - 1; i >= 0; i--) {
+	if (ND_node_type(GD_rank(g)[r].v[i]) == nullType)
+	    k++;
+	else
+	    break;
+    }
+
+    for (i = sepIndex + 1; i < GD_rank(g)[r].n; i++) {
+	if (ND_node_type(GD_rank(g)[r].v[i]) == nullType)
+	    m++;
+	else
+	    break;
+    }
+
+    /* now exchange v,w and calculate the same counts */
+
+    exchange(v, w);
+
+    /* get the separator node index */
+    for (i = 0; i < GD_rank(g)[r].n; i++) {
+	if (GD_rank(g)[r].v[i] == s)
+	    sepIndex = i;
+    }
+
+    /* count the number of null nodes to the left and 
+     * right of the separator node 
+     */
+    for (i = sepIndex - 1; i >= 0; i--) {
+	if (ND_node_type(GD_rank(g)[r].v[i]) == nullType)
+	    k1++;
+	else
+	    break;
+    }
+
+    for (i = sepIndex + 1; i < GD_rank(g)[r].n; i++) {
+	if (ND_node_type(GD_rank(g)[r].v[i]) == nullType)
+	    m1++;
+	else
+	    break;
+    }
+
+    if (abs(k1 - m1) > abs(k - m)) {
+	exchange(v, w);		//revert to the original ordering
+    }
+}
+
+static int balance(graph_t * g)
+{
+    int i, c0, c1, rv;
+    node_t *v, *w;
+    int r;
+
+    rv = 0;
+
+    for (r = GD_maxrank(g); r >= GD_minrank(g); r--) {
+
+	GD_rank(g)[r].candidate = FALSE;
+	for (i = 0; i < GD_rank(g)[r].n - 1; i++) {
+	    v = GD_rank(g)[r].v[i];
+	    w = GD_rank(g)[r].v[i + 1];
+	    assert(ND_order(v) < ND_order(w));
+	    if (left2right(g, v, w))
+		continue;
+	    c0 = c1 = 0;
+	    if (r > 0) {
+		c0 += in_cross(v, w);
+		c1 += in_cross(w, v);
+	    }
+
+	    if (GD_rank(g)[r + 1].n > 0) {
+		c0 += out_cross(v, w);
+		c1 += out_cross(w, v);
+	    }
+#if 0
+	    if ((c1 < c0) || ((c0 > 0) && reverse && (c1 == c0))) {
+		exchange(v, w);
+		rv += (c0 - c1);
+		GD_rank(Root)[r].valid = FALSE;
+		GD_rank(g)[r].candidate = TRUE;
+
+		if (r > GD_minrank(g)) {
+		    GD_rank(Root)[r - 1].valid = FALSE;
+		    GD_rank(g)[r - 1].candidate = TRUE;
+		}
+		if (r < GD_maxrank(g)) {
+		    GD_rank(Root)[r + 1].valid = FALSE;
+		    GD_rank(g)[r + 1].candidate = TRUE;
+		}
+	    }
+#endif
+
+	    if (c1 <= c0) {
+		balanceNodes(g, r, v, w);
+	    }
+	}
+    }
+    return rv;
+}
+
+static int transpose_step(graph_t * g, int r, int reverse)
 {
     int i, c0, c1, rv;
     node_t *v, *w;
@@ -381,8 +521,7 @@ transpose_step(graph_t * g, int r, int reverse)
     return rv;
 }
 
-static void 
-transpose(graph_t * g, int reverse)
+static void transpose(graph_t * g, int reverse)
 {
     int r, delta;
 
@@ -407,7 +546,7 @@ transpose(graph_t * g, int reverse)
     } while (delta >= 1);
 }
 
-static int mincross(graph_t * g, int startpass, int endpass)
+static int mincross(graph_t * g, int startpass, int endpass, int doBalance)
 {
     int maxthispass, iter, trying, pass;
     int cur_cross, best_cross;
@@ -420,7 +559,7 @@ static int mincross(graph_t * g, int startpass, int endpass)
     for (pass = startpass; pass <= endpass; pass++) {
 	if (pass <= 1) {
 	    maxthispass = MIN(4, MaxIter);
-	    if (g == g->root)
+	    if (g == agroot(g))
 		build_ranks(g, pass);
 	    if (pass == 0)
 		flat_breakcycles(g);
@@ -464,6 +603,11 @@ static int mincross(graph_t * g, int startpass, int endpass)
 	transpose(g, FALSE);
 	best_cross = ncross(g);
     }
+    if (doBalance) {
+	for (iter = 0; iter < maxthispass; iter++)
+	    balance(g);
+    }
+
     return best_cross;
 }
 
@@ -489,8 +633,7 @@ static void save_best(graph_t * g)
 }
 
 /* merges the connected components of g */
-static void 
-merge_components(graph_t * g)
+static void merge_components(graph_t * g)
 {
     int c;
     node_t *u, *v;
@@ -533,7 +676,7 @@ static void merge2(graph_t * g)
 		if (Verbose)
 		    fprintf(stderr,
 			    "merge2: graph %s, rank %d has only %d < %d nodes\n",
-			    g->name, r, i, GD_rank(g)[r].n);
+			    agnameof(g), r, i, GD_rank(g)[r].n);
 		GD_rank(g)[r].n = i;
 		break;
 	    }
@@ -567,8 +710,7 @@ static void cleanup2(graph_t * g, int nc)
 	    ND_order(v) = i;
 	    if (ND_flat_out(v).list) {
 		for (j = 0; (e = ND_flat_out(v).list[j]); j++)
-		    if ((ED_edge_type(e) == FLATORDER) ||
-		        (ED_edge_type(e) == REVERSED)) {
+		    if (ED_edge_type(e) == FLATORDER) {
 			delete_flat_edge(e);
 			free(e);
 			j--;
@@ -579,7 +721,7 @@ static void cleanup2(graph_t * g, int nc)
     }
     if (Verbose)
 	fprintf(stderr, "mincross %s: %d crossings, %.2f secs.\n",
-		g->name, nc, elapsed_sec());
+		agnameof(g), nc, elapsed_sec());
 }
 
 static node_t *neighbor(node_t * v, int dir)
@@ -595,14 +737,12 @@ static node_t *neighbor(node_t * v, int dir)
     return rv;
 }
 
-static int 
-is_a_normal_node_of(graph_t * g, node_t * v)
+static int is_a_normal_node_of(graph_t * g, node_t * v)
 {
     return ((ND_node_type(v) == NORMAL) && agcontains(g, v));
 }
 
-static int 
-is_a_vnode_of_an_edge_of(graph_t * g, node_t * v)
+static int is_a_vnode_of_an_edge_of(graph_t * g, node_t * v)
 {
     if ((ND_node_type(v) == VIRTUAL)
 	&& (ND_in(v).size == 1) && (ND_out(v).size == 1)) {
@@ -615,8 +755,7 @@ is_a_vnode_of_an_edge_of(graph_t * g, node_t * v)
     return FALSE;
 }
 
-static int 
-inside_cluster(graph_t * g, node_t * v)
+static int inside_cluster(graph_t * g, node_t * v)
 {
     return (is_a_normal_node_of(g, v) | is_a_vnode_of_an_edge_of(g, v));
 }
@@ -676,7 +815,11 @@ void rec_reset_vlists(graph_t * g)
 #ifdef DEBUG
 	    assert(GD_rank(g->root)[r].v[ND_order(u)] == u);
 #endif
+#ifndef WITH_CGRAPH
 	    GD_rank(g)[r].v = ND_rank(g->root)[r].v + ND_order(u);
+#else /* WITH_CGRAPH */
+	    GD_rank(g)[r].v = GD_rank(agroot(g))[r].v + ND_order(u);
+#endif /* WITH_CGRAPH */
 	    GD_rank(g)[r].n = ND_order(w) - ND_order(u) + 1;
 	}
 }
@@ -693,7 +836,7 @@ static void init_mincross(graph_t * g)
     /* alloc +1 for the null terminator usage in do_ordering() */
     /* also, the +1 avoids attempts to alloc 0 sizes, something
        that efence complains about */
-    size = agnedges(g->root) + 1;
+    size = agnedges(agroot(g)) + 1;
     TE_list = N_NEW(size, edge_t *);
     TI_list = N_NEW(size, int);
     mincross_options(g);
@@ -705,14 +848,17 @@ static void init_mincross(graph_t * g)
     GlobalMaxRank = GD_maxrank(g);
 }
 
-void flat_rev(Agraph_t *g, Agedge_t *e) 
+void flat_rev(Agraph_t * g, Agedge_t * e)
 {
     int j;
     Agedge_t *rev;
 
-    for (j = 0; (rev = ND_flat_out(e->head).list[j]); j++)
-	if (rev->head == e->tail)
-	    break;
+    if (!ND_flat_out(aghead(e)).list)
+	rev = NULL;
+    else
+	for (j = 0; (rev = ND_flat_out(aghead(e)).list[j]); j++)
+	    if (aghead(rev) == agtail(e))
+		break;
     if (rev) {
 	merge_oneway(e, rev);
 	if (ED_to_virt(e) == 0)
@@ -720,10 +866,13 @@ void flat_rev(Agraph_t *g, Agedge_t *e)
 	if ((ED_edge_type(rev) == FLATORDER)
 	    && (ED_to_orig(rev) == 0))
 	    ED_to_orig(rev) = e;
-	elist_append(e, ND_other(e->tail));
+	elist_append(e, ND_other(agtail(e)));
     } else {
-	rev = new_virtual_edge(e->head, e->tail, e);
-	ED_edge_type(rev) = REVERSED;
+	rev = new_virtual_edge(aghead(e), agtail(e), e);
+	if (ED_edge_type(e) == FLATORDER)
+	    ED_edge_type(rev) = FLATORDER;
+	else
+	    ED_edge_type(rev) = REVERSED;
 	ED_label(rev) = ED_label(e);
 	flat_edge(g, rev);
     }
@@ -738,29 +887,33 @@ static void flat_search(graph_t * g, node_t * v)
 
     ND_mark(v) = TRUE;
     ND_onstack(v) = TRUE;
+#ifndef WITH_CGRAPH
     hascl = (ND_n_cluster(g->root) > 0);
+#else /* WITH_CGRAPH */
+    hascl = (GD_n_cluster(agroot(g)) > 0);
+#endif /* WITH_CGRAPH */
     if (ND_flat_out(v).list)
 	for (i = 0; (e = ND_flat_out(v).list[i]); i++) {
 	    if (hascl
-		&& NOT(agcontains(g, e->tail) && agcontains(g, e->head)))
+		&& NOT(agcontains(g, agtail(e)) && agcontains(g, aghead(e))))
 		continue;
 	    if (ED_weight(e) == 0)
 		continue;
-	    if (ND_onstack(e->head) == TRUE) {
-		assert(flatindex(e->head) < M->nrows);
-		assert(flatindex(e->tail) < M->ncols);
-		ELT(M, flatindex(e->head), flatindex(e->tail)) = 1;
+	    if (ND_onstack(aghead(e)) == TRUE) {
+		assert(flatindex(aghead(e)) < M->nrows);
+		assert(flatindex(agtail(e)) < M->ncols);
+		ELT(M, flatindex(aghead(e)), flatindex(agtail(e))) = 1;
 		delete_flat_edge(e);
 		i--;
 		if (ED_edge_type(e) == FLATORDER)
 		    continue;
-		flat_rev(g,e);
+		flat_rev(g, e);
 	    } else {
-		assert(flatindex(e->head) < M->nrows);
-		assert(flatindex(e->tail) < M->ncols);
-		ELT(M, flatindex(e->tail), flatindex(e->head)) = 1;
-		if (ND_mark(e->head) == FALSE)
-		    flat_search(g, e->head);
+		assert(flatindex(aghead(e)) < M->nrows);
+		assert(flatindex(agtail(e)) < M->ncols);
+		ELT(M, flatindex(agtail(e)), flatindex(aghead(e))) = 1;
+		if (ND_mark(aghead(e)) == FALSE)
+		    flat_search(g, aghead(e));
 	    }
 	}
     ND_onstack(v) = FALSE;
@@ -797,8 +950,7 @@ static void flat_breakcycles(graph_t * g)
  * Allocate rank structure, determining number of nodes per rank.
  * Note that no nodes are put into the structure yet.
  */
-void 
-allocate_ranks(graph_t * g)
+void allocate_ranks(graph_t * g)
 {
     int r, low, high, *cn;
     node_t *n;
@@ -808,8 +960,8 @@ allocate_ranks(graph_t * g)
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	cn[ND_rank(n)]++;
 	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
-	    low = ND_rank(e->tail);
-	    high = ND_rank(e->head);
+	    low = ND_rank(agtail(e));
+	    high = ND_rank(aghead(e));
 	    if (low > high) {
 		int t = low;
 		low = high;
@@ -824,7 +976,7 @@ allocate_ranks(graph_t * g)
 	GD_rank(g)[r].an = GD_rank(g)[r].n = cn[r];
 	GD_rank(g)[r].av = GD_rank(g)[r].v = N_NEW(cn[r] + 1, node_t *);
     }
-    free (cn);
+    free(cn);
 }
 
 /* install a node at the current right end of its rank */
@@ -836,7 +988,7 @@ void install_in_rank(graph_t * g, node_t * n)
     i = GD_rank(g)[r].n;
     if (GD_rank(g)[r].an <= 0) {
 	agerr(AGERR, "install_in_rank %s %s rank %d i = %d an = 0\n",
-	      g->name, n->name, r, i);
+	      agnameof(g), agnameof(n), r, i);
 	abort();
     }
 
@@ -883,74 +1035,74 @@ void build_ranks(graph_t * g, int pass)
 	edge_t *e;
 	for (n = GD_nlist(g); n; n = ND_next(n)) {
 	    for (i = 0; (e = ND_out(n).list[i]); i++)
-		assert(MARK(e->head) == FALSE);
+		assert(MARK(aghead(e)) == FALSE);
 	    for (i = 0; (e = ND_in(n).list[i]); i++)
-		assert(MARK(e->tail) == FALSE);
+		assert(MARK(agtail(e)) == FALSE);
 	}
     }
 #endif
 
-for (i = GD_minrank(g); i <= GD_maxrank(g); i++)
-    GD_rank(g)[i].n = 0;
+    for (i = GD_minrank(g); i <= GD_maxrank(g); i++)
+	GD_rank(g)[i].n = 0;
 
-for (n = GD_nlist(g); n; n = ND_next(n)) {
-    otheredges = ((pass == 0) ? ND_in(n).list : ND_out(n).list);
-    if (otheredges[0] != NULL)
-	continue;
-    if (MARK(n) == FALSE) {
-	MARK(n) = TRUE;
-	enqueue(q, n);
-	while ((n0 = dequeue(q))) {
-	    if (ND_ranktype(n0) != CLUSTER) {
-		install_in_rank(g, n0);
-		enqueue_neighbors(q, n0, pass);
-	    } else {
-		install_cluster(g, n0, pass, q);
+    for (n = GD_nlist(g); n; n = ND_next(n)) {
+	otheredges = ((pass == 0) ? ND_in(n).list : ND_out(n).list);
+	if (otheredges[0] != NULL)
+	    continue;
+	if (MARK(n) == FALSE) {
+	    MARK(n) = TRUE;
+	    enqueue(q, n);
+	    while ((n0 = dequeue(q))) {
+		if (ND_ranktype(n0) != CLUSTER) {
+		    install_in_rank(g, n0);
+		    enqueue_neighbors(q, n0, pass);
+		} else {
+		    install_cluster(g, n0, pass, q);
+		}
 	    }
 	}
     }
-}
-if (dequeue(q))
-    agerr(AGERR, "surprise\n");
-for (i = GD_minrank(g); i <= GD_maxrank(g); i++) {
-    GD_rank(Root)[i].valid = FALSE;
-    if (GD_flip(g) && (GD_rank(g)[i].n > 0)) {
-	int n, ndiv2;
-	node_t **vlist = GD_rank(g)[i].v;
-	n = GD_rank(g)[i].n - 1;
-	ndiv2 = n / 2;
-	for (j = 0; j <= ndiv2; j++)
-	    exchange(vlist[j], vlist[n - j]);
+    if (dequeue(q))
+	agerr(AGERR, "surprise\n");
+    for (i = GD_minrank(g); i <= GD_maxrank(g); i++) {
+	GD_rank(Root)[i].valid = FALSE;
+	if (GD_flip(g) && (GD_rank(g)[i].n > 0)) {
+	    int n, ndiv2;
+	    node_t **vlist = GD_rank(g)[i].v;
+	    n = GD_rank(g)[i].n - 1;
+	    ndiv2 = n / 2;
+	    for (j = 0; j <= ndiv2; j++)
+		exchange(vlist[j], vlist[n - j]);
+	}
     }
-}
 
-if ((g == g->root) && ncross(g) > 0)
-    transpose(g, FALSE);
-free_queue(q);
+    if ((g == agroot(g)) && ncross(g) > 0)
+	transpose(g, FALSE);
+    free_queue(q);
 }
 
 void enqueue_neighbors(nodequeue * q, node_t * n0, int pass)
 {
-int i;
-edge_t *e;
+    int i;
+    edge_t *e;
 
-if (pass == 0) {
-    for (i = 0; i < ND_out(n0).size; i++) {
-	e = ND_out(n0).list[i];
-	if ((MARK(e->head)) == FALSE) {
-	    MARK(e->head) = TRUE;
-	    enqueue(q, e->head);
+    if (pass == 0) {
+	for (i = 0; i < ND_out(n0).size; i++) {
+	    e = ND_out(n0).list[i];
+	    if ((MARK(aghead(e))) == FALSE) {
+		MARK(aghead(e)) = TRUE;
+		enqueue(q, aghead(e));
+	    }
+	}
+    } else {
+	for (i = 0; i < ND_in(n0).size; i++) {
+	    e = ND_in(n0).list[i];
+	    if ((MARK(agtail(e))) == FALSE) {
+		MARK(agtail(e)) = TRUE;
+		enqueue(q, agtail(e));
+	    }
 	}
     }
-} else {
-    for (i = 0; i < ND_in(n0).size; i++) {
-	e = ND_in(n0).list[i];
-	if ((MARK(e->tail)) == FALSE) {
-	    MARK(e->tail) = TRUE;
-	    enqueue(q, e->tail);
-	}
-    }
-}
 }
 
 /* construct nodes reachable from 'here' in post-order.
@@ -958,95 +1110,98 @@ if (pass == 0) {
 */
 static int postorder(graph_t * g, node_t * v, node_t ** list, int r)
 {
-edge_t *e;
-int i, cnt = 0;
+    edge_t *e;
+    int i, cnt = 0;
 
-MARK(v) = TRUE;
-if (ND_flat_out(v).size > 0) {
-    for (i = 0; (e = ND_flat_out(v).list[i]); i++) {
-	if (ED_weight(e) == 0) continue;
-	if ((ND_node_type(e->head) == NORMAL) &
-	    (NOT(agcontains(g, e->head))))
-	    continue;
-	if (ND_clust(e->head) != ND_clust(e->tail))
-	    continue;
+    MARK(v) = TRUE;
+    if (ND_flat_out(v).size > 0) {
+	for (i = 0; (e = ND_flat_out(v).list[i]); i++) {
+	    if (ED_weight(e) == 0)
+		continue;
+	    if ((ND_node_type(aghead(e)) == NORMAL) &
+		(NOT(agcontains(g, aghead(e)))))
+		continue;
+	    if (ND_clust(aghead(e)) != ND_clust(agtail(e)))
+		continue;
 
-	if (MARK(e->head) == FALSE)
-	    cnt += postorder(g, e->head, list + cnt, r);
+	    if (MARK(aghead(e)) == FALSE)
+		cnt += postorder(g, aghead(e), list + cnt, r);
+	}
     }
-}
-assert(ND_rank(v) == r);
-list[cnt++] = v;
-return cnt;
+    assert(ND_rank(v) == r);
+    list[cnt++] = v;
+    return cnt;
 }
 
 static void flat_reorder(graph_t * g)
 {
-int i, j, r, pos, n_search, local_in_cnt, local_out_cnt;
-node_t *v, **left, **right, *t;
-node_t **temprank = NULL;
-edge_t *flat_e, *e;
+    int i, j, r, pos, n_search, local_in_cnt, local_out_cnt;
+    node_t *v, **left, **right, *t;
+    node_t **temprank = NULL;
+    edge_t *flat_e, *e;
 
-if (GD_has_flat_edges(g) == FALSE)
-    return;
-for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
-    for (i = 0; i < GD_rank(g)[r].n; i++)
-	MARK(GD_rank(g)[r].v[i]) = FALSE;
-    temprank = ALLOC(i + 1, temprank, node_t *);
-    pos = 0;
-    for (i = 0; i < GD_rank(g)[r].n; i++) {
-	v = GD_rank(g)[r].v[i];
+    if (GD_has_flat_edges(g) == FALSE)
+	return;
+    for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
+	for (i = 0; i < GD_rank(g)[r].n; i++)
+	    MARK(GD_rank(g)[r].v[i]) = FALSE;
+	temprank = ALLOC(i + 1, temprank, node_t *);
+	pos = 0;
+	for (i = 0; i < GD_rank(g)[r].n; i++) {
+	    v = GD_rank(g)[r].v[i];
 
-	local_in_cnt = local_out_cnt = 0;
-	for (j = 0; j < ND_flat_in(v).size; j++) {
-	    flat_e = ND_flat_in(v).list[j];
-	    if ((ED_weight(flat_e) > 0) && (inside_cluster(g, flat_e->tail)))
-		local_in_cnt++;
-	}
-	for (j = 0; j < ND_flat_out(v).size; j++) {
-	    flat_e = ND_flat_out(v).list[j];
-	    if ((ED_weight(flat_e) > 0) && (inside_cluster(g, flat_e->head)))
-		local_out_cnt++;
-	}
-	if ((local_in_cnt == 0) && (local_out_cnt == 0))
-	    temprank[pos++] = v;
-	else {
-	    if ((MARK(v) == FALSE) && (local_in_cnt == 0)) {
-		left = temprank + pos;
-		n_search = postorder(g, v, left, r);
-		if (GD_flip(g) == FALSE) {
-		    right = left + n_search - 1;
-		    while (left < right) {
-			t = *left;
-			*left = *right;
-			*right = t;
-			left++;
-			right--;
+	    local_in_cnt = local_out_cnt = 0;
+	    for (j = 0; j < ND_flat_in(v).size; j++) {
+		flat_e = ND_flat_in(v).list[j];
+		if ((ED_weight(flat_e) > 0)
+		    && (inside_cluster(g, aghead(flat_e))))
+		    local_in_cnt++;
+	    }
+	    for (j = 0; j < ND_flat_out(v).size; j++) {
+		flat_e = ND_flat_out(v).list[j];
+		if ((ED_weight(flat_e) > 0)
+		    && (inside_cluster(g, aghead(flat_e))))
+		    local_out_cnt++;
+	    }
+	    if ((local_in_cnt == 0) && (local_out_cnt == 0))
+		temprank[pos++] = v;
+	    else {
+		if ((MARK(v) == FALSE) && (local_in_cnt == 0)) {
+		    left = temprank + pos;
+		    n_search = postorder(g, v, left, r);
+		    if (GD_flip(g) == FALSE) {
+			right = left + n_search - 1;
+			while (left < right) {
+			    t = *left;
+			    *left = *right;
+			    *right = t;
+			    left++;
+			    right--;
+			}
 		    }
+		    pos += n_search;
 		}
-		pos += n_search;
 	    }
 	}
-    }
-    if (pos) {
-	for (i = 0; i < GD_rank(g)[r].n; i++) {
-		    v = GD_rank(g)[r].v[i] = temprank[i];
-		    ND_order(v) = i + (GD_rank(g)[r].v - GD_rank(Root)[r].v);
-	}
-    
+	if (pos) {
+	    for (i = 0; i < GD_rank(g)[r].n; i++) {
+		v = GD_rank(g)[r].v[i] = temprank[i];
+		ND_order(v) = i + (GD_rank(g)[r].v - GD_rank(Root)[r].v);
+	    }
+
 	    /* nonconstraint flat edges must be made LR */
 	    for (i = 0; i < GD_rank(g)[r].n; i++) {
-	    	v = GD_rank(g)[r].v[i];
-	    	if (ND_flat_out(v).list) {
-				for (j = 0; (e = ND_flat_out(v).list[j]); j++) {
-					if (ND_order(e->head) < ND_order(e->tail)) {
-						/*assert(ED_weight(e) == 0);*/
-						delete_flat_edge(e);
-						j--;
-						flat_rev(g,e);
-					}
-				}
+		v = GD_rank(g)[r].v[i];
+		if (ND_flat_out(v).list) {
+		    for (j = 0; (e = ND_flat_out(v).list[j]); j++) {
+			if (ND_order(aghead(e)) < ND_order(agtail(e))) {
+			    /*assert(ED_weight(e) == 0); */
+			    delete_flat_edge(e);
+			    j--;
+			    flat_rev(g, e);
 			}
+		    }
+		}
 	    }
 	}
 	/* else do no harm! */
@@ -1056,8 +1211,7 @@ for (r = GD_minrank(g); r <= GD_maxrank(g); r++) {
 	free(temprank);
 }
 
-static void 
-reorder(graph_t * g, int r, int reverse, int hasfixed)
+static void reorder(graph_t * g, int r, int reverse, int hasfixed)
 {
     int changed = 0, nelt;
     boolean muststay, sawclust;
@@ -1068,29 +1222,29 @@ reorder(graph_t * g, int r, int reverse, int hasfixed)
 	lp = vlist;
 	while (lp < ep) {
 	    /* find leftmost node that can be compared */
-	    while ((lp < ep) && ((*lp)->u.mval < 0))
+	    while ((lp < ep) && (ND_mval(*lp) < 0))
 		lp++;
 	    if (lp >= ep)
 		break;
 	    /* find the node that can be compared */
 	    sawclust = muststay = FALSE;
 	    for (rp = lp + 1; rp < ep; rp++) {
-		if (sawclust && (*rp)->u.clust)
+		if (sawclust && ND_clust(*rp))
 		    continue;	/* ### */
 		if (left2right(g, *lp, *rp)) {
 		    muststay = TRUE;
 		    break;
 		}
-		if ((*rp)->u.mval >= 0)
+		if (ND_mval(*rp) >= 0)
 		    break;
-		if ((*rp)->u.clust)
+		if (ND_clust(*rp))
 		    sawclust = TRUE;	/* ### */
 	    }
 	    if (rp >= ep)
 		break;
 	    if (muststay == FALSE) {
-		register int p1 = ((*lp)->u.mval);
-		register int p2 = ((*rp)->u.mval);
+		register int p1 = (ND_mval(*lp));
+		register int p2 = (ND_mval(*rp));
 		if ((p1 > p2) || ((p1 == p2) && (reverse))) {
 		    exchange(*lp, *rp);
 		    changed++;
@@ -1109,8 +1263,7 @@ reorder(graph_t * g, int r, int reverse, int hasfixed)
     }
 }
 
-static void 
-mincross_step(graph_t * g, int pass)
+static void mincross_step(graph_t * g, int pass)
 {
     int r, other, first, last, dir;
     int hasfixed, reverse;
@@ -1150,8 +1303,7 @@ mincross_step(graph_t * g, int pass)
     transpose(g, NOT(reverse));
 }
 
-static int 
-local_cross(elist l, int dir)
+static int local_cross(elist l, int dir)
 {
     int i, j, is_out;
     int cross = 0;
@@ -1163,23 +1315,20 @@ local_cross(elist l, int dir)
     for (i = 0; (e = l.list[i]); i++) {
 	if (is_out)
 	    for (j = i + 1; (f = l.list[j]); j++) {
-		if ((ND_order(f->head) -
-		     ND_order(e->head)) * (ED_tail_port(f).p.x -
-					   ED_tail_port(e).p.x) < 0)
+		if ((ND_order(aghead(f)) - ND_order(aghead(e)))
+			 * (ED_tail_port(f).p.x - ED_tail_port(e).p.x) < 0)
 		    cross += ED_xpenalty(e) * ED_xpenalty(f);
 	} else
 	    for (j = i + 1; (f = l.list[j]); j++) {
-		if ((ND_order(f->tail) -
-		     ND_order(e->tail)) * (ED_head_port(f).p.x -
-					   ED_head_port(e).p.x) < 0)
+		if ((ND_order(agtail(f)) - ND_order(agtail(e)))
+			* (ED_head_port(f).p.x - ED_head_port(e).p.x) < 0)
 		    cross += ED_xpenalty(e) * ED_xpenalty(f);
 	    }
     }
     return cross;
 }
 
-static int 
-rcross(graph_t * g, int r)
+static int rcross(graph_t * g, int r)
 {
     static int *Count, C;
     int top, bot, cross, max, i, k;
@@ -1200,13 +1349,13 @@ rcross(graph_t * g, int r)
     for (top = 0; top < GD_rank(g)[r].n; top++) {
 	register edge_t *e;
 	if (max > 0) {
-	    for (i = 0; (e = rtop[top]->u.out.list[i]); i++) {
-		for (k = ND_order(e->head) + 1; k <= max; k++)
+	    for (i = 0; (e = ND_out(rtop[top]).list[i]); i++) {
+		for (k = ND_order(aghead(e)) + 1; k <= max; k++)
 		    cross += Count[k] * ED_xpenalty(e);
 	    }
 	}
-	for (i = 0; (e = rtop[top]->u.out.list[i]); i++) {
-	    register int inv = ND_order(e->head);
+	for (i = 0; (e = ND_out(rtop[top]).list[i]); i++) {
+	    register int inv = ND_order(aghead(e));
 	    if (inv > max)
 		max = inv;
 	    Count[inv] += ED_xpenalty(e);
@@ -1243,8 +1392,7 @@ int ncross(graph_t * g)
     return count;
 }
 
-static int 
-ordercmpf(int *i0, int *i1)
+static int ordercmpf(int *i0, int *i1)
 {
     return (*i0) - (*i1);
 }
@@ -1259,8 +1407,7 @@ ordercmpf(int *i0, int *i1)
  * a.mval is > 0.
  * Return true if n.mval is left -1, indicating a fixed node for sorting.
  */
-static int 
-flat_mval(node_t * n)
+static int flat_mval(node_t * n)
 {
     int i;
     edge_t *e, **fl;
@@ -1268,20 +1415,20 @@ flat_mval(node_t * n)
 
     if (ND_flat_in(n).size > 0) {
 	fl = ND_flat_in(n).list;
-	nn = fl[0]->tail;
+	nn = agtail(fl[0]);
 	for (i = 1; (e = fl[i]); i++)
-	    if (ND_order(e->tail) > ND_order(nn))
-		nn = e->tail;
+	    if (ND_order(agtail(e)) > ND_order(nn))
+		nn = agtail(e);
 	if (ND_mval(nn) >= 0) {
 	    ND_mval(n) = ND_mval(nn) + 1;
 	    return FALSE;
 	}
     } else if (ND_flat_out(n).size > 0) {
 	fl = ND_flat_out(n).list;
-	nn = fl[0]->head;
+	nn = aghead(fl[0]);
 	for (i = 1; (e = fl[i]); i++)
-	    if (ND_order(e->head) < ND_order(nn))
-		nn = e->head;
+	    if (ND_order(aghead(e)) < ND_order(nn))
+		nn = aghead(e);
 	if (ND_mval(nn) > 0) {
 	    ND_mval(n) = ND_mval(nn) - 1;
 	    return FALSE;
@@ -1290,7 +1437,7 @@ flat_mval(node_t * n)
     return TRUE;
 }
 
-#define VAL(node,port) (MC_SCALE * (node)->u.order + (port).order)
+#define VAL(node,port) (MC_SCALE * ND_order(node) + (port).order)
 
 static boolean medians(graph_t * g, int r0, int r1)
 {
@@ -1307,11 +1454,11 @@ static boolean medians(graph_t * g, int r0, int r1)
 	if (r1 > r0)
 	    for (j0 = 0; (e = ND_out(n).list[j0]); j0++) {
 		if (ED_xpenalty(e) > 0)
-		    list[j++] = VAL(e->head, ED_head_port(e));
+		    list[j++] = VAL(aghead(e), ED_head_port(e));
 	} else
 	    for (j0 = 0; (e = ND_in(n).list[j0]); j0++) {
 		if (ED_xpenalty(e) > 0)
-		    list[j++] = VAL(e->tail, ED_tail_port(e));
+		    list[j++] = VAL(agtail(e), ED_tail_port(e));
 	    }
 	switch (j) {
 	case 0:
@@ -1352,12 +1499,12 @@ static boolean medians(graph_t * g, int r0, int r1)
 
 static int nodeposcmpf(node_t ** n0, node_t ** n1)
 {
-    return ((*n0)->u.order - (*n1)->u.order);
+    return (ND_order(*n0) - ND_order(*n1));
 }
 
 static int edgeidcmpf(edge_t ** e0, edge_t ** e1)
 {
-    return ((*e0)->id - (*e1)->id);
+    return (AGID(*e0) - AGID(*e1));
 }
 
 /* following code deals with weights of edges of "virtual" nodes */
@@ -1389,7 +1536,7 @@ static int endpoint_class(node_t * n)
 void virtual_weight(edge_t * e)
 {
     int t;
-    t = table[endpoint_class(e->tail)][endpoint_class(e->head)];
+    t = table[endpoint_class(agtail(e))][endpoint_class(aghead(e))];
     ED_weight(e) *= t;
 }
 
