@@ -1,4 +1,4 @@
-/* $Id: excc.c,v 1.1.1.1 2004/12/23 04:05:01 ellson Exp $ $Revision: 1.1.1.1 $ */
+/* $Id: excc.c,v 1.2 2008/08/01 19:31:29 erg Exp $ $Revision: 1.2 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -230,6 +230,14 @@ static void gen(Excc_t * cc, register Exnode_t * expr)
 	gen(cc, x);
 	sfprintf(cc->ccdisc->text, ");\n");
 	return;
+    case FUNCTION:
+	gen(cc, x);
+	sfprintf(cc->ccdisc->text, "(");
+	if ((y = expr->data.operand.right)) {
+	    gen(cc, y);
+	}
+	sfprintf(cc->ccdisc->text, ")");
+	return;
     case RAND:
 	sfprintf(cc->ccdisc->text, "rand();\n");
 	return;
@@ -486,6 +494,16 @@ static void gen(Excc_t * cc, register Exnode_t * expr)
 	gen(cc, x);
 	sfprintf(cc->ccdisc->text, ",(char**)0,0)");
 	return;
+     case X2I:
+	sfprintf(cc->ccdisc->text, "X2I(");
+	gen(cc, x);
+	sfprintf(cc->ccdisc->text, ")");
+	return;
+     case X2X:
+	sfprintf(cc->ccdisc->text, "X2X(");
+	gen(cc, x);
+	sfprintf(cc->ccdisc->text, ")");
+	return;
     }
     y = expr->data.operand.right;
     if (x->type == STRING) {
