@@ -1,4 +1,4 @@
-/* $Id: exeval.c,v 1.5 2007/07/17 21:49:24 erg Exp $ $Revision: 1.5 $ */
+/* $Id: exeval.c,v 1.7 2008/08/01 19:32:26 erg Exp $ $Revision: 1.7 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -1298,7 +1298,7 @@ static Extype_t eval(Expr_t * ex, register Exnode_t * expr, void *env)
     v = eval(ex, x, env);
     if ((x = expr->data.operand.right)) {
 	r = eval(ex, x, env);
-	if (!BUILTIN(x->type)) {
+	if (!BUILTIN(x->type) && expr->binary) {
 	    tmp = *expr->data.operand.left;
 	    tmp.data.constant.value = v;
 	    rtmp = *x;
@@ -1619,9 +1619,13 @@ static Extype_t eval(Expr_t * ex, register Exnode_t * expr, void *env)
 				       data.operand.right->data.variable.
 				       symbol : (Exid_t *) 0, 0,
 				       ex->disc)) {
-		tmp.data.constant.value.integer = strToL(v.string, &e);
-		if (*e)
-		    tmp.data.constant.value.integer = *v.string != 0;
+		if (v.string) {
+		    tmp.data.constant.value.integer = strToL(v.string, &e);
+		    if (*e)
+			tmp.data.constant.value.integer = *v.string != 0;
+		}
+		else
+		    tmp.data.constant.value.integer = 0;
 	    }
 	    tmp.type = INTEGER;
 	    return tmp.data.constant.value;
