@@ -1,4 +1,4 @@
-/* $Id: xlayout.c,v 1.9 2008/04/08 21:34:51 erg Exp $ $Revision: 1.9 $ */
+/* $Id: xlayout.c,v 1.10 2008/10/21 21:42:04 ellson Exp $ $Revision: 1.10 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -409,7 +409,7 @@ static int adjust(Agraph_t * g, double temp)
 	    overlaps += ov;
 	}
 	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
-	    applyAttr(n, e->head);
+	    applyAttr(n,aghead(e));
 	}
     }
     if (overlaps == 0)
