@@ -1,4 +1,4 @@
-/* $Id: gvcommon.h,v 1.7 2006/12/07 22:49:36 erg Exp $ $Revision: 1.7 $ */
+/* $Id: gvcommon.h,v 1.12 2009/02/25 17:46:45 ellson Exp $ $Revision: 1.12 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -22,18 +22,19 @@ extern "C" {
 #endif
 
     typedef struct GVCOMMON_s {
-	char *user;
 	char **info;
 	char *cmdname;
 	int verbose;
 	boolean config, auto_outfile_names;
-        void (*errorfn) (char *fmt, ...);
-	char **show_boxes; /* emit code for correct box coordinates */
-	char **lib; 
+        void (*errorfn) (const char *fmt, ...);
+	const char **show_boxes; /* emit code for correct box coordinates */
+	const char **lib; 
 
 	/* rendering state */
 	int viewNum;     /* current view - 1 based count of views,
 			    all pages in all layers */
+	const lt_symlist_t *builtins;
+	int demand_loading;
     } GVCOMMON_t;
 
 #ifdef __cplusplus
