@@ -1,4 +1,4 @@
-/* $Id: gvconfig.c,v 1.87 2008/06/06 14:29:54 ellson Exp $ $Revision: 1.87 $ */
+/* $Id: gvconfig.c,v 1.107 2009/02/25 17:46:45 ellson Exp $ $Revision: 1.107 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -18,6 +18,8 @@
 #include "config.h"
 #endif
 
+#include "gvconfig.h"
+
 #include	<string.h>
 
 #ifdef ENABLE_LTDL
@@ -38,7 +40,7 @@ typedef struct {
     char **gl_pathv;        /* list of paths matching pattern */
 } glob_t;
 static void globfree (glob_t* pglob);
-static int glob (char*, int, int (*errfunc)(const char *, int), glob_t*);
+static int glob (GVC_t * gvc, char*, int, int (*errfunc)(const char *, int), glob_t*);
 #else
 #include        <regex.h>
 #include	<glob.h>
@@ -52,15 +54,12 @@ static int glob (char*, int, int (*errfunc)(const char *, int), glob_t*);
 #include        "memory.h"
 #include        "const.h"
 #include        "types.h"
-#include        "graph.h"
 
 #include	"gvplugin.h"
 #include	"gvcjob.h"
 #include	"gvcint.h"
 #include        "gvcproc.h"
 
-extern const int Demand_Loading;
-
 #ifdef WITH_CODEGENS
     extern codegen_t HPGL_CodeGen, MIF_CodeGen, MP_CodeGen, PIC_CodeGen, DIA_CodeGen, VTX_CodeGen;
 #endif
@@ -94,6 +93,16 @@ extern const int Demand_Loading;
 
  */
 
+static gvplugin_package_t * gvplugin_package_record(GVC_t * gvc, char *path, char *name)
+{
+    gvplugin_package_t *package = gmalloc(sizeof(gvplugin_package_t));
+    package->path = (path) ? strdup(path) : NULL;
+    package->name = strdup(name);
+    package->next = gvc->packages;
+    gvc->packages = package;
+    return package;
+}
+
 #ifdef ENABLE_LTDL
 /*
   separator - consume all non-token characters until next token.  This includes:
@@ -162,18 +171,21 @@ static char *token(int *nest, char **tokens)
 
 static int gvconfig_plugin_install_from_config(GVC_t * gvc, char *s)
 {
-    char *path, *packagename, *api, *type;
+    char *path, *name, *api;
+    const char *type;
     api_t gv_api;
     int quality, rc;
     int nest = 0;
+    gvplugin_package_t *package;
 
     separator(&nest, &s);
     while (*s) {
 	path = token(&nest, &s);
 	if (nest == 0)
-	    packagename = token(&nest, &s);
+	    name = token(&nest, &s);
         else
-	    packagename = "x";
+	    name = "x";
+        package = gvplugin_package_record(gvc, path, name);
 	do {
 	    api = token(&nest, &s);
 	    gv_api = gvplugin_api(api);
@@ -189,7 +201,7 @@ static int gvconfig_plugin_install_from_config(GVC_t * gvc, char *s)
 		    else
 		        quality = 0;
 		    rc = gvplugin_install (gvc, gv_api,
-				    type, quality, packagename, path, NULL);
+				    type, quality, package, NULL);
 		    if (!rc) {
 		        agerr(AGERR, "config error: %s %s %s\n", path, api, type);
 		        return 0;
@@ -202,19 +214,18 @@ static int gvconfig_plugin_install_from_config(GVC_t * gvc, char *s)
 }
 #endif
 
-static void gvconfig_plugin_install_from_library(GVC_t * gvc, char *path, gvplugin_library_t *library)
+void gvconfig_plugin_install_from_library(GVC_t * gvc, char *path, gvplugin_library_t *library)
 {
     gvplugin_api_t *apis;
     gvplugin_installed_t *types;
+    gvplugin_package_t *package;
     int i;
 
+    package = gvplugin_package_record(gvc, path, library->packagename);
     for (apis = library->apis; (types = apis->types); apis++) {
 	for (i = 0; types[i].type; i++) {
-	    /* FIXME - should only install if dependencies on other plugins are satisfied */
-	    /*         e.g. "render" "gtk" depends on "device" "gtk" */
-	    /*         only need to check during actual loading, so no need to store dependencies in config */
 	    gvplugin_install(gvc, apis->api, types[i].type,
-			types[i].quality, library->packagename, path, &types[i]);
+			types[i].quality, package, &types[i]);
         }
     }
 }
@@ -224,14 +235,9 @@ static void gvconfig_plugin_install_builtins(GVC_t * gvc)
     const lt_symlist_t *s;
     const char *name;
 
-/* For Windows DLLs using builtins, we need to initialize
- * the lt_preloaded_symbols table.
- */
-#if defined(GVDLL) && !defined(ENABLE_LTDL)
-    init_lt_preloaded_symbols();
-#endif
-    s = lt_preloaded_symbols;
-    for (s = lt_preloaded_symbols; (name = s->name); s++)
+    if (gvc->common.builtins == NULL) return;
+
+    for (s = gvc->common.builtins; (name = s->name); s++)
 	if (name[0] == 'g' && strstr(name, "_LTX_library")) 
 	    gvconfig_plugin_install_from_library(gvc, NULL,
 		    (gvplugin_library_t *)(s->address));
@@ -260,7 +266,7 @@ static void gvconfig_write_library_config(GVC_t *gvc, char *path, gvplugin_libra
 
 #define BSZ 1024
 
-char * gvconfig_libdir(void)
+char * gvconfig_libdir(GVC_t * gvc)
 {
     static char line[BSZ];
     static char *libdir;
@@ -320,6 +326,9 @@ char * gvconfig_libdir(void)
 	    }
 #endif
 	}
+	if (gvc->common.verbose > 1) 
+	    fprintf (stderr, "libdir = \"%s\"\n",
+		(libdir ? libdir : "<null>"));
     }
     return libdir;
 }
@@ -374,7 +383,7 @@ static void config_rescan(GVC_t *gvc, char *config_path)
 	fprintf(f, "# Manual edits to this file **will be lost** on upgrade.\n\n");
     }
 
-    libdir = gvconfig_libdir();
+    libdir = gvconfig_libdir(gvc);
 
     config_re = gmalloc(strlen(plugin_re_beg) + 20 + strlen(plugin_re_end) + 1);
 
@@ -396,7 +405,12 @@ static void config_rescan(GVC_t *gvc, char *config_path)
     strcat(config_glob, plugin_glob);
 
     /* load all libraries even if can't save config */
+
+#if defined(WIN32)
+    rc = glob(gvc, config_glob, GLOB_NOSORT, NULL, &globbuf);
+#else
     rc = glob(config_glob, GLOB_NOSORT, NULL, &globbuf);
+#endif
     if (rc == 0) {
 	for (i = 0; i < globbuf.gl_pathc; i++) {
 	    re_status = regexec(&re, globbuf.gl_pathv[i], (size_t) 0, NULL, 0);
@@ -474,7 +488,7 @@ void gvconfig(GVC_t * gvc, boolean rescan)
     FILE *f = NULL;
     char *config_text = NULL;
     char *libdir;
-    char *config_file_name = "config";
+    char *config_file_name = GVPLUGIN_CONFIG_FILE;
 
 #define MAX_SZ_CONFIG 100000
 #endif
@@ -482,9 +496,12 @@ void gvconfig(GVC_t * gvc, boolean rescan)
 #ifdef WITH_CODEGENS
     codegen_info_t *p;
 
+    gvplugin_package_t *package;
+
+    package = gvplugin_package_record(gvc, NULL, "cg");
     for (p = cg; p->name; ++p)
         gvplugin_install(gvc, API_device, p->name, 0,
-                        "cg", NULL, (gvplugin_installed_t *) p);
+		package, (gvplugin_installed_t *) p);
 #endif
 
     /* builtins don't require LTDL */
@@ -492,9 +509,9 @@ void gvconfig(GVC_t * gvc, boolean rescan)
    
     gvc->config_found = FALSE;
 #ifdef ENABLE_LTDL
-    if (Demand_Loading) {
+    if (gvc->common.demand_loading) {
         /* see if there are any new plugins */
-        libdir = gvconfig_libdir();
+        libdir = gvconfig_libdir(gvc);
         rc = stat(libdir, &libdir_st);
         if (rc == -1) {
     	    /* if we fail to stat it then it probably doesn't exist so just fail silently */
@@ -528,6 +545,7 @@ void gvconfig(GVC_t * gvc, boolean rescan)
     	    f = fopen(gvc->config_path,"r");
     	    if (!f) {
     	        agerr (AGERR,"failed to open %s for read.\n", gvc->config_path);
+		exit(1);
     	    }
     	    else {
     	        config_text = gmalloc(config_st.st_size + 1);
@@ -561,11 +579,9 @@ void gvconfig(GVC_t * gvc, boolean rescan)
  * Assumes only GLOB_NOSORT flag given. That is, there is no offset,
  * and no previous call to glob.
  */
-char * gvconfig_libdir(void);
 
 static int
-glob (char* pattern, int flags, int (*errfunc)(const char *, int),
-         glob_t *pglob)
+glob (GVC_t* gvc, char* pattern, int flags, int (*errfunc)(const char *, int), glob_t *pglob)
 {
     char* libdir;
     WIN32_FIND_DATA wfd;
@@ -579,7 +595,7 @@ glob (char* pattern, int flags, int (*errfunc)(const char *, int),
     
     h = FindFirstFile (pattern, &wfd);
     if (h == INVALID_HANDLE_VALUE) return GLOB_NOMATCH;
-    libdir = gvconfig_libdir();
+    libdir = gvconfig_libdir(gvc);
     do {
       if (cnt >= arrsize-1) {
         arrsize += 512;
