@@ -1,4 +1,4 @@
-/* $Id: gvplugin_loadimage.h,v 1.5 2008/04/18 14:10:43 glenlow Exp $ $Revision: 1.5 $ */
+/* $Id: gvplugin_loadimage.h,v 1.10 2009/01/21 16:47:00 arif Exp $ $Revision: 1.10 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -21,13 +21,22 @@
 #include "gvplugin.h"
 #include "gvcjob.h"
 
-extern void gvdevice_fputs(GVJ_t * job, char *s);
-extern void gvdevice_printf(GVJ_t * job, const char *format, ...);
-
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#ifdef GVDLL
+#  define extern __declspec(dllexport)
+#endif
+
+/*visual studio*/
+#ifdef WIN32_DLL
+#ifndef GVC_EXPORTS
+#define extern __declspec(dllimport)
+#endif
+#endif
+/*end visual studio*/
+
 extern boolean gvusershape_file_access(usershape_t *us);
 extern void gvusershape_file_release(usershape_t *us);
 
@@ -35,6 +44,10 @@ extern void gvusershape_file_release(usershape_t *us);
 	void (*loadimage) (GVJ_t *job, usershape_t *us, boxf b, boolean filled);
     };
 
+#ifdef extern
+#undef extern
+#endif
+
 #ifdef __cplusplus
 }
 #endif
