@@ -1,4 +1,4 @@
-/* $Id: gvrender.c,v 1.144 2008/06/06 18:15:54 ellson Exp $ $Revision: 1.144 $ */
+/* $Id: gvrender.c,v 1.157 2009/02/24 19:47:18 ellson Exp $ $Revision: 1.157 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -30,7 +30,11 @@
 #include "macros.h"
 #include "colorprocs.h"
 #include "gvplugin_render.h"
+#if WITH_CGRAPH
+#include "cgraph.h"
+#else
 #include "graph.h"
+#endif
 #include "gvcint.h"
 #include "geom.h"
 #include "geomprocs.h"
@@ -49,7 +53,7 @@ static int sizeA;
 #endif
 /* end hack */
 
-int gvrender_select(GVJ_t * job, char *str)
+int gvrender_select(GVJ_t * job, const char *str)
 {
     GVC_t *gvc = job->gvc;
     gvplugin_available_t *plugin;
@@ -60,11 +64,15 @@ int gvrender_select(GVJ_t * job, char *str)
 
     gvplugin_load(gvc, API_device, str);
 
-    job->flags = 0;
+    /* When job is created, it is zeroed out.
+     * Some flags, such as OUTPUT_NOT_REQUIRED, may already be set,
+     * so don't reset.
+     */
+    /* job->flags = 0; */
     plugin = gvc->api[API_device];
     if (plugin) {
 #ifdef WITH_CODEGENS
-	if (strcmp(plugin->packagename, "cg") == 0) {
+	if (strcmp(plugin->package->name, "cg") == 0) {
 	    cg_info = (codegen_info_t *) (plugin->typeptr);
 	    job->codegen = cg_info->cg;
 	    job->render.engine = NULL;
@@ -131,7 +139,9 @@ int gvrender_features(GVJ_t * job)
 
 void gvrender_begin_job(GVJ_t * job)
 {
+#ifdef WITH_CODEGENS
     GVC_t *gvc = job->gvc;
+#endif
     gvrender_engine_t *gvre = job->render.engine;
 
     gvdevice_initialize(job);
@@ -146,7 +156,6 @@ void gvrender_begin_job(GVJ_t * job)
 	if (cg && cg->begin_job)
 	    cg->begin_job(job->output_file, gvc->g,
 		    gvc->common.lib,
-		    gvc->common.user,
 		    gvc->common.info,
 		    job->pagesArraySize);
     }
@@ -586,13 +595,13 @@ void gvrender_end_context(GVJ_t * job)
 #endif
 }
 
-void gvrender_begin_anchor(GVJ_t * job, char *href, char *tooltip, char *target)
+void gvrender_begin_anchor(GVJ_t * job, char *href, char *tooltip, char *target, char *id)
 {
     gvrender_engine_t *gvre = job->render.engine;
 
     if (gvre) {
 	if (gvre->begin_anchor)
-	    gvre->begin_anchor(job, href, tooltip, target);
+	    gvre->begin_anchor(job, href, tooltip, target, id);
     }
 #ifdef WITH_CODEGENS
     else {
@@ -622,11 +631,31 @@ void gvrender_end_anchor(GVJ_t * job)
 #endif
 }
 
-void gvrender_set_font(GVJ_t * job, char *fontname, double fontsize)
+void gvrender_begin_label(GVJ_t * job, label_type type)
 {
     gvrender_engine_t *gvre = job->render.engine;
+	
+    if (gvre) {
+		if (gvre->begin_label)
+			gvre->begin_label(job, type);
+    }
+}
 
+void gvrender_end_label(GVJ_t * job)
+{
+    gvrender_engine_t *gvre = job->render.engine;
+	
+    if (gvre) {
+		if (gvre->end_label)
+			gvre->end_label(job);
+    }
+}
+
+void gvrender_set_font(GVJ_t * job, char *fontname, double fontsize)
+{
 #ifdef WITH_CODEGENS
+    gvrender_engine_t *gvre = job->render.engine;
+
     if (!gvre) {
 	codegen_t *cg = job->codegen;
 
