@@ -1,4 +1,4 @@
-/* $Id: gvusershape.c,v 1.36 2008/03/15 23:19:30 ellson Exp $ $Revision: 1.36 $ */
+/* $Id: gvusershape.c,v 1.48 2009/01/06 22:58:41 arif Exp $ $Revision: 1.48 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -25,9 +25,12 @@
 #include "types.h"
 #include "logic.h"
 #include "memory.h"
-#include "graph.h"
 #include "agxbuf.h"
+
+#define _BLD_gvc 1
 #include "utils.h"
+#include "gvplugin_loadimage.h"
+
 
 extern shape_desc *find_user_shape(char *);
 
@@ -147,7 +150,7 @@ static void svg_size (usershape_t *us)
     char u[10];
     char *token;
     char line[200];
-    bool wFlag = false, hFlag = false;
+    boolean wFlag = FALSE, hFlag = FALSE;
 
     fseek(us->f, -strlen(line), SEEK_CUR);
     while (fgets(line, sizeof(line), us->f) != NULL && (!wFlag || !hFlag)) {
@@ -155,13 +158,13 @@ static void svg_size (usershape_t *us)
         while (token != NULL && token[strlen(token)-1] != '>') {
 	    if (sscanf(token, "width=\"%lf%2s\"", &n, u) == 2) {
 	        w = svg_units_convert(n, u);
-	        wFlag = true;
+	        wFlag = TRUE;
 		if (hFlag)
 		    break;
 	    }
 	    if (sscanf(token, "height=\"%lf%2s\"", &n, u) == 2) {
 	        h = svg_units_convert(n, u);
-	        hFlag = true;
+	        hFlag = TRUE;
                 if (wFlag)
 		    break;
 	    }
@@ -291,12 +294,23 @@ static void ps_size (usershape_t *us)
     char line[BUFSIZ];
     boolean saw_bb;
     int lx, ly, ux, uy;
+    char* linep;
 
     us->dpi = POINTS_PER_INCH;
     fseek(us->f, 0, SEEK_SET);
     saw_bb = FALSE;
     while (fgets(line, sizeof(line), us->f)) {
-        if (sscanf (line, "%%%%BoundingBox: %d %d %d %d", &lx, &ly, &ux, &uy) == 4) {
+	/* PostScript accepts \r as EOL, so using fgets () and looking for a
+	 * bounding box comment at the beginning doesn't work in this case. 
+	 * As a heuristic, we first search for a bounding box comment in line.
+	 * This obviously fails if not all of the numbers make it into the
+	 * current buffer. This shouldn't be a problem, as the comment is
+	 * typically near the beginning, and so should be read within the first
+	 * BUFSIZ bytes (even on Windows where this is 512).
+	 */
+	if (!(linep = strstr (line, "%%BoundingBox:")))
+	    continue;
+        if (sscanf (linep, "%%%%BoundingBox: %d %d %d %d", &lx, &ly, &ux, &uy) == 4) {
             saw_bb = TRUE;
 	    break;
         }
@@ -347,7 +361,7 @@ usershape_t *gvusershape_find(char *name)
 boolean gvusershape_file_access(usershape_t *us)
 {
     static int usershape_files_open_cnt;
-    char *fn;
+    const char *fn;
 
     assert(us);
     assert(us->name);
@@ -363,15 +377,15 @@ boolean gvusershape_file_access(usershape_t *us)
 #endif
 	    if (us->f == NULL) {
 		agerr(AGWARN, "%s while opening %s\n", strerror(errno), fn);
-		return false;
+		return FALSE;
 	    }
 	    if (usershape_files_open_cnt >= MAX_USERSHAPE_FILES_OPEN)
-		us->nocache = true;
+		us->nocache = TRUE;
 	    else
 	        usershape_files_open_cnt++;
 	}
     }
-    return true;
+    return TRUE;
 }
 
 void gvusershape_file_release(usershape_t *us)
