@@ -1,3 +1,4 @@
+/* $Id: delaunay.h,v 1.3 2008/07/25 15:35:06 ellson Exp $ $Revision: 1.3 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -18,10 +19,22 @@
 
 #include "sparsegraph.h"
 
+typedef struct {
+    int  nedges; /* no. of edges in triangulation */
+    int* edges;  /* 2*nsegs indices of points */
+    int  nfaces; /* no. of faces in triangulation */
+    int* faces;  /* 3*nfaces indices of points */ 
+    int* neigh;  /* 3*nfaces indices of neighbor triangles */ 
+} surface_t;
+
 v_data *delaunay_triangulation(double *x, double *y, int n);
 
 int *delaunay_tri (double *x, double *y, int n, int* nedges);
 
 v_data *UG_graph(double *x, double *y, int n, int accurate_computation);
 
+surface_t* mkSurface (double *x, double *y, int n, int* segs, int nsegs);
+
+void freeSurface (surface_t* s);
+
 #endif
