@@ -1,4 +1,4 @@
-/* $Id: quad_prog_vpsc.c,v 1.5 2007/08/16 18:10:13 erg Exp $ $Revision: 1.5 $ */
+/* $Id: quad_prog_vpsc.c,v 1.6 2008/04/28 16:52:59 erg Exp $ $Revision: 1.6 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**
@@ -143,7 +143,10 @@ constrained_majorization_vpsc(CMajEnvVPSC * e, float *b, float *place,
 	    }
 	    denominator += r * d[i];
 	}
-	beta = numerator / denominator;
+	if (denominator != 0.0)
+	    beta = numerator / denominator;
+	else
+	    beta = 1.0;
 
 	for (i = 0; i < n; i++) {
 	    /* beta > 1.0 takes us back outside the feasible region
