@@ -1,4 +1,4 @@
-/* $Id: stress.c,v 1.8 2008/04/25 14:38:29 erg Exp $ $Revision: 1.8 $ */
+/* $Id: stress.c,v 1.9 2008/10/21 21:24:34 ellson Exp $ $Revision: 1.9 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -1819,10 +1819,19 @@ int stress_majorization_kD_mkernel(vtx_data * graph,	/* Input graph in sparse re
 		compute_stressf(coords, lap2, dim, n),
 		iterations, elapsed_sec());
     }
+#ifdef WITH_CGRAPH
+    if (Verbose)
+	fprintf (stderr, "coords\n");
+#endif /* WITH_CGRAPH */
 
     for (i = 0; i < dim; i++) {
 	for (j = 0; j < n; j++) {
 	    d_coords[i][j] = coords[i][j];
+#ifdef WITH_CGRAPH
+            if (Verbose)
+		fprintf (stderr, "%d\n",coords[i][j]);
+			
+#endif /* WITH_CGRAPH */
 	}
     }
     free(coords[0]);
