@@ -1,4 +1,4 @@
-/* $Id: pack.c,v 1.6 2006/12/07 22:49:37 erg Exp $ $Revision: 1.6 $ */
+/* $Id: pack.c,v 1.14 2008/10/22 18:17:13 ellson Exp $ $Revision: 1.14 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -20,17 +20,16 @@
  * K. Freivalds et al., GD0'01, LNCS 2265, pp. 378-391.
  */
 
-#include <render.h>
-#include <pack.h>
-#include <pointset.h>
 #include <math.h>
+#include "render.h"
+#include "pack.h"
+#include "pointset.h"
 
 #define C 100			/* Max. avg. polyomino size */
 
 #define MOVEPT(p) ((p).x += dx, (p).y += dy)
 #define GRID(x,s) (((x) + ((s)-1)) / (s))
 #define CELL(p,s) ((p).x = (p).x/(s), (p).y = ((p).y/(s)))
-#define SGN(a)    (((a)<0)? -1 : 1)
 
 typedef struct {
     Agraph_t *graph;		/* related graph */
@@ -101,12 +100,12 @@ static int cmpf(const void *X, const void *Y)
  * Bresenham's algorithm, from Graphics Gems I, pp. 99-100.
  */
 /* static  */
-void fillLine(point p, point q, PointSet * ps)
+void fillLine(pointf p, pointf q, PointSet * ps)
 {
-    int x1 = p.x;
-    int y1 = p.y;
-    int x2 = q.x;
-    int y2 = q.y;
+    int x1 = ROUND(p.x);
+    int y1 = ROUND(p.y);
+    int x2 = ROUND(q.x);
+    int y2 = ROUND(q.y);
     int d, x, y, ax, ay, sx, sy, dx, dy;
 
     dx = x2 - x1;
@@ -119,34 +118,34 @@ void fillLine(point p, point q, PointSet * ps)
 /* fprintf (stderr, "fillLine %d %d - %d %d\n", x1,y1,x2,y2); */
     x = x1;
     y = y1;
-    if (ax > ay) {		/* x dominant */
-	d = ay - (ax >> 1);
-	for (;;) {
+    if (ax > ay) {              /* x dominant */
+        d = ay - (ax >> 1);
+        for (;;) {
 /* fprintf (stderr, "  addPS %d %d\n", x,y); */
-	    addPS(ps, x, y);
-	    if (x == x2)
-		return;
-	    if (d >= 0) {
-		y += sy;
-		d -= ax;
-	    }
-	    x += sx;
-	    d += ay;
-	}
-    } else {			/* y dominant */
-	d = ax - (ay >> 1);
-	for (;;) {
+            addPS(ps, x, y);
+            if (x == x2)
+                return;
+            if (d >= 0) {
+                y += sy;
+                d -= ax;
+            }
+            x += sx;
+            d += ay;
+        }
+    } else {                    /* y dominant */
+        d = ax - (ay >> 1);
+        for (;;) {
 /* fprintf (stderr, "  addPS %d %d\n", x,y); */
-	    addPS(ps, x, y);
-	    if (y == y2)
-		return;
-	    if (d >= 0) {
-		x += sx;
-		d -= ay;
-	    }
-	    y += sy;
-	    d += ax;
-	}
+            addPS(ps, x, y);
+            if (y == y2)
+                return;
+            if (d >= 0) {
+                x += sx;
+                d -= ay;
+            }
+            y += sy;
+            d += ax;
+        }
     }
 }
 
@@ -155,17 +154,19 @@ void fillLine(point p, point q, PointSet * ps)
  * beginning and the end point at the end.
  */
 static void
-fillEdge(Agedge_t * e, point pt, PointSet * ps, int dx, int dy,
+fillEdge(Agedge_t * e, point p, PointSet * ps, int dx, int dy,
 	 int ssize, int doS)
 {
     int j, k;
     bezier bz;
-    point hpt;
+    pointf pt, hpt;
     Agnode_t *h;
 
+    P2PF(p, pt);
+
     /* If doS is false or the edge has not splines, use line segment */
     if (!doS || !ED_spl(e)) {
-	h = e->head;
+	h = aghead(e);
 	hpt = coord(h);
 	MOVEPT(hpt);
 	CELL(hpt, ssize);
@@ -219,9 +220,10 @@ genBox(Agraph_t * g, ginfo * info, int ssize, int margin, point center)
     PointSet *ps;
     int W, H;
     point UR, LL;
-    box bb = GD_bb(g);
+    box bb;
     int x, y;
 
+    BF2B(GD_bb(g), bb);
     ps = newPS();
 
     LL.x = center.x - margin;
@@ -244,8 +246,8 @@ genBox(Agraph_t * g, ginfo * info, int ssize, int margin, point center)
 
     if (Verbose > 2) {
 	int i;
-	fprintf(stderr, "%s no. cells %d W %d H %d\n", g->name, info->nc,
-		W, H);
+	fprintf(stderr, "%s no. cells %d W %d H %d\n",
+		agnameof(g), info->nc, W, H);
 	for (i = 0; i < info->nc; i++)
 	    fprintf(stderr, "  %d %d cell\n", info->cells[i].x,
 		    info->cells[i].y);
@@ -272,6 +274,7 @@ genPoly(Agraph_t * root, Agraph_t * g, ginfo * info,
     int W, H;
     point LL, UR;
     point pt, s2;
+    pointf ptf;
     Agraph_t *eg;		/* graph containing edges */
     Agnode_t *n;
     Agedge_t *e;
@@ -288,8 +291,8 @@ genPoly(Agraph_t * root, Agraph_t * g, ginfo * info,
 	eg = g;
 
     ps = newPS();
-    dx = center.x - GD_bb(g).LL.x;
-    dy = center.y - GD_bb(g).LL.y;
+    dx = center.x - ROUND(GD_bb(g).LL.x);
+    dy = center.y - ROUND(GD_bb(g).LL.y);
 
     if (pinfo->mode == l_clust) {
 	int i;
@@ -298,14 +301,14 @@ genPoly(Agraph_t * root, Agraph_t * g, ginfo * info,
 	/* backup the alg data */
 	alg = N_GNEW(agnnodes(g), void *);
 	for (i = 0, n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	    alg[i++] = n->u.alg;
-	    n->u.alg = 0;
+	    alg[i++] = ND_alg(n);
+	    ND_alg(n) = 0;
 	}
 
 	/* do bbox of top clusters */
 	for (i = 1; i <= GD_n_cluster(g); i++) {
 	    subg = GD_clust(g)[i];
-	    bb = GD_bb(subg);
+	    BF2B(GD_bb(subg), bb);
 	    if ((bb.UR.x > bb.LL.x) && (bb.UR.y > bb.LL.y)) {
 		MOVEPT(bb.LL);
 		MOVEPT(bb.UR);
@@ -328,13 +331,14 @@ genPoly(Agraph_t * root, Agraph_t * g, ginfo * info,
 
 	/* now do remaining nodes and edges */
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	    pt = coord(n);
+	    ptf = coord(n);
+	    PF2P(ptf, pt);
 	    MOVEPT(pt);
 	    if (!ND_clust(n)) {	/* n is not in a top-level cluster */
 		s2.x = margin + ND_xsize(n) / 2;
 		s2.y = margin + ND_ysize(n) / 2;
-		LL = sub_points(pt, s2);
-		UR = add_points(pt, s2);
+		LL = sub_point(pt, s2);
+		UR = add_point(pt, s2);
 		CELL(LL, ssize);
 		CELL(UR, ssize);
 
@@ -349,7 +353,7 @@ genPoly(Agraph_t * root, Agraph_t * g, ginfo * info,
 	    } else {
 		CELL(pt, ssize);
 		for (e = agfstout(eg, n); e; e = agnxtout(eg, e)) {
-		    if (ND_clust(n) == ND_clust(e->head))
+		    if (ND_clust(n) == ND_clust(aghead(e)))
 			continue;
 		    fillEdge(e, pt, ps, dx, dy, ssize, doSplines);
 		}
@@ -358,18 +362,19 @@ genPoly(Agraph_t * root, Agraph_t * g, ginfo * info,
 
 	/* restore the alg data */
 	for (i = 0, n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	    n->u.alg = alg[i++];
+	    ND_alg(n)= alg[i++];
 	}
 	free(alg);
 
     } else
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	    pt = coord(n);
+	    ptf = coord(n);
+	    PF2P(ptf, pt);
 	    MOVEPT(pt);
 	    s2.x = margin + ND_xsize(n) / 2;
 	    s2.y = margin + ND_ysize(n) / 2;
-	    LL = sub_points(pt, s2);
-	    UR = add_points(pt, s2);
+	    LL = sub_point(pt, s2);
+	    UR = add_point(pt, s2);
 	    CELL(LL, ssize);
 	    CELL(UR, ssize);
 
@@ -386,14 +391,14 @@ genPoly(Agraph_t * root, Agraph_t * g, ginfo * info,
     info->graph = g;
     info->cells = pointsOf(ps);
     info->nc = sizeOf(ps);
-    W = GRID(GD_bb(g).UR.x - GD_bb(g).LL.x + 2 * margin, ssize);
-    H = GRID(GD_bb(g).UR.y - GD_bb(g).LL.y + 2 * margin, ssize);
+    W = GRID(ROUND(GD_bb(g).UR.x - GD_bb(g).LL.x) + 2 * margin, ssize);
+    H = GRID(ROUND(GD_bb(g).UR.y - GD_bb(g).LL.y) + 2 * margin, ssize);
     info->perim = W + H;
 
     if (Verbose > 2) {
 	int i;
-	fprintf(stderr, "%s no. cells %d W %d H %d\n", g->name, info->nc,
-		W, H);
+	fprintf(stderr, "%s no. cells %d W %d H %d\n",
+		agnameof(g), info->nc, W, H);
 	for (i = 0; i < info->nc; i++)
 	    fprintf(stderr, "  %d %d cell\n", info->cells[i].x,
 		    info->cells[i].y);
@@ -425,7 +430,7 @@ fits(int x, int y, ginfo * info, PointSet * ps, point * place, int step)
 	cells++;
     }
 
-    LL = GD_bb(info->graph).LL;
+    PF2P(GD_bb(info->graph).LL, LL);
     place->x = step * x - LL.x;
     place->y = step * y - LL.y;
 
@@ -484,16 +489,16 @@ placeGraph(int i, ginfo * info, PointSet * ps, point * place, int step,
 
     if (i == 0) {
 	Agraph_t *g = info->graph;
-	W = GRID(GD_bb(g).UR.x - GD_bb(g).LL.x + 2 * margin, step);
-	H = GRID(GD_bb(g).UR.y - GD_bb(g).LL.y + 2 * margin, step);
+	W = GRID(ROUND(GD_bb(g).UR.x - GD_bb(g).LL.x) + 2 * margin, step);
+	H = GRID(ROUND(GD_bb(g).UR.y - GD_bb(g).LL.y) + 2 * margin, step);
 	if (fits(-W / 2, -H / 2, info, ps, place, step))
 	    return;
     }
 
     if (fits(0, 0, info, ps, place, step))
 	return;
-    W = GD_bb(info->graph).UR.x - GD_bb(info->graph).LL.x;
-    H = GD_bb(info->graph).UR.y - GD_bb(info->graph).LL.y;
+    W = ROUND(GD_bb(info->graph).UR.x - GD_bb(info->graph).LL.x);
+    H = ROUND(GD_bb(info->graph).UR.y - GD_bb(info->graph).LL.y);
     if (W >= H) {
 	for (bnd = 1;; bnd++) {
 	    x = 0;
@@ -581,7 +586,7 @@ point *putGraphs(int ng, Agraph_t ** gs, Agraph_t * root,
     int i;
     boolean *fixed = pinfo->fixed;
     int fixed_cnt = 0;
-    box fixed_bb = { {0, 0}, {0, 0} };
+    box bb, fixed_bb = { {0, 0}, {0, 0} };
     point center;
 
     if (ng <= 0)
@@ -593,19 +598,20 @@ point *putGraphs(int ng, Agraph_t ** gs, Agraph_t * root,
 	Agraph_t *g = gs[i];
 	compute_bb(g);
 	if (fixed && fixed[i]) {
+	    BF2B(GD_bb(g), bb);
 	    if (fixed_cnt) {
-		box bb = GD_bb(g);
 		fixed_bb.LL.x = MIN(bb.LL.x, fixed_bb.LL.x);
 		fixed_bb.LL.y = MIN(bb.LL.y, fixed_bb.LL.y);
 		fixed_bb.UR.x = MAX(bb.UR.x, fixed_bb.UR.x);
 		fixed_bb.UR.y = MAX(bb.UR.y, fixed_bb.UR.y);
 	    } else
-		fixed_bb = GD_bb(g);
+		fixed_bb = bb;
 	    fixed_cnt++;
 	}
 	if (Verbose > 2) {
-	    fprintf(stderr, "bb[%s] %d %d %d %d\n", g->name, GD_bb(g).LL.x,
-		    GD_bb(g).LL.y, GD_bb(g).UR.x, GD_bb(g).UR.y);
+	    fprintf(stderr, "bb[%s] %.5g %.5g %.5g %.5g\n", agnameof(g),
+		GD_bb(g).LL.x, GD_bb(g).LL.y,
+		GD_bb(g).UR.x, GD_bb(g).UR.y);
 	}
     }
 
@@ -681,11 +687,11 @@ static void shiftEdge(Agedge_t * e, int dx, int dy)
     bezier bz;
 
     if (ED_label(e))
-	MOVEPT(ED_label(e)->p);
+	MOVEPT(ED_label(e)->pos);
     if (ED_head_label(e))
-	MOVEPT(ED_head_label(e)->p);
+	MOVEPT(ED_head_label(e)->pos);
     if (ED_tail_label(e))
-	MOVEPT(ED_tail_label(e)->p);
+	MOVEPT(ED_tail_label(e)->pos);
 
     if (ED_spl(e) == NULL)
 	return;
@@ -706,9 +712,10 @@ static void shiftEdge(Agedge_t * e, int dx, int dy)
 static void shiftGraph(Agraph_t * g, int dx, int dy)
 {
     graph_t *subg;
-    box bb = GD_bb(g);
+    boxf bb = GD_bb(g);
     int i;
 
+    bb = GD_bb(g);
     bb.LL.x += dx;
     bb.UR.x += dx;
     bb.LL.y += dy;
@@ -716,7 +723,7 @@ static void shiftGraph(Agraph_t * g, int dx, int dy)
     GD_bb(g) = bb;
 
     if (GD_label(g))
-	MOVEPT(GD_label(g)->p);
+	MOVEPT(GD_label(g)->pos);
 
     for (i = 1; i <= GD_n_cluster(g); i++) {
 	subg = GD_clust(g)[i];
@@ -778,7 +785,7 @@ shiftGraphs(int ng, Agraph_t ** gs, point * pp, Agraph_t * root,
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	    ND_pos(n)[0] += fx;
 	    ND_pos(n)[1] += fy;
-	    MOVEPT(ND_coord_i(n));
+	    MOVEPT(ND_coord(n));
 	    if (doSplines) {
 		for (e = agfstout(eg, n); e; e = agnxtout(eg, e))
 		    shiftEdge(e, dx, dy);
@@ -827,7 +834,7 @@ packSubgraphs(int ng, Agraph_t ** gs, Agraph_t * root, pack_info * info)
     ret = packGraphs(ng, gs, root, info);
     if (ret == 0) {
 	int i, j;
-	box bb;
+	boxf bb;
 	graph_t* g;
 
 	compute_bb(root);
