@@ -1,4 +1,4 @@
-/* $Id: inpoly.c,v 1.2 2005/04/07 22:17:37 erg Exp $ $Revision: 1.2 $ */
+/* $Id: inpoly.c,v 1.3 2009/01/02 16:49:30 ellson Exp $ $Revision: 1.3 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -22,8 +22,8 @@
  */
 
 #include <stdlib.h>
-#include <vispath.h>
-#include <pathutil.h>
+#include "vispath.h"
+#include "pathutil.h"
 
 #ifdef DMALLOC
 #include "dmalloc.h"
