@@ -1,4 +1,4 @@
-/* $Id: pathutil.h,v 1.2 2005/02/10 23:16:10 erg Exp $ $Revision: 1.2 $ */
+/* $Id: pathutil.h,v 1.4 2009/01/21 16:47:01 arif Exp $ $Revision: 1.4 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -18,7 +18,9 @@
 
 #ifndef _PATHUTIL_INCLUDE
 #define _PATHUTIL_INCLUDE
-#include <pathplan.h>
+#define _BLD_pathplan 1
+
+#include "pathplan.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -32,7 +34,14 @@ extern "C" {
 #define TRUE	(NOT(FALSE))
 #endif
 
-    typedef double COORD;
+/*visual studio*/
+#ifdef WIN32_DLL
+#ifndef PATHPLAN_EXPORTS
+#define extern __declspec(dllimport)
+#endif
+#endif
+/*end visual studio*/
+	typedef double COORD;
     extern COORD area2(Ppoint_t, Ppoint_t, Ppoint_t);
     extern int wind(Ppoint_t a, Ppoint_t b, Ppoint_t c);
     extern COORD dist2(Ppoint_t, Ppoint_t);
@@ -42,6 +51,7 @@ extern "C" {
     Ppoly_t copypoly(Ppoly_t);
     void freepoly(Ppoly_t);
 
+#undef extern
 #ifdef __cplusplus
 }
 #endif
