@@ -1,4 +1,4 @@
-/* $Id: visibility.c,v 1.2 2005/02/10 23:16:10 erg Exp $ $Revision: 1.2 $ */
+/* $Id: visibility.c,v 1.3 2009/01/02 16:49:30 ellson Exp $ $Revision: 1.3 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -15,7 +15,7 @@
 **********************************************************/
 
 
-#include <vis.h>
+#include "vis.h"
 
 #ifdef DMALLOC
 #include "dmalloc.h"
