@@ -1,4 +1,4 @@
-/* $Id: sfhdr.h,v 1.5 2008/04/23 17:16:28 ellson Exp $ $Revision: 1.5 $ */
+/* $Id: sfhdr.h,v 1.8 2009/02/05 19:21:15 arif Exp $ $Revision: 1.8 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -409,7 +409,7 @@ extern "C" {
 
 #ifndef S_ISFIFO
 #	ifdef S_IFIFO
-#		define S_ISFIFO(m)	(((m)&S_IFIFO) == S_IFIFO)
+#		define S_ISFIFO(m)	(((m)&S_IFMT) == S_IFIFO)
 #	else
 #		define S_ISFIFO(m)	(0)
 #	endif
