@@ -1,4 +1,4 @@
-/* $Id: sfio.h,v 1.3 2007/05/11 17:06:35 erg Exp $ $Revision: 1.3 $ */
+/* $Id: sfio.h,v 1.6 2009/01/19 18:18:07 arif Exp $ $Revision: 1.6 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -99,12 +99,15 @@ extern "C" {
 #endif				/* _PACKAGE_ast */
 
 /* Sfoff_t should be large enough for largest file address */
+
+
 #define Sfoff_t		_ast_intmax_t
 #define Sflong_t	_ast_intmax_t
 #define Sfulong_t	unsigned _ast_intmax_t
 #define Sfdouble_t	_ast_fltmax_t
 
-    typedef struct _sfio_s Sfio_t;
+	typedef struct _sfio_s Sfio_t;
+
     typedef struct _sfdisc_s Sfdisc_t;
     typedef ssize_t(*Sfread_f)
 	_ARG_((Sfio_t *, Void_t *, size_t, Sfdisc_t *));
@@ -277,8 +280,8 @@ extern "C" {
 
      _BEGIN_EXTERNS_ extern ssize_t _Sfi;
 
-#if !_BLD_sfio && defined(GVDLL)
-#define extern	__declspec(dllimport)
+#if _BLD_sfio && defined(GVDLL)
+#define extern	__declspec(dllexport)
 #endif
 /* standard in/out/err streams */
     extern Sfio_t *sfstdin;
