@@ -1,4 +1,4 @@
-/* $Id: GVApplicationDelegate.m,v 1.1 2008/02/11 12:34:07 glenlow Exp $ $Revision: 1.1 $ */
+/* $Id: GVApplicationDelegate.m,v 1.2 2009/02/26 12:44:55 glenlow Exp $ $Revision: 1.2 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -26,4 +26,16 @@
 	[_attributeInspectorController showWindow:sender];
 }
 
+- (BOOL)applicationOpenUntitledFile:(NSApplication*)theApplication
+{
+	if (!_applicationStarted)
+		[[NSDocumentController sharedDocumentController] openDocument:self];
+	return YES;
+}
+
+- (void)applicationDidFinishLaunching:(NSNotification*)aNotification
+{
+	_applicationStarted = YES;
+}
+
 @end
