@@ -1,4 +1,4 @@
-/* $Id: GVGraph.m,v 1.3 2008/05/26 06:09:46 glenlow Exp $ $Revision: 1.3 $ */
+/* $Id: GVGraph.m,v 1.8 2009/02/19 14:01:23 ellson Exp $ $Revision: 1.8 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -14,6 +14,7 @@
 *              AT&T Research, Florham Park NJ             *
 **********************************************************/
 
+
 #import "GVGraph.h"
 #import "GVGraphArguments.h"
 #import "GVGraphDefaultAttributes.h"
@@ -22,73 +23,6 @@ extern double PSinputscale;
 
 static GVC_t *_graphContext = nil;
 
-@interface GVGraphPluginEnumerator : NSEnumerator
-{
-	gvplugin_available_t *_plugin;
-	NSMutableSet *_distinctPlugins;
-}
-
-- (id)initWithAPI:(api_t)api;
-- (NSArray *)allObjects;
-- (id)nextObject;
-- (void)dealloc;
-
-@end
-
-@implementation GVGraphPluginEnumerator
-
-- (id)initWithAPI:(api_t)api
-{
-	if (self = [super init]) {
-		_plugin = gvFirstPlugin(_graphContext, api);
-		_distinctPlugins = [[NSMutableSet alloc] init];
-	}
-	return self;
-}
-
-- (NSArray *)allObjects
-{
-	NSMutableArray *allPlugins = [NSMutableArray array];
-	for (; _plugin; _plugin = gvNextPlugin(_plugin)) {
-		/* get the type */
-		char *pluginType = gvPluginType(_plugin);
-		NSString *nextPlugin = [[[NSString alloc] initWithBytesNoCopy:pluginType length:strlen(pluginType) encoding:NSUTF8StringEncoding freeWhenDone:NO] autorelease];
-		
-		/* if distinct, add to the list */
-		if (![_distinctPlugins containsObject:nextPlugin]) {
-			[_distinctPlugins addObject:nextPlugin];
-			[allPlugins addObject:nextPlugin];
-		}
-	}
-	return allPlugins;
-}
-
-- (id)nextObject
-{
-	NSString *nextPlugin = nil;
-	BOOL plugging = YES;
-	for (; plugging && _plugin; _plugin = gvNextPlugin(_plugin)) {
-		/* get the type */
-		char *pluginType = gvPluginType(_plugin);
-		nextPlugin = [[[NSString alloc] initWithBytesNoCopy:pluginType length:strlen(pluginType) encoding:NSUTF8StringEncoding freeWhenDone:NO] autorelease];
-		
-		/* if distinct, stop plugging away */
-		if (![_distinctPlugins containsObject:nextPlugin]) {
-			[_distinctPlugins addObject:nextPlugin];
-			plugging = NO;
-		}
-	}
-	
-	return nextPlugin;
-}
-
-- (void)dealloc
-{
-	[_distinctPlugins release];
-	[super dealloc];
-}
-@end
-
 @implementation GVGraph
 
 @synthesize graph = _graph;
@@ -97,14 +31,33 @@ static GVC_t *_graphContext = nil;
 @synthesize defaultNodeAttributes = _defaultNodeAttributes;
 @synthesize defaultEdgeAttributes = _defaultEdgeAttributes;
 
+extern char *gvplugin_list(GVC_t * gvc, api_t api, const char *str);
+
 + (void)initialize
 {
 	_graphContext = gvContext();
 }
 
-+ (NSEnumerator *)devices
++ (NSArray *)pluginsWithAPI:(api_t)api
 {
-	return [[[GVGraphPluginEnumerator alloc] initWithAPI:API_device] autorelease];
+	NSMutableSet *plugins = [NSMutableSet set];
+	
+	/* go through each non-empty plugin in the list, ignoring the package part */
+	char *pluginList = gvplugin_list(_graphContext, api, ":");
+	char *restOfPlugins;
+	char *nextPlugin;
+	for (restOfPlugins = pluginList; nextPlugin = strsep(&restOfPlugins, " ");) {
+		if (*nextPlugin) {
+			char *lastColon = strrchr(nextPlugin, ':');
+			if (lastColon) {
+				*lastColon = '\0';
+				[plugins addObject:[NSString stringWithCString:nextPlugin encoding:NSUTF8StringEncoding]];
+			}
+		}
+	}
+	free(pluginList);
+
+	return [[plugins allObjects] sortedArrayUsingSelector:@selector(compare:)];
 }
 
 - (id)initWithURL:(NSURL *)URL error:(NSError **)outError
