@@ -1,4 +1,4 @@
-/* $Id: gvrender_core_dot.c,v 1.30 2009/02/26 03:10:13 ellson Exp $ $Revision: 1.30 $ */
+/* $Id: gvrender_core_dot.c,v 1.32 2009/03/12 16:06:58 ellson Exp $ $Revision: 1.32 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -34,6 +34,7 @@
 #include "gvplugin_device.h"
 #include "agxbuf.h"
 #include "utils.h"
+#include "gvio.h"
 
 #define GNEW(t)          (t*)malloc(sizeof(t))
 
@@ -413,6 +414,11 @@ static void dot_end_graph(GVJ_t *job)
 {
     graph_t *g = job->obj->u.g;
 
+#ifndef WITH_CGRAPH
+    agsetiodisc(NULL, gvfwrite, gvferror);
+#else
+    // FIXME
+#endif
     switch (job->render.id) {
 	case FORMAT_PLAIN:
 	    write_plain(job, g, (FILE*)job, FALSE);
@@ -431,6 +437,11 @@ static void dot_end_graph(GVJ_t *job)
 		agwrite(g, (FILE*)job);
 	    break;
     }
+#ifndef WITH_CGRAPH
+    agsetiodisc(NULL, NULL, NULL);
+#else
+    // FIXME
+#endif
 }
 
 static void xdot_textpara(GVJ_t * job, pointf p, textpara_t * para)
