@@ -1,4 +1,4 @@
-/* $Id: gvrender_core_fig.c,v 1.21 2008/03/03 23:01:52 ellson Exp $ $Revision: 1.21 $ */
+/* $Id: gvrender_core_fig.c,v 1.29 2009/03/01 16:55:49 arif Exp $ $Revision: 1.29 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -32,7 +32,8 @@
 #include "const.h"
 
 #include "gvplugin_render.h"
-#include "graph.h"
+#include "gvplugin_device.h"
+#include "gvio.h"
 #include "agxbuf.h"
 #include "utils.h"
 #include "color.h"
@@ -51,13 +52,13 @@ static void figptarray(GVJ_t *job, pointf * A, int n, int close)
 
     for (i = 0; i < n; i++) {
 	PF2P(A[i],p);
-        gvdevice_printf(job, " %d %d", p.x, p.y);
+        gvprintf(job, " %d %d", p.x, p.y);
     }
     if (close) {
 	PF2P(A[0],p);
-        gvdevice_printf(job, " %d %d", p.x, p.y);
+        gvprintf(job, " %d %d", p.x, p.y);
     }
-    gvdevice_fputs(job, "\n");
+    gvputs(job, "\n");
 }
 
 static char *fig_string(char *s)
@@ -156,7 +157,7 @@ static void fig_resolve_color(GVJ_t *job, gvcolor_t * color)
 			color->u.rgba[1],
 			color->u.rgba[2]);
 	    if (new)
-		gvdevice_printf(job, "%d %d #%02x%02x%02x\n",
+		gvprintf(job, "%d %d #%02x%02x%02x\n",
 			object_code, i,
 			color->u.rgba[0],
 			color->u.rgba[1],
@@ -191,33 +192,32 @@ static void fig_line_style(obj_state_t *obj, int *line_style, double *style_val)
 
 static void fig_comment(GVJ_t *job, char *str)
 {
-    gvdevice_printf(job, "# %s\n", str);
+    gvprintf(job, "# %s\n", str);
 }
 
 static void fig_begin_graph(GVJ_t * job)
 {
     obj_state_t *obj = job->obj;
 
-    gvdevice_fputs(job, "#FIG 3.2\n");
-    gvdevice_printf(job, "# Generated by %s version %s (%s)\n",
+    gvputs(job, "#FIG 3.2\n");
+    gvprintf(job, "# Generated by %s version %s (%s)\n",
 	job->common->info[0], job->common->info[1], job->common->info[2]);
-    gvdevice_printf(job, "# For: %s\n", job->common->user);
-    gvdevice_printf(job, "# Title: %s\n", obj->u.g->name);
-    gvdevice_printf(job, "# Pages: %d\n", job->pagesArraySize.x * job->pagesArraySize.y);
-    gvdevice_fputs(job, "Portrait\n"); /* orientation */
-    gvdevice_fputs(job, "Center\n");   /* justification */
-    gvdevice_fputs(job, "Inches\n");   /* units */
-    gvdevice_fputs(job, "Letter\n");   /* papersize */
-    gvdevice_fputs(job, "100.00\n");   /* magnification % */
-    gvdevice_fputs(job, "Single\n");   /* multiple-page */
-    gvdevice_fputs(job, "-2\n");       /* transparent color (none) */
-    gvdevice_fputs(job, "1200");	     /* resolution */
-    gvdevice_fputs(job, " 2\n");       /* coordinate system (upper left) */
+    gvprintf(job, "# Title: %s\n", agnameof(obj->u.g));
+    gvprintf(job, "# Pages: %d\n", job->pagesArraySize.x * job->pagesArraySize.y);
+    gvputs(job, "Portrait\n"); /* orientation */
+    gvputs(job, "Center\n");   /* justification */
+    gvputs(job, "Inches\n");   /* units */
+    gvputs(job, "Letter\n");   /* papersize */
+    gvputs(job, "100.00\n");   /* magnification % */
+    gvputs(job, "Single\n");   /* multiple-page */
+    gvputs(job, "-2\n");       /* transparent color (none) */
+    gvputs(job, "1200");	     /* resolution */
+    gvputs(job, " 2\n");       /* coordinate system (upper left) */
 }
 
 static void fig_end_graph(GVJ_t * job)
 {
-    gvdevice_fputs(job, "# end of FIG file\n");
+    gvputs(job, "# end of FIG file\n");
 }
 
 static void fig_begin_page(GVJ_t * job)
@@ -281,7 +281,7 @@ static void fig_textpara(GVJ_t * job, pointf p, textpara_t * para)
         break;
     }
 
-    gvdevice_printf(job,
+    gvprintf(job,
             "%d %d %d %d %d %d %.1f %.4f %d %.1f %.1f %d %d %s\\001\n",
             object_code, sub_type, color, depth, pen_style, font,
             font_size, angle, font_flags, height, length, ROUND(p.x), ROUND(p.y),
@@ -316,7 +316,7 @@ static void fig_ellipse(GVJ_t * job, pointf * A, int filled)
     end_x = ROUND(A[1].x);
     end_y = ROUND(A[1].y);
 
-    gvdevice_printf(job,
+    gvprintf(job,
             "%d %d %d %d %d %d %d %d %d %.3f %d %.4f %d %d %d %d %d %d %d %d\n",
             object_code, sub_type, line_style, thickness, pen_color,
             fill_color, depth, pen_style, area_fill, style_val, direction,
@@ -345,6 +345,7 @@ static void fig_bezier(GVJ_t * job, pointf * A, int n, int arrow_at_start,
     int npoints = n;
     int i;
 
+
     pointf pf, V[4];
     point p;
     int j, step;
@@ -353,7 +354,9 @@ static void fig_bezier(GVJ_t * job, pointf * A, int n, int arrow_at_start,
 
     char *buffer;
     char *buf;
-    buffer =
+    assert (n >= 4);
+
+	buffer =
         malloc((npoints + 1) * (BEZIERSUBDIVISION +
                                 1) * 20 * sizeof(char));
     buf = buffer;
@@ -393,7 +396,7 @@ static void fig_bezier(GVJ_t * job, pointf * A, int n, int arrow_at_start,
         }
     }
 
-    gvdevice_printf(job, "%d %d %d %d %d %d %d %d %d %.1f %d %d %d %d\n",
+    gvprintf(job, "%d %d %d %d %d %d %d %d %d %.1f %d %d %d %d\n",
             object_code,
             sub_type,
             line_style,
@@ -405,12 +408,12 @@ static void fig_bezier(GVJ_t * job, pointf * A, int n, int arrow_at_start,
             area_fill,
             style_val, cap_style, forward_arrow, backward_arrow, count);
 
-    gvdevice_printf(job, " %s\n", buffer);      /* print points */
+    gvprintf(job, " %s\n", buffer);      /* print points */
     free(buffer);
     for (i = 0; i < count; i++) {
-        gvdevice_printf(job, " %d", i % (count - 1) ? 1 : 0);   /* -1 on all */
+        gvprintf(job, " %d", i % (count - 1) ? 1 : 0);   /* -1 on all */
     }
-    gvdevice_fputs(job, "\n");
+    gvputs(job, "\n");
 }
 
 static void fig_polygon(GVJ_t * job, pointf * A, int n, int filled)
@@ -436,7 +439,7 @@ static void fig_polygon(GVJ_t * job, pointf * A, int n, int filled)
 
     fig_line_style(obj, &line_style, &style_val);
 
-    gvdevice_printf(job,
+    gvprintf(job,
             "%d %d %d %d %d %d %d %d %d %.1f %d %d %d %d %d %d\n",
             object_code, sub_type, line_style, thickness, pen_color,
             fill_color, depth, pen_style, area_fill, style_val, join_style,
@@ -467,7 +470,7 @@ static void fig_polyline(GVJ_t * job, pointf * A, int n)
 
     fig_line_style(obj, &line_style, &style_val);
 
-    gvdevice_printf(job,
+    gvprintf(job,
             "%d %d %d %d %d %d %d %d %d %.1f %d %d %d %d %d %d\n",
             object_code, sub_type, line_style, thickness, pen_color,
             fill_color, depth, pen_style, area_fill, style_val, join_style,
@@ -496,6 +499,8 @@ gvrender_engine_t fig_engine = {
     fig_end_edge,
     0,				/* fig_begin_anchor */
     0,				/* fig_end_anchor */
+    0,				/* fig_begin_label */
+    0,				/* fig_end_label */
     fig_textpara,
     fig_resolve_color,
     fig_ellipse,
