@@ -1,4 +1,4 @@
-/* $Id: gvrender_core_svg.c,v 1.64 2008/04/27 21:48:34 ellson Exp $ $Revision: 1.64 $ */
+/* $Id: gvrender_core_svg.c,v 1.81 2009/02/23 15:25:21 ellson Exp $ $Revision: 1.81 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -37,9 +37,11 @@
 #include "const.h"
 
 #include "gvplugin_render.h"
+#include "gvplugin_device.h"
+#include "gvio.h"
 #include "gvcint.h"
-#include "graph.h"
-#include "types.h"		/* need the SVG font name schemes */
+
+extern char *strdup_and_subst_obj(char *str, void *obj);
 
 typedef enum { FORMAT_SVG, FORMAT_SVGZ, } format_type;
 
@@ -58,7 +60,7 @@ static void svg_bzptarray(GVJ_t * job, pointf * A, int n)
 
     c = 'M';			/* first point */
     for (i = 0; i < n; i++) {
-	gvdevice_printf(job, "%c%g,%g", c, A[i].x, -A[i].y);
+	gvprintf(job, "%c%g,%g", c, A[i].x, -A[i].y);
 	if (i == 0)
 	    c = 'C';		/* second point */
 	else
@@ -70,13 +72,13 @@ static void svg_print_color(GVJ_t * job, gvcolor_t color)
 {
     switch (color.type) {
     case COLOR_STRING:
-	gvdevice_fputs(job, color.u.string);
+	gvputs(job, color.u.string);
 	break;
     case RGBA_BYTE:
 	if (color.u.rgba[3] == 0) /* transparent */
-	    gvdevice_fputs(job, "none");
+	    gvputs(job, "none");
 	else
-	    gvdevice_printf(job, "#%02x%02x%02x",
+	    gvprintf(job, "#%02x%02x%02x",
 		color.u.rgba[0], color.u.rgba[1], color.u.rgba[2]);
 	break;
     default:
@@ -88,115 +90,97 @@ static void svg_grstyle(GVJ_t * job, int filled)
 {
     obj_state_t *obj = job->obj;
 
-    gvdevice_fputs(job, " style=\"fill:");
+    gvputs(job, " fill=\"");
     if (filled)
 	svg_print_color(job, obj->fillcolor);
     else
-	gvdevice_fputs(job, "none");
-    gvdevice_fputs(job, ";stroke:");
+	gvputs(job, "none");
+    gvputs(job, "\" stroke=\"");
     svg_print_color(job, obj->pencolor);
     if (obj->penwidth != PENWIDTH_NORMAL)
-	gvdevice_printf(job, ";stroke-width:%g", obj->penwidth);
+	gvprintf(job, "\" stroke-width=\"%g", obj->penwidth);
     if (obj->pen == PEN_DASHED) {
-	gvdevice_printf(job, ";stroke-dasharray:%s", sdasharray);
+	gvprintf(job, "\" stroke-dasharray=\"%s", sdasharray);
     } else if (obj->pen == PEN_DOTTED) {
-	gvdevice_printf(job, ";stroke-dasharray:%s", sdotarray);
+	gvprintf(job, "\" stroke-dasharray=\"%s", sdotarray);
     }
-    gvdevice_fputs(job, ";\"");
+    gvputs(job, "\"");
 }
 
 static void svg_comment(GVJ_t * job, char *str)
 {
-    gvdevice_fputs(job, "<!-- ");
-    gvdevice_fputs(job, xml_string(str));
-    gvdevice_fputs(job, " -->\n");
-}
-
-/* isAscii:
- * Return true if all characters in the string are ascii.
- */
-static int isAscii (char* s)
-{
-    int c;
-    while ((c = *s++) != '\0') {
-	if (!isascii (c)) return 0;
-    }
-    return 1;
+    gvputs(job, "<!-- ");
+    gvputs(job, xml_string(str));
+    gvputs(job, " -->\n");
 }
 
 static void svg_begin_job(GVJ_t * job)
 {
     char *s;
-    gvdevice_fputs(job, "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n");
+    gvputs(job, "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n");
     if ((s = agget(job->gvc->g, "stylesheet")) && s[0]) {
-        gvdevice_fputs(job, "<?xml-stylesheet href=\"");
-        gvdevice_fputs(job, s);
-        gvdevice_fputs(job, "\" type=\"text/css\"?>\n");
+        gvputs(job, "<?xml-stylesheet href=\"");
+        gvputs(job, s);
+        gvputs(job, "\" type=\"text/css\"?>\n");
     }
-    gvdevice_fputs(job, "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.0//EN\"\n");
-    gvdevice_fputs(job, " \"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\"");
-
+#if 0
+    gvputs(job, "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.0//EN\"\n");
+    gvputs(job, " \"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\"");
     /* This is to work around a bug in the SVG 1.0 DTD */
-    gvdevice_fputs(job, " [\n <!ATTLIST svg xmlns:xlink CDATA #FIXED \"http://www.w3.org/1999/xlink\">\n]");
+    gvputs(job, " [\n <!ATTLIST svg xmlns:xlink CDATA #FIXED \"http://www.w3.org/1999/xlink\">\n]");
+#else
+    gvputs(job, "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"\n"); 
+    gvputs(job, " \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n");
+#endif
 
-    gvdevice_fputs(job, ">\n<!-- Generated by ");
-    gvdevice_fputs(job, xml_string(job->common->info[0]));
-    gvdevice_fputs(job, " version ");
-    gvdevice_fputs(job, xml_string(job->common->info[1]));
-    gvdevice_fputs(job, " (");
-    gvdevice_fputs(job, xml_string(job->common->info[2]));
-    /* We have absolutely no idea what character set the username
-     * may be in. To be conservative, we only output the username
-     * if it is all ascii. Since SVG output is UTF-8, we could check
-     * if the string appears to be in this format and allow it.
-     */
-    if (isAscii (job->common->user)) {
-	gvdevice_fputs(job, ")\n     For user: ");
-	gvdevice_fputs(job, xml_string(job->common->user));
-    }
-    else
-	gvdevice_fputs(job, ")\n");
-    gvdevice_fputs(job, " -->\n");
+    gvputs(job, "<!-- Generated by ");
+    gvputs(job, xml_string(job->common->info[0]));
+    gvputs(job, " version ");
+    gvputs(job, xml_string(job->common->info[1]));
+    gvputs(job, " (");
+    gvputs(job, xml_string(job->common->info[2]));
+    gvputs(job, ")\n");
+    gvputs(job, " -->\n");
 }
 
 static void svg_begin_graph(GVJ_t * job)
 {
     obj_state_t *obj = job->obj;
 
-    gvdevice_fputs(job, "<!--");
-    if (obj->u.g->name[0]) {
-        gvdevice_fputs(job, " Title: ");
-	gvdevice_fputs(job, xml_string(obj->u.g->name));
+    gvputs(job, "<!--");
+    if (agnameof(obj->u.g)[0]) {
+        gvputs(job, " Title: ");
+	gvputs(job, xml_string(agnameof(obj->u.g)));
     }
-    gvdevice_printf(job, " Pages: %d -->\n", job->pagesArraySize.x * job->pagesArraySize.y);
+    gvprintf(job, " Pages: %d -->\n", job->pagesArraySize.x * job->pagesArraySize.y);
 
-    gvdevice_printf(job, "<svg width=\"%dpt\" height=\"%dpt\"\n",
+    gvprintf(job, "<svg width=\"%dpt\" height=\"%dpt\"\n",
 	job->width, job->height);
-    gvdevice_printf(job, " viewBox=\"%.2f %.2f %.2f %.2f\"",
+    gvprintf(job, " viewBox=\"%.2f %.2f %.2f %.2f\"",
         job->canvasBox.LL.x, job->canvasBox.LL.y,
         job->canvasBox.UR.x, job->canvasBox.UR.y);
     /* namespace of svg */
-    gvdevice_fputs(job, " xmlns=\"http://www.w3.org/2000/svg\"");
+    gvputs(job, " xmlns=\"http://www.w3.org/2000/svg\"");
     /* namespace of xlink */
-    gvdevice_fputs(job, " xmlns:xlink=\"http://www.w3.org/1999/xlink\"");
-    gvdevice_fputs(job, ">\n");
+    gvputs(job, " xmlns:xlink=\"http://www.w3.org/1999/xlink\"");
+    gvputs(job, ">\n");
 }
 
 static void svg_end_graph(GVJ_t * job)
 {
-    gvdevice_fputs(job, "</svg>\n");
+    gvputs(job, "</svg>\n");
 }
 
 static void svg_begin_layer(GVJ_t * job, char *layername, int layerNum, int numLayers)
 {
-    gvdevice_fputs(job, "<g id=\"");
-    gvdevice_fputs(job, xml_string(layername));
-    gvdevice_fputs(job, "\" class=\"layer\">\n");
+    gvputs(job, "<g id=\"");
+    gvputs(job, xml_string(layername));
+    gvputs(job, "\" class=\"layer\">\n");
 }
 
 static void svg_end_layer(GVJ_t * job)
 {
-    gvdevice_fputs(job, "</g>\n");
+    gvputs(job, "</g>\n");
 }
 
 static void svg_begin_page(GVJ_t * job)
@@ -205,95 +189,110 @@ static void svg_begin_page(GVJ_t * job)
 
     /* its really just a page of the graph, but its still a graph,
      * and it is the entire graph if we're not currently paging */
-    gvdevice_printf(job, "<g id=\"graph%d\" class=\"graph\"", job->common->viewNum);
-    gvdevice_printf(job, " transform=\"scale(%g %g) rotate(%d) translate(%g %g)\">\n",
+    gvputs(job, "<g id=\"");
+    gvputs(job, xml_string(obj->id));
+    gvputs(job, "\" class=\"graph\"");
+    gvprintf(job, " transform=\"scale(%g %g) rotate(%d) translate(%g %g)\">\n",
 	    job->scale.x, job->scale.y, -job->rotation,
 	    job->translation.x, -job->translation.y);
     /* default style */
-    if (obj->u.g->name[0]) {
-        gvdevice_fputs(job, "<title>");
-        gvdevice_fputs(job, xml_string(obj->u.g->name));
-        gvdevice_fputs(job, "</title>\n");
+    if (agnameof(obj->u.g)[0]) {
+        gvputs(job, "<title>");
+        gvputs(job, xml_string(agnameof(obj->u.g)));
+        gvputs(job, "</title>\n");
     }
 }
 
 static void svg_end_page(GVJ_t * job)
 {
-    gvdevice_fputs(job, "</g>\n");
+    gvputs(job, "</g>\n");
 }
 
 static void svg_begin_cluster(GVJ_t * job)
 {
     obj_state_t *obj = job->obj;
 
-    gvdevice_printf(job, "<g id=\"cluster%ld\" class=\"cluster\">",
-	    obj->u.sg->meta_node->id);
-    gvdevice_fputs(job, "<title>");
-    gvdevice_fputs(job, xml_string(obj->u.sg->name));
-    gvdevice_fputs(job, "</title>\n");
+    gvputs(job, "<g id=\"");
+    gvputs(job, xml_string(obj->id));
+    gvputs(job, "\" class=\"cluster\">");
+    gvputs(job, "<title>");
+    gvputs(job, xml_string(agnameof(obj->u.g)));
+    gvputs(job, "</title>\n");
 }
 
 static void svg_end_cluster(GVJ_t * job)
 {
-    gvdevice_fputs(job, "</g>\n");
+    gvputs(job, "</g>\n");
 }
 
 static void svg_begin_node(GVJ_t * job)
 {
     obj_state_t *obj = job->obj;
 
-    gvdevice_printf(job, "<g id=\"node%ld\" class=\"node\">", obj->u.n->id);
-    gvdevice_fputs(job, "<title>");
-    gvdevice_fputs(job, xml_string(obj->u.n->name));
-    gvdevice_fputs(job, "</title>\n");
+    gvputs(job, "<g id=\"");
+    gvputs(job, xml_string(obj->id));
+    gvputs(job, "\" class=\"node\">");
+    gvputs(job, "<title>");
+    gvputs(job, xml_string(agnameof(obj->u.n)));
+    gvputs(job, "</title>\n");
 }
 
 static void svg_end_node(GVJ_t * job)
 {
-    gvdevice_fputs(job, "</g>\n");
+    gvputs(job, "</g>\n");
 }
 
 static void
 svg_begin_edge(GVJ_t * job)
 {
     obj_state_t *obj = job->obj;
-    char *edgeop;
+    char *ename;
 
-    gvdevice_printf(job, "<g id=\"edge%ld\" class=\"edge\">", obj->u.e->id);
-    if (obj->u.e->tail->graph->root->kind & AGFLAG_DIRECTED)
-	edgeop = "&#45;&gt;";
-    else
-	edgeop = "&#45;&#45;";
-    gvdevice_fputs(job, "<title>");
-    gvdevice_fputs(job, xml_string(obj->u.e->tail->name));
-    gvdevice_fputs(job, edgeop);
-    /* can't do this in single gvdevice_printf because
-     * xml_string's buffer gets reused. */
-    gvdevice_fputs(job, xml_string(obj->u.e->head->name));
-    gvdevice_fputs(job, "</title>\n");
+    gvputs(job, "<g id=\"");
+    gvputs(job, xml_string(obj->id));
+    gvputs(job, "\" class=\"edge\">");
+    
+    gvputs(job, "<title>");
+    ename = strdup_and_subst_obj("\\E", (void*)(obj->u.e));
+    gvputs(job, xml_string(ename));
+    free(ename);
+    gvputs(job, "</title>\n");
 }
 
 static void svg_end_edge(GVJ_t * job)
 {
-    gvdevice_fputs(job, "</g>\n");
+    gvputs(job, "</g>\n");
 }
 
 static void
-svg_begin_anchor(GVJ_t * job, char *href, char *tooltip, char *target)
+svg_begin_anchor(GVJ_t * job, char *href, char *tooltip, char *target, char *id)
 {
-    gvdevice_fputs(job, "<a");
+    gvputs(job, "<a");
+#if 0
+    /* the svg spec implies this can be omitted: http://www.w3.org/TR/SVG/linking.html#Links */
+    gvputs(job, " xlink:type=\"simple\"");
+#endif
     if (href && href[0])
-	gvdevice_printf(job, " xlink:href=\"%s\"", xml_url_string(href));
+	gvprintf(job, " xlink:href=\"%s\"", xml_url_string(href));
+#if 0
+    /* linking to itself, just so that it can have a xlink:link in the anchor, seems wrong.
+     * it changes the behavior in browsers, the link apears in the bottom information bar
+     */
+    else {
+        assert (id && id[0]); /* there should always be an id available */
+	gvprintf(job, " xlink:href=\"#%s\"", xml_url_string(id));
+    }
+#endif
     if (tooltip && tooltip[0])
-	gvdevice_printf(job, " xlink:title=\"%s\"", xml_string(tooltip));
+	gvprintf(job, " xlink:title=\"%s\"", xml_string(tooltip));
     if (target && target[0])
-	gvdevice_printf(job, " target=\"%s\"", xml_string(target));
-    gvdevice_fputs(job, ">\n");
+	gvprintf(job, " target=\"%s\"", xml_string(target));
+    gvputs(job, ">\n");
 }
 
 static void svg_end_anchor(GVJ_t * job)
 {
-    gvdevice_fputs(job, "</a>\n");
+    gvputs(job, "</a>\n");
 }
 
 static void svg_textpara(GVJ_t * job, pointf p, textpara_t * para)
@@ -301,22 +300,21 @@ static void svg_textpara(GVJ_t * job, pointf p, textpara_t * para)
     obj_state_t *obj = job->obj;
     PostscriptAlias *pA;
 
-    gvdevice_fputs(job, "<text");
+    gvputs(job, "<text");
     switch (para->just) {
     case 'l':
-	gvdevice_fputs(job, " text-anchor=\"start\"");
+	gvputs(job, " text-anchor=\"start\"");
 	break;
     case 'r':
-	gvdevice_fputs(job, " text-anchor=\"end\"");
+	gvputs(job, " text-anchor=\"end\"");
 	break;
     default:
     case 'n':
-	gvdevice_fputs(job, " text-anchor=\"middle\"");
+	gvputs(job, " text-anchor=\"middle\"");
 	break;
     }
     p.y += para->yoffset_centerline;
-    gvdevice_printf(job, " x=\"%g\" y=\"%g\"", p.x, -p.y);
-    gvdevice_fputs(job, " style=\"");
+    gvprintf(job, " x=\"%g\" y=\"%g\"", p.x, -p.y);
     pA = para->postscript_alias;
     if (pA) {
 	char *family=NULL, *weight=NULL, *stretch=NULL, *style=NULL;
@@ -340,76 +338,78 @@ static void svg_textpara(GVJ_t * job, pointf p, textpara_t * para)
 	}
 	stretch = pA->stretch;
 
-        gvdevice_printf(job, "font-family:%s;", family);
-        if (weight) gvdevice_printf(job, "font-weight:%s;", weight);
-        if (stretch) gvdevice_printf(job, "font-stretch:%s;", stretch);
-        if (style) gvdevice_printf(job, "font-style:%s;", style);
+        gvprintf(job, " font-family=\"%s", family);
+	if (pA->svg_font_family) gvprintf(job, ",%s", pA->svg_font_family);
+        gvputs(job, "\"");
+        if (weight) gvprintf(job, " font-weight=\"%s\"", weight);
+        if (stretch) gvprintf(job, " font-stretch=\"%s\"", stretch);
+        if (style) gvprintf(job, " font-style=\"%s\"", style);
     }
     else
-	gvdevice_printf(job, "font-family:%s;", para->fontname);
-    gvdevice_printf(job, "font-size:%.2f;", para->fontsize);
+	gvprintf(job, " font-family=\"%s\"", para->fontname);
+    gvprintf(job, " font-size=\"%.2f\"", para->fontsize);
     switch (obj->pencolor.type) {
     case COLOR_STRING:
 	if (strcasecmp(obj->pencolor.u.string, "black"))
-	    gvdevice_printf(job, "fill:%s;", obj->pencolor.u.string);
+	    gvprintf(job, " fill=\"%s\"", obj->pencolor.u.string);
 	break;
     case RGBA_BYTE:
-	gvdevice_printf(job, "fill:#%02x%02x%02x;",
+	gvprintf(job, " fill=\"#%02x%02x%02x\"",
 		obj->pencolor.u.rgba[0], obj->pencolor.u.rgba[1], obj->pencolor.u.rgba[2]);
 	break;
     default:
 	assert(0);		/* internal error */
     }
-    gvdevice_fputs(job, "\">");
-    gvdevice_fputs(job, xml_string(para->str));
-    gvdevice_fputs(job, "</text>\n");
+    gvputs(job, ">");
+    gvputs(job, xml_string(para->str));
+    gvputs(job, "</text>\n");
 }
 
 static void svg_ellipse(GVJ_t * job, pointf * A, int filled)
 {
     /* A[] contains 2 points: the center and corner. */
-    gvdevice_fputs(job, "<ellipse");
+    gvputs(job, "<ellipse");
     svg_grstyle(job, filled);
-    gvdevice_printf(job, " cx=\"%g\" cy=\"%g\"", A[0].x, -A[0].y);
-    gvdevice_printf(job, " rx=\"%g\" ry=\"%g\"",
+    gvprintf(job, " cx=\"%g\" cy=\"%g\"", A[0].x, -A[0].y);
+    gvprintf(job, " rx=\"%g\" ry=\"%g\"",
 	    A[1].x - A[0].x, A[1].y - A[0].y);
-    gvdevice_fputs(job, "/>\n");
+    gvputs(job, "/>\n");
 }
 
 static void
 svg_bezier(GVJ_t * job, pointf * A, int n, int arrow_at_start,
 	      int arrow_at_end, int filled)
 {
-    gvdevice_fputs(job, "<path");
+    gvputs(job, "<path");
     svg_grstyle(job, filled);
-    gvdevice_fputs(job, " d=\"");
+    gvputs(job, " d=\"");
     svg_bzptarray(job, A, n);
-    gvdevice_fputs(job, "\"/>\n");
+    gvputs(job, "\"/>\n");
 }
 
 static void svg_polygon(GVJ_t * job, pointf * A, int n, int filled)
 {
     int i;
 
-    gvdevice_fputs(job, "<polygon");
+    gvputs(job, "<polygon");
     svg_grstyle(job, filled);
-    gvdevice_fputs(job, " points=\"");
+    gvputs(job, " points=\"");
     for (i = 0; i < n; i++)
-	gvdevice_printf(job, "%g,%g ", A[i].x, -A[i].y);
-    gvdevice_printf(job, "%g,%g", A[0].x, -A[0].y);	/* because Adobe SVG is broken */
-    gvdevice_fputs(job, "\"/>\n");
+	gvprintf(job, "%g,%g ", A[i].x, -A[i].y);
+    gvprintf(job, "%g,%g", A[0].x, -A[0].y);	/* because Adobe SVG is broken */
+    gvputs(job, "\"/>\n");
 }
 
 static void svg_polyline(GVJ_t * job, pointf * A, int n)
 {
     int i;
 
-    gvdevice_fputs(job, "<polyline");
+    gvputs(job, "<polyline");
     svg_grstyle(job, 0);
-    gvdevice_fputs(job, " points=\"");
+    gvputs(job, " points=\"");
     for (i = 0; i < n; i++)
-	gvdevice_printf(job, "%g,%g ", A[i].x, -A[i].y);
-    gvdevice_fputs(job, "\"/>\n");
+	gvprintf(job, "%g,%g ", A[i].x, -A[i].y);
+    gvputs(job, "\"/>\n");
 }
 
 /* color names from http://www.w3.org/TR/SVG/types.html */
@@ -478,6 +478,8 @@ gvrender_engine_t svg_engine = {
     svg_end_edge,
     svg_begin_anchor,
     svg_end_anchor,
+    0,				/* svg_begin_anchor */
+	0,				/* svg_end_anchor */
     svg_textpara,
     0,				/* svg_resolve_color */
     svg_ellipse,
