@@ -1,4 +1,4 @@
-/* $Id: gvrender_core_tk.c,v 1.18 2009/02/18 13:36:50 glenlow Exp $ $Revision: 1.18 $ */
+/* $Id: gvrender_core_tk.c,v 1.20 2009/03/12 16:06:58 ellson Exp $ $Revision: 1.20 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -14,8 +14,6 @@
 *              AT&T Research, Florham Park NJ             *
 **********************************************************/
 
-/* FIXME - incomplete replacement for codegen */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -62,7 +60,10 @@ static void tkgen_print_tags(GVJ_t *job)
     char *ObjType, *ObjPart;
     unsigned int ObjId;
     obj_state_t *obj = job->obj;
-    int ObjHandle, ObjFlag;
+    int ObjFlag;
+#ifndef WITH_CGRAPH
+    int ObjHandle;
+#endif
 
     switch (obj->emit_state) {
     case EMIT_NDRAW:
@@ -70,14 +71,18 @@ static void tkgen_print_tags(GVJ_t *job)
 	ObjPart = "shape";
 	ObjFlag = 1;
         ObjId = AGID(obj->u.n);
+#ifndef WITH_CGRAPH
 	ObjHandle = obj->u.n->handle;
+#endif
 	break;
     case EMIT_NLABEL:
 	ObjType = "node";
 	ObjPart = "label";
 	ObjFlag = 0;
         ObjId = AGID(obj->u.n);
+#ifndef WITH_CGRAPH
 	ObjHandle = obj->u.n->handle;
+#endif
 	break;
     case EMIT_EDRAW:
     case EMIT_TDRAW:
@@ -86,7 +91,9 @@ static void tkgen_print_tags(GVJ_t *job)
 	ObjPart = "shape";
 	ObjFlag = 1;
         ObjId = AGID(obj->u.e);
+#ifndef WITH_CGRAPH
 	ObjHandle = obj->u.e->handle;
+#endif
 	break;
     case EMIT_ELABEL:
     case EMIT_TLABEL:
@@ -95,14 +102,18 @@ static void tkgen_print_tags(GVJ_t *job)
 	ObjPart = "label";
 	ObjFlag = 0;
         ObjId = AGID(obj->u.e);
+#ifndef WITH_CGRAPH
 	ObjHandle = obj->u.e->handle;
+#endif
 	break;
     case EMIT_GDRAW:
 	ObjType = "graph";
 	ObjPart = "shape";
 	ObjFlag = 1;
 	ObjId = -1;  /* hack! */
+#ifndef WITH_CGRAPH
 	ObjHandle = obj->u.g->handle;
+#endif
 	break;
     case EMIT_GLABEL:
 	ObjType = "graph";
@@ -110,38 +121,40 @@ static void tkgen_print_tags(GVJ_t *job)
 	ObjFlag = 0;
 	ObjType = "graph label";
 	ObjId = -1;  /* hack! */
+#ifndef WITH_CGRAPH
 	ObjHandle = obj->u.g->handle;
+#endif
 	break;
     case EMIT_CDRAW:
-	ObjType = "cluster";
+	ObjType = "graph";
 	ObjPart = "shape";
 	ObjFlag = 1;
 #ifndef WITH_CGRAPH
 	ObjId = obj->u.sg->meta_node->id;
+	ObjHandle = obj->u.sg->handle;
 #else
 	ObjId = AGID(obj->u.sg);
 #endif
-	ObjHandle = obj->u.sg->handle;
 	break;
     case EMIT_CLABEL:
-	ObjType = "cluster";
+	ObjType = "graph";
 	ObjPart = "label";
 	ObjFlag = 0;
 #ifndef WITH_CGRAPH
 	ObjId = obj->u.sg->meta_node->id;
+	ObjHandle = obj->u.sg->handle;
 #else
 	ObjId = AGID(obj->u.sg);
 #endif
-	ObjHandle = obj->u.sg->handle;
 	break;
     default:
 	assert (0);
 	break;
     }
-#if 0
-    gvprintf(job, " -tags {id%d %s%d %s %s}", ObjId, ObjType, ObjHandle, ObjType, ObjPart);
-#else
+#ifndef WITH_CGRAPH
     gvprintf(job, " -tags {%d%s%d}", ObjFlag, ObjType, ObjHandle);
+#else
+    gvprintf(job, " -tags {%d%s%d}", ObjFlag, ObjType, ObjId);
 #endif
 }
 
