@@ -1,4 +1,4 @@
-/* $Id: gvtextlayout_gd.c,v 1.25 2008/05/14 21:28:50 erg Exp $ $Revision: 1.25 $ */
+/* $Id: gvtextlayout_gd.c,v 1.28 2008/10/30 22:06:59 ellson Exp $ $Revision: 1.28 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -23,9 +23,6 @@
 #include <string.h>
 #include "gvplugin_textlayout.h"
 
-typedef enum { AGWARN, AGERR, AGMAX, AGPREV } agerrlevel_t;
-extern int agerr(agerrlevel_t level, char *fmt, ...);
-
 #ifdef HAVE_LIBGD
 #include "gd.h"
 
@@ -100,7 +97,30 @@ char *gd_alternate_fontlist(char *font)
 }
 #endif				/* HAVE_GD_FONTCONFIG */
 
-extern char* psfontResolve (PostscriptAlias* pa);
+/* gd_psfontResolve:
+ *  * Construct alias for postscript fontname.
+ *   * NB. Uses a static array - non-reentrant.
+ *    */
+
+#define ADD_ATTR(a) \
+  if (a) { \
+        strcat(buf, comma ? " " : ", "); \
+        comma = 1; \
+        strcat(buf, a); \
+  }
+
+char* gd_psfontResolve (PostscriptAlias* pa)
+{
+    static char buf[1024];
+    int comma=0;
+    strcpy(buf, pa->family);
+
+    ADD_ATTR(pa->weight);
+    ADD_ATTR(pa->stretch);
+    ADD_ATTR(pa->style);
+   
+    return buf;
+}
 
 static boolean gd_textlayout(textpara_t * para, char **fontpath)
 {
@@ -139,8 +159,9 @@ static boolean gd_textlayout(textpara_t * para, char **fontpath)
 	}
 	/* call gdImageStringFT with null *im to get brect and to set font cache */
 #ifdef HAVE_GD_FONTCONFIG
+	gdFTUseFontConfig(1);  /* tell gd that we really want to use fontconfig, 'cos it s not the default */
 	if (para->postscript_alias)
-	    fontlist = psfontResolve (para->postscript_alias);
+	    fontlist = gd_psfontResolve (para->postscript_alias);
 	else
 	    fontlist = para->fontname;
 #else
