@@ -1,4 +1,4 @@
-/* $Id: gvplugin_gdiplus.h,v 1.1 2008/04/18 14:10:43 glenlow Exp $ $Revision: 1.1 $ */
+/* $Id: gvplugin_gdiplus.h,v 1.2 2008/07/05 06:43:09 glenlow Exp $ $Revision: 1.2 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -17,6 +17,9 @@
 #ifndef GVPLUGIN_GDIPLUS_H
 #define GVPLUGIN_GDIPLUS_H
 
+#include <Windows.h>
+#include <GdiPlus.h>
+
 typedef enum {
 	FORMAT_NONE,
 	FORMAT_BMP,
@@ -28,4 +31,16 @@ typedef enum {
 	FORMAT_TIFF
 } format_type;
 
+struct GraphicsContext
+{
+	ULONG_PTR token;
+	
+	GraphicsContext();
+	~GraphicsContext();
+};
+
+static const int BYTES_PER_PIXEL = 4;		/* bytes per pixel */
+
+extern void SaveBitmapToStream(Gdiplus::Bitmap &bitmap, IStream *stream, int format);
+
 #endif
