@@ -1,4 +1,4 @@
-/* $Id: gvdevice_gdk_pixbuf.c,v 1.12 2008/02/06 20:28:50 ellson Exp $ $Revision: 1.12 $ */
+/* $Id: gvdevice_gdk_pixbuf.c,v 1.14 2008/09/07 17:42:57 ellson Exp $ $Revision: 1.14 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -19,6 +19,7 @@
 #endif
 
 #include "gvplugin_device.h"
+#include "gvio.h"
 #ifdef HAVE_PANGOCAIRO
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
@@ -34,7 +35,7 @@ typedef enum {
  * Does an in-place conversion of a CAIRO ARGB32 image to GDK RGBA
  */
 static void
-argb2rgba ( unsigned int width, unsigned int height, unsigned char *data)
+argb2rgba ( unsigned int width, unsigned int height, char *data)
 {
 /* define indexes to color bytes in each format */
 #define Ba 0
@@ -60,12 +61,10 @@ argb2rgba ( unsigned int width, unsigned int height, unsigned char *data)
     }
 }
 
-extern size_t gvdevice_write(GVJ_t * job, const unsigned char *s, unsigned int len);
-
 static gboolean
 writer ( const gchar *buf, gsize count, GError **error, gpointer data)
 {
-    if (count == gvdevice_write((GVJ_t *)data, (const unsigned char*)buf, count))
+    if (count == gvwrite((GVJ_t *)data, buf, count))
         return TRUE;
     return FALSE;
 }
@@ -96,7 +95,7 @@ static void gdk_pixbuf_format(GVJ_t * job)
     argb2rgba(job->width, job->height, job->imagedata);
 
     pixbuf = gdk_pixbuf_new_from_data(
-                job->imagedata,         // data
+                (unsigned char*)(job->imagedata), // data
                 GDK_COLORSPACE_RGB,     // colorspace
                 TRUE,                   // has_alpha
                 8,                      // bits_per_sample
