@@ -1,4 +1,4 @@
-/* $Id: gvplugin_gdk_pixbuf.c,v 1.2 2007/08/29 19:39:49 ellson Exp $ $Revision: 1.2 $ */
+/* $Id: gvplugin_gdk_pixbuf.c,v 1.3 2008/12/21 14:26:46 ellson Exp $ $Revision: 1.3 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -16,10 +16,10 @@
 
 #include "gvplugin.h"
 
-extern gvplugin_installed_t gvdevice_gdk_pixbuf_types;
+extern gvplugin_installed_t gvdevice_gdk_pixbuf_types[];
 
 static gvplugin_api_t apis[] = {
-    {API_device, &gvdevice_gdk_pixbuf_types},
+    {API_device, gvdevice_gdk_pixbuf_types},
     {(api_t)0, 0},
 };
 
