@@ -1,4 +1,4 @@
-/* $Id: gvplugin_glitz.c,v 1.1 2008/03/11 18:55:46 ellson Exp $ $Revision: 1.1 $ */
+/* $Id: gvplugin_glitz.c,v 1.2 2008/12/21 14:26:46 ellson Exp $ $Revision: 1.2 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -16,10 +16,10 @@
 
 #include "gvplugin.h"
 
-extern gvplugin_installed_t gvdevice_types_glitz;
+extern gvplugin_installed_t gvdevice_types_glitz[];
 
 static gvplugin_api_t apis[] = {
-    {API_device, &gvdevice_types_glitz},
+    {API_device, gvdevice_types_glitz},
     {(api_t)0, 0},
 };
 
