@@ -1,4 +1,4 @@
-/* $Id: gvplugin_gtk.c,v 1.2 2006/06/28 02:49:58 ellson Exp $ $Revision: 1.2 $ */
+/* $Id: gvplugin_gtk.c,v 1.3 2008/12/21 14:26:46 ellson Exp $ $Revision: 1.3 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -16,10 +16,10 @@
 
 #include "gvplugin.h"
 
-extern gvplugin_installed_t gvdevice_types_gtk;
+extern gvplugin_installed_t gvdevice_types_gtk[];
 
 static gvplugin_api_t apis[] = {
-    {API_device, &gvdevice_types_gtk},
+    {API_device, gvdevice_types_gtk},
     {(api_t)0, 0},
 };
 
