@@ -1,4 +1,4 @@
-/* $Id: gvrender_ming.c,v 1.20 2007/10/18 17:11:41 ellson Exp $ $Revision: 1.20 $ */
+/* $Id: gvrender_ming.c,v 1.21 2009/02/18 13:36:50 glenlow Exp $ $Revision: 1.21 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -277,6 +277,8 @@ static gvrender_engine_t ming_engine = {
     0,				/* ming_end_edge */
     0,				/* ming_begin_anchor */
     0,				/* ming_end_anchor */
+    0,				/* ming_begin_label */
+    0,				/* ming_end_label */
     ming_textpara,
     0,				/* ming_resolve_color */
     ming_ellipse,
