@@ -1,4 +1,4 @@
-/* $Id: gvrender_pango.c,v 1.60 2008/05/05 18:08:54 ellson Exp $ $Revision: 1.60 $ */
+/* $Id: gvrender_pango.c,v 1.65 2009/02/18 13:36:50 glenlow Exp $ $Revision: 1.65 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -27,6 +27,7 @@
 
 #include "gvplugin_render.h"
 #include "gvplugin_device.h"
+#include "gvio.h"
 
 #ifdef HAVE_PANGOCAIRO
 #include <pango/pangocairo.h>
@@ -68,16 +69,17 @@ static void cairogen_set_color(cairo_t * cr, gvcolor_t * color)
                         color->u.RGBA[2], color->u.RGBA[3]);
 }
 
-extern size_t gvdevice_write(GVJ_t * job, const unsigned char *s, unsigned int len);
-
 static cairo_status_t
 writer (void *closure, const unsigned char *data, unsigned int length)
 {
-    if (length == gvdevice_write((GVJ_t *)closure, data, length))
+    if (length == gvwrite((GVJ_t *)closure, (const char*)data, length))
 	return CAIRO_STATUS_SUCCESS;
     return CAIRO_STATUS_WRITE_ERROR;
 }
 
+#define CAIRO_XMAX 32767
+#define CAIRO_YMAX 32767
+
 static void cairogen_begin_page(GVJ_t * job)
 {
     cairo_t *cr = NULL;
@@ -107,13 +109,25 @@ static void cairogen_begin_page(GVJ_t * job)
         case FORMAT_CAIRO:
         case FORMAT_PNG:
         default:
+	    if (job->width >= CAIRO_XMAX || job->height >= CAIRO_YMAX) {
+		double scale = MIN((double)CAIRO_XMAX / job->width,
+			(double)CAIRO_YMAX / job->height);
+		job->width *= scale;
+		job->height *= scale;
+		job->scale.x *= scale;
+		job->scale.y *= scale;
+                fprintf(stderr,
+                        "%s: graph is too large for bitmap. Scaling by %g to fit\n", 
+                        job->common->cmdname, scale);
+	    }
 	    surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
 			job->width, job->height);
             if (job->common->verbose)
                 fprintf(stderr,
-                        "%s: allocating a %dK cairo image surface\n",
+                        "%s: allocating a %dK cairo image surface (%d x %d pixels)\n",
                         job->common->cmdname,
-                        ROUND(job->width * job->height * 4 / 1024.));
+			ROUND(job->width * job->height * 4 / 1024.),
+			job->width, job->height);
 	    break;
         }
 	status = cairo_surface_status(surface);
@@ -125,17 +139,6 @@ static void cairogen_begin_page(GVJ_t * job)
 		job->context = NULL;
 		return;
 	}
-	if (job->width >= 32768 || job->height >= 32768) {
-		if (job->width >= 32768)
-		    fprintf(stderr, "%s: width (%d >= 32768) is too large.\n",
-			job->common->cmdname, job->width);
-		if (job->height >= 32768)
-		    fprintf(stderr, "%s: height (%d >= 32768) is too large.\n",
-			job->common->cmdname, job->height);
-		cairo_surface_destroy (surface);
-		job->context = NULL;
-		return;
-	}
         cr = cairo_create(surface);
         cairo_surface_destroy (surface);
         job->context = (void *) cr;
@@ -145,11 +148,8 @@ static void cairogen_begin_page(GVJ_t * job)
     cairo_rotate(cr, -job->rotation * M_PI / 180.);
     cairo_translate(cr, job->translation.x, -job->translation.y);
 
-    cairo_rectangle(cr,
-	    job->clip.LL.x,
-	    - job->clip.LL.y,
-	    job->clip.UR.x - job->clip.LL.x,
-	    - (job->clip.UR.y - job->clip.LL.y));
+    cairo_rectangle(cr, job->clip.LL.x, - job->clip.LL.y,
+	    job->clip.UR.x - job->clip.LL.x, - (job->clip.UR.y - job->clip.LL.y));
     cairo_clip(cr);
 }
 
@@ -185,7 +185,7 @@ static void cairogen_end_page(GVJ_t * job)
     case FORMAT_CAIRO:
     default:
         surface = cairo_get_target(cr);
-	job->imagedata = cairo_image_surface_get_data(surface);	
+	job->imagedata = (char *)(cairo_image_surface_get_data(surface));	
 	break;
        	/* formatting will be done by gvdevice_format() */
     }
@@ -348,6 +348,8 @@ static gvrender_engine_t cairogen_engine = {
     0,				/* cairogen_end_edge */
     0,				/* cairogen_begin_anchor */
     0,				/* cairogen_end_anchor */
+    0,				/* cairogen_begin_label */
+    0,				/* cairogen_end_label */
     cairogen_textpara,
     0,				/* cairogen_resolve_color */
     cairogen_ellipse,
