@@ -1,4 +1,4 @@
-/* $Id: gvplugin_rsvg.c,v 1.1 2007/12/28 21:27:36 ellson Exp $ $Revision: 1.1 $ */
+/* $Id: gvplugin_rsvg.c,v 1.3 2009/01/07 15:44:47 arif Exp $ $Revision: 1.3 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -16,11 +16,15 @@
 
 #include "gvplugin.h"
 
-extern gvplugin_installed_t gvloadimage_rsvg_types;
+extern gvplugin_installed_t gvloadimage_rsvg_types[];
 
 static gvplugin_api_t apis[] = {
-    {API_loadimage, &gvloadimage_rsvg_types},
+    {API_loadimage, gvloadimage_rsvg_types},
     {(api_t)0, 0},
 };
-
+#ifdef GVDLL
+__declspec(dllexport) gvplugin_library_t gvplugin_rsvg_LTX_library = { "rsvg", apis };
+#else
 gvplugin_library_t gvplugin_rsvg_LTX_library = { "rsvg", apis };
+#endif
+
