@@ -1,4 +1,4 @@
-/* $Id: gvdevice_xlib.c,v 1.22 2007/11/05 19:11:53 ellson Exp $ $Revision: 1.22 $ */
+/* $Id: gvdevice_xlib.c,v 1.28 2008/11/27 11:19:02 ellson Exp $ $Revision: 1.28 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -55,20 +55,6 @@
 #include <X11/Xutil.h>
 #include <X11/extensions/Xrender.h>
 
-/* Don't load up the xlib plugin with all that gnome stuff */
-/*   - OTOH now we are dependent on a firefox installation */
-/*                                                         */
-/* This undef must follow gvplugin_device.h because        */
-/*	 config.h is reincluded via:                       */
-/*             types.h->cdt.h->ast_common.h                */
-/*                      FIXME                              */
-#undef HAVE_GNOMEUI
-#define BROWSER "firefox"
-
-#ifdef HAVE_GNOMEUI
-#include <libgnome/libgnome.h>
-#endif
-
 typedef struct window_xlib_s {
     Window win;
     unsigned long event_mask;
@@ -88,9 +74,9 @@ static void handle_configure_notify(GVJ_t * job, XConfigureEvent * cev)
 /*	plugin/xlib/gvdevice_xlib.c */
 /*	lib/gvc/gvevent.c */
 
-    if (job->fit_mode)
-        job->zoom = MIN((double) cev->width / (double) job->width,
-			(double) cev->height / (double) job->height);
+    job->zoom *= 1 + MIN(
+	((double) cev->width - (double) job->width) / (double) job->width,
+	((double) cev->height - (double) job->height) / (double) job->height);
     if (cev->width > job->width || cev->height > job->height)
         job->has_grown = 1;
     job->width = cev->width;
@@ -164,9 +150,6 @@ static Visual *find_argb_visual(Display * dpy, int scr)
 
 static void browser_show(GVJ_t *job)
 {
-#ifdef HAVE_GNOMEUI
-   gnome_url_show(job->selected_href, NULL);
-#else
 #if defined HAVE_SYS_TYPES_H && defined HAVE_UNISTD_H && defined HAVE_ERRNO_H
    char *exec_argv[3] = {BROWSER, NULL, NULL};
    pid_t pid;
@@ -185,7 +168,6 @@ static void browser_show(GVJ_t *job)
 #else
    fprintf(stdout,"browser_show: %s\n", job->selected_href);
 #endif
-#endif
 }
 
 static int handle_xlib_events (GVJ_t *firstjob, Display *dpy)
@@ -291,12 +273,26 @@ static void init_window(GVJ_t *job, Display *dpy, int scr)
     unsigned long attributemask = 0;
     char *name;
     window_t *window;
+    int w, h;
+    double zoom_to_fit;
 
     window = (window_t *)malloc(sizeof(window_t));
     if (window == NULL) {
 	fprintf(stderr, "Failed to malloc window_t\n");
 	return;
     }
+
+    w = 480;    /* FIXME - w,h should be set by a --geometry commandline option */
+    h = 325;
+    
+    zoom_to_fit = MIN((double) w / (double) job->width,
+			(double) h / (double) job->height);
+    if (zoom_to_fit < 1.0) /* don't make bigger */
+	job->zoom *= zoom_to_fit;
+
+    job->width  = w;    /* use window geometry */
+    job->height = h;
+
     job->window = (void *)window;
     job->fit_mode = 0;
     job->needs_refresh = 1;
@@ -602,6 +598,7 @@ static gvdevice_engine_t device_engine_xlib = {
 gvplugin_installed_t gvdevice_types_xlib[] = {
 #ifdef CAIRO_HAS_XLIB_SURFACE
     {0, "xlib:cairo", 0, &device_engine_xlib, &device_features_xlib},
+    {0, "x11:cairo", 0, &device_engine_xlib, &device_features_xlib},
 #endif
     {0, NULL, 0, NULL, NULL}
 };
