@@ -2,12 +2,11 @@
 # next line is a comment in tcl \
 exec wish "$0" ${1+"$@"}
 
-package require Tkspline
 package require Tcldot
 
-# doted - dot graph editor - John Ellson (ellson@graphviz.org)
+# doted - dot/gv graph editor - John Ellson (ellson@graphviz.org)
 #
-# Usage: doted <file.dot>
+# Usage: doted <file.gv>
 #
 # doted displays the graph described in the input file and allows
 # the user to add/delete nodes/edges, to modify their attributes,
@@ -21,7 +20,7 @@ proc mouse_anyenter {c} {
 	set item [string range [lindex [$c gettags current] 0] 1 end]
 	set saveFill [list $item [lindex [$c itemconfigure 1$item -fill] 4]]
 	$c itemconfigure 1$item -fill black \
-		-stipple @$tk_library/demos/images/gray25.bmp
+		-stipple @$tk_library/demos/images/gray25.xbm
 }
 
 # as the mouse moves out of an object restore its shading
@@ -131,7 +130,7 @@ proc loadFileByNameDontAsk {c name} {
 		warning "Unable to open file: $fileName"
 	}
 	if {[catch {dotread $f} g]} {
-		warning "Invalid dot file: $fileName"
+		warning "Invalid .gv file: $fileName"
 		close $f
 	}
 	close $f
@@ -163,11 +162,12 @@ proc loadFile {c} {
 	global fileName
 
 	set types {
+		{{GV Graph Files} {.gv}}
 		{{DOT Graph Files} {.dot}}
 		{{All Files} *}
 	}
 	set fn [tk_getOpenFile \
-		-defaultextension .dot \
+		-defaultextension .gv \
 		-filetypes $types \
 		-initialfile $fileName]
 	if {[string length $fn]} {
@@ -199,7 +199,7 @@ proc saveFileByNameDontAsk {name type} {
 	if {[catch {open $name w} f]} {
 		warning "Unable to open file for write:\n$name; return"
 	}
-	if {$type == "dot"} {
+	if {$type == "gv"} {
 		set type canon
 		set fileName $name
 		set modified 0
@@ -217,13 +217,18 @@ proc saveFileAs {type} {
 	set dot {{{DOT Graph Files} {.dot}} {{All Files} *}}
 	set fig {{{FIG Image Files} {.fig}} {{All Files} *}}
 	set gif {{{GIF Image Files} {.gif}} {{All Files} *}}
+	set gv  {{{GV Graph Files} {.gv}} {{All Files} *}}
 	set hpgl {{{HPGL Image Files} {.hpgl}} {{All Files} *}}
 	set jpg {{{JPG Image Files} {.jpg}} {{All Files} *}}
 	set mif {{{MIF Image Files} {.mif}} {{All Files} *}}
 	set pcl {{{PCL Image Files} {.pcl}} {{All Files} *}}
+	set pdf {{{PDF Image Files} {.pdf}} {{All Files} *}}
 	set png {{{PNG Image Files} {.png}} {{All Files} *}}
 	set ps {{{PostScript Files} {.ps}} {{All Files} *}}
-	set svg {{{SVG Image Files} {.png}} {{All Files} *}}
+	set svg {{{SVG Image Files} {.svg}} {{All Files} *}}
+	set tiff {{{TIFF Image Files} {.tiff}} {{All Files} *}}
+	set vml {{{VML Image Files} {.vml}} {{All Files} *}}
+	set vtx {{{VTX Image Files} {.vtx}} {{All Files} *}}
 
 	set fn [tk_getSaveFile \
 		-defaultextension .$type \
@@ -599,7 +604,7 @@ button .b.layout \
 	-height [.b.h cget -width] \
 	-foreground green \
 	-activeforeground green\
-	-bitmap @$tk_library/demos/images/gray25.bmp \
+	-bitmap @$tk_library/demos/images/gray25.xbm \
 	-command "layout $c"
 
 # initialize zoom state
@@ -633,9 +638,9 @@ menu .m.file.m
 .m.file.m add command -label "New - undirected" -underline 6 \
 	-command "newGraph $c graph"
 .m.file.m add command -label "Save" -underline 0 \
-	-command "saveFile dot"
+	-command "saveFile gv"
 .m.file.m add command -label "Save As ..." -underline 5 \
-	-command "saveFileAs dot"
+	-command "saveFileAs gv"
 .m.file.m add separator
 .m.file.m add cascade -label "Export" -underline 1 \
 	-menu .m.file.m.export
@@ -652,12 +657,20 @@ menu .m.file.m.export
 	-command "saveFileAs hpgl"
 .m.file.m.export add command -label "MIF ..." -underline 0 \
 	-command "saveFileAs mif"
+.m.file.m.export add command -label "PDF ..." -underline 0 \
+	-command "saveFileAs pdf"
 .m.file.m.export add command -label "PNG ..." -underline 0 \
 	-command "saveFileAs png"
 .m.file.m.export add command -label "PS ..." -underline 0 \
 	-command "saveFileAs ps"
 .m.file.m.export add command -label "SVG ..." -underline 0 \
 	-command "saveFileAs svg"
+.m.file.m.export add command -label "TIFF ..." -underline 0 \
+	-command "saveFileAs tiff"
+.m.file.m.export add command -label "VML ..." -underline 0 \
+	-command "saveFileAs vml"
+.m.file.m.export add command -label "VTX ..." -underline 0 \
+	-command "saveFileAs vtx"
 .m.file.m add separator
 .m.file.m add command -label "Print Setup ..." -underline 0 \
 	-command "printSetup"
