@@ -1,4 +1,4 @@
-/* $Id: tclpathplan.c,v 1.1.1.1 2004/12/23 04:05:35 ellson Exp $ $Revision: 1.1.1.1 $ */
+/* $Id: tclpathplan.c,v 1.3 2008/11/08 20:29:14 ellson Exp $ $Revision: 1.3 $ */
 /* vim:set shiftwidth=4 ts=8: */
 
 /**********************************************************
@@ -54,6 +54,13 @@
 #define CONST84
 #endif
 
+#if ((TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION >= 6)) || ( TCL_MAJOR_VERSION > 8)
+#else
+#ifndef Tcl_GetStringResult
+#define Tcl_GetStringResult(interp) interp->result
+#endif
+#endif
+
 typedef Ppoint_t point;
 
 typedef struct poly_s {
@@ -186,7 +193,7 @@ static void expandPercentsEval(Tcl_Interp * interp,	/* interpreter context */
     }
     if (Tcl_GlobalEval(interp, Tcl_DStringValue(&scripts)) != TCL_OK)
 	fprintf(stderr, "%s while in binding: %s\n\n",
-		interp->result, Tcl_DStringValue(&scripts));
+		Tcl_GetStringResult(interp), Tcl_DStringValue(&scripts));
     Tcl_DStringFree(&scripts);
 }
 
